package infomat.mro;

import infomat.algorithms.grouping.*;
import infomat.algorithms.labeling.*;
import infomat.evaluation.*;
import infomat.io.*;
import infomat.io.textresult.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.weighting.*;

import mro.util.*;
import mro.util.properties.*;
import mro.util.measures.*;

import java.io.*;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.xml.sax.SAXException;



/**
 * Simple class for running clustering without the GUI.
 *
 * <br>
 * <br>
 * <code>java -cp classes/ infomat.mro.DumpMatrixStatistics</code>
 *
 * @version          
 * @author           Magnus Rosell
 *
 */
public class DumpMatrixStatistics{

    public static void dump(IMatrix matrix, String fileName){
	File file = new File(fileName);
	PrintWriter pw = new PrintWriter(new FileWriter(file));

	IObjectSet rowSet = matrix.getRowSet();
	writeSetStat(matrix, rowSet, "row");

	IObjectSet colSet = matrix.getColumnSet();
	writeSetStat(matrix, colSet, "col");

    }

    private static void writeSetStat(PrintWriter pw, IMatrix matrix, 
				     IObjectSet set, String nameStart){

	Vector<IObject> ios = set.getOrdered();

	Vector<Integer> lenVec = new Vector<Integer>();
	Vector<Integer> diffVec = new Vector<Integer>();

	for(int i=0; i<ios.size(); i++){
	    IObject io = ios.get(i);
	    Vector cells = matrix.getCellsWith(io);
	    int len = 0;
	    for(int j=0; j<cells.size(); j++){
		IMatrixCell mc = (IMatrixCell) cells.get(j);
		len += mc.getLocalCount();
	    }
	    lenVec.add(len);
	    diffVec.add(cells.size());
	}
	writeVector(pw, nameStart+"Length", lenVec);
	writeVector(pw, nameStart+"DifferentWords", diffVec);
    }



    private static void writeIVector(PrintWriter pw, 
				     String vecName, 
				     IVector<IVectorIndex> ivec){
	
	pw.print(vecName+" = [");
	Vector<IVectorIndex> vec = ivec.getOrderedVector();
	for(int i=0; i<vec.size()-1; i++){
	    IVectorIndex ivi = vec.get(i);
	    pw.print(ivi.getValue()+", ");
	}
	pw.print(vec.get(vec.size()-1).getValue());
	pw.println("]");
    }

    private static void writeVector(PrintWriter pw, 
				    String vecName, 
				    Vector<Number> vec){
	
	pw.print(vecName+" = [");
	for(int i=0; i<vec.size()-1; i++){
	    pw.print(vec.get(i)+", ");
	}
	pw.print(vec.get(vec.size()-1));
	pw.println("]");
    }



}
