package infomat.mro;

import infomat.algorithms.grouping.*;
import infomat.algorithms.labeling.*;
import infomat.evaluation.*;
import infomat.io.*;
import infomat.io.textresult.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.weighting.*;

import mro.util.*;
import mro.util.properties.*;
import mro.util.measures.*;

import java.io.*;
import java.util.Iterator;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.xml.sax.SAXException;




/****************/


/**
 * Simple class for running clustering without the GUI.
 *
 * <br>
 * <br>
 * <code>java -cp .:classes/ infomat.WTClusterer</code>
 *
 * @version          080605
 * @author           Magnus Rosell
 *
 */
public class WTClusterer{

    private static String sweStoplistFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/stoppord/Stoplist.txt";
    private static String engStoplistFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/english/porter_plus_glasgow_stoplist.txt";

    public static void main(String[] args) throws IOException, PropertyNotFoundException{

	// Load the matrix
	// This one has texts as rows and words as columns
	//	String tokenFile = "/afs/nada.kth.se/home/f/u1ya3frf/Korpus/tokenFiles/jtteLitenKorpus.corpus.tokenFile.xml";
	String tokenFile = "/afs/nada.kth.se/home/f/u1ya3frf/Korpus/tokenFiles/ATot20.randomPart1.corpus.tokenFile.xml";
	File file = new File(tokenFile);
	TokenFileTextWordIO tokenIO = new TokenFileTextWordIO();
	IMatrix textMatrix = tokenIO.load(file);

	// Stoplista
	String stoplistFileName = sweStoplistFileName;
	//	if(language==1) // engelska
	//	    stoplistFileName = engStoplistFileName;
	Stoplist sl = new Stoplist(stoplistFileName);
	boolean stopRow=false;
	sl.removeIObjectsInStoplist(textMatrix, stopRow);

	IObjectGrouping rowGrouping = 
	    textMatrix.getColumnIObjectSet().makeStartGrouping();
	IObjectGrouping colGrouping = 
	    textMatrix.getRowIObjectSet().makeStartGrouping();
	/*
	rowGrouping.sort();
	colGrouping.sort();
	*/

	// Filter the matrix
	// The example is to small

	IMatrixCellFilter matrixCellFilter = new IMatrixCellFilter();
	matrixCellFilter.filter(textMatrix);


	// Orden
	IMatrix wordMatrix = new IMatrix(textMatrix);
	

	LocationGrouper locationGrouper = new LocationGrouper();
	IObjectGrouping textCategorization = 
	    locationGrouper.cluster(textMatrix.getRowIObjectSet());
	
	// Weight the matrix
	TFIDFWeighter weighter = new TFIDFWeighter();
	weighter.weight(textMatrix);
	weighter.weight(wordMatrix);

	IObjectSet textSet = textMatrix.getRowIObjectSet();
	IObjectSet wordSet = wordMatrix.getColumnIObjectSet();

	// Similarity
	boolean row = false;
	SparseDotSimilarity wordDotSim = 
	    new SparseDotSimilarity(wordMatrix, row);

	row = true; 
	SparseDotSimilarity textDotSim = 
	    new SparseDotSimilarity(textMatrix, row);


	// Cluster
	KMeans wordKmeans = new KMeans(wordDotSim);
	wordKmeans.setProperty("Number of clusters", 20);
	IObjectGrouping wordClustering = wordKmeans.cluster(wordSet);
	wordClustering.removeEmptyGroups();


	KMeans textKmeans = new KMeans(textDotSim);
	IObjectGrouping textClustering = textKmeans.cluster(textSet);
	textClustering.removeEmptyGroups();


	// Projicera p ordklustren.
	IMatrixProjector mProj = new IMatrixProjector();
	IMatrix projTextMatrix = 
	    mProj.projectAlongColumns(textMatrix, wordDotSim, wordClustering);
	    //	    mProj.projectOnBestColumnGroups(textMatrix, wordDotSim, wordClustering);


	//	weighter.weight(projTextMatrix);
	Normalizer normalizer = new Normalizer();
	normalizer.normalize(projTextMatrix);

	IObjectSet projTextSet = projTextMatrix.getRowIObjectSet();
	row = true; 
	SparseDotSimilarity projTextDotSim = 
	    new SparseDotSimilarity(projTextMatrix, row);
	
	KMeans projTextKmeans = new KMeans(projTextDotSim);
	IObjectGrouping projTextClustering = 
	    projTextKmeans.cluster(projTextSet);
	projTextClustering.removeEmptyGroups();


	/*	
	printMatrix(textMatrix);
	System.out.println("---");
	printMatrix(projTextMatrix);
	*/

	/*
	try{
	    kmeans.setProperty("Number of clusters", 2);
	}catch(PropertyNotFoundException pnfe){
	    // Do nothing
	    System.out.println("PropertyNotFoundException!");
	}
	TopDownSplitter tds = new TopDownSplitter();
	tds.setSimilarity(dotSim);
	IObjectGrouping clustering = tds.cluster(rowSet, kmeans);
	clustering.removeEmptyGroups();
	*/


	// The current time
	long time = System.currentTimeMillis();
	
	String resultPath = 
	    "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/WordAndText/";

	// Write Text Result
	String textResultFileName = 
	    resultPath+"textClusteringResult."+time+".xml";
	File textResFile = new File(textResultFileName);
	IObjectGroupingTextResult gtio = new IObjectGroupingTextResult();
	gtio.save(textMatrix, textClustering, textDotSim, textResFile);

	String wordResultFileName = 
	    resultPath+"wordClusteringResult."+time+".xml";
	File wordResFile = new File(wordResultFileName);
	gtio.save(wordMatrix, wordClustering, wordDotSim, wordResFile);

	String projTextResultFileName = 
	    resultPath+"projTextClusteringResult."+time+".xml";
	File projTextResFile = new File(projTextResultFileName);
	gtio.save(projTextMatrix, projTextClustering, 
		  projTextDotSim, projTextResFile);




	// Evaluate and write the results
	String textEvaluationFileName = 
	    resultPath+"textClusteringEvaluation."+time+".xml";
	Evaluation textEval = 
	    new Evaluation(textMatrix, textClustering, textCategorization, textDotSim, ""+time);
	calculateAndSaveEvaluation(textEval, textEvaluationFileName);
	
	String wordEvaluationFileName = 
	    resultPath+"wordClusteringEvaluation."+time+".xml";
	Evaluation wordEval = 
	    new Evaluation(wordMatrix, wordClustering, wordDotSim, ""+time);
	calculateAndSaveEvaluation(wordEval, wordEvaluationFileName);


	String projTextEvaluationFileName = 
	    resultPath+"projTextClusteringEvaluation."+time+".xml";
	Evaluation projTextEval = 
	    new Evaluation(projTextMatrix, projTextClustering, textCategorization, projTextDotSim, ""+time);
	calculateAndSaveEvaluation(projTextEval, projTextEvaluationFileName);


	System.out.println("Clustered!!!");

    }


    private static void calculateAndSaveEvaluation(Evaluation eval, 
						   String resFileName){
	Measures measures = new Measures("Measures");
	try{
	    measures = eval.calculateAllMeasures();
	}
	catch(MeasureNameConflictException mnce){
	    System.out.println("Measure Name Conflict!");
	}
	measures.save(resFileName);
    }


    private static void printMatrix(IMatrix matrix){
	
	IObjectSet rowSet = matrix.getRowIObjectSet();
	Vector<IObject> rios = rowSet.getOrdered();
	for(int i=0; i<rios.size(); i++){
	    HashSet set = 
		matrix.getColumnIMatrixCellsForRowObject(rios.elementAt(i));
	    Iterator it = set.iterator();
	    while(it.hasNext()){
		IMatrixCell mc = (IMatrixCell) it.next();
		System.out.print(" "+mc.getLocalCount());
	    }
	    System.out.println();
	}

    }


}
