package infomat.mro.ri;

import infomat.algorithms.grouping.*;
import infomat.algorithms.labeling.*;
import infomat.bridges.jsdm.*;
import infomat.evaluation.*;
import infomat.io.*;
import infomat.io.textresult.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.weighting.*;

import mro.util.*;
import mro.util.properties.*;
import mro.util.measures.*;

import java.io.*;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.xml.sax.SAXException;


/**

* java -cp .:classes/:lib/sdm.jar -Xms1024m -Xmx1024m infomat.mro.ri.TextRIClusterer

 * javac -classpath .:classes/:lib/sdm.jar -sourcepath ./src ./src/infomat/mro/ri/*.java -d classes

 *
 * @version          081013
 * @author           Magnus Rosell
 *
 */
public class TextRIClusterer{

    public static void main(String[] args) throws IOException, PropertyNotFoundException{

	String resultPath = "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/riResult/";


	//	String filePath = "/afs/nada.kth.se/misc/tcs/language/vol/5/KTHNC/pre/jtteLitenPre/";
	//	String filePath = "/afs/nada.kth.se/misc/tcs/language/vol/5/KTHNC/pre/A1pre/";
	//	String filePath = "/afs/nada.kth.se/misc/tcs/language/vol/5/KTHNC/pre/DN20pre/";
	//	String filePath = "/afs/nada.kth.se/misc/tcs/language/infomat/tvilling/yrke/infomat/answers_lemma/";
	String filePath = "/afs/nada.kth.se/misc/tcs/language/vol/5/20newsgroups/processed/20news-bydate-test/";

	TextDirectoryReader tdr = new TextDirectoryReader();
	IMatrix matrix = tdr.load(new File(filePath));
	
	//	String stoplistFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/stoppord/snowballStop.txt";  
	String stoplistFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/english/stemmed/porter_plus_glasgow_stoplist.stemmed.txt";
	Stoplist stoplist = new Stoplist(stoplistFileName);
	boolean stopRows = false;
	stoplist.removeIObjectsInStoplist(matrix, stopRows);

	System.out.println("Stoppat");
	System.out.println("Matrix: "+matrix.getSize());
	System.out.println();
	

	// Filter the matrix
	// The example is to small
	IMatrixCellFilter matrixCellFilter = new IMatrixCellFilter();
	matrixCellFilter.filter(matrix);

	/*
			 
	Normalizer normalizer = new Normalizer();
	normalizer.normalize(matrix);

	System.out.println("Normaliserat");
	System.out.println("Matrix: "+matrix.getSize());
	System.out.println();
	*/


	// Weight the matrix
	TFIDFWeighter weighter = new TFIDFWeighter();
	weighter.weight(matrix);

	// Cluster the rows (texts) of the matrix
	IObjectSet rowSet = matrix.getRowIObjectSet();
	boolean row = true;

	// Reference
	LocationGrouper lg = new LocationGrouper();
	IObjectGrouping categorization = 
	    lg.cluster(rowSet);

	/*
	//	String groupingName = "/afs/nada.kth.se/misc/tcs/language/infomat/tvilling/yrke/infomat/AMSYK_kod4_grouping.xml";
	String groupingName = "/afs/nada.kth.se/misc/tcs/language/infomat/tvilling/yrke/infomat/YK80_kod1_grouping.xml";
	File groupingFile = new File(groupingName);
	IObjectGroupingIO iogingIO = new IObjectGroupingIO();
	IObjectGrouping categorization = 
	    iogingIO.load(row, matrix, groupingFile);
	categorization.removeObjectsNotIn(rowSet);
	System.out.println("categorization");
	System.out.println("objects: "+categorization.numberOfIObjects());
	System.out.println("groups: "+categorization.numberOfIObjectGroups());

	boolean success = rowSet.setGrouping(categorization);
	System.out.println("setGrouping: "+success);
	matrix.purgeMatrixToCurrentGroupings();
	*/

	// Number of clusters
	int numOfClusters = 100;


	// Time
	long time = 0;


	// Similarity
	//	boolean row = true; // Similarity between the rows of the matrix.
	SparseDotSimilarity dotSim = new SparseDotSimilarity(matrix, row);

	// Cluster
	/*
	KMeans alg = new KMeans(dotSim);
	alg.setProperty("Number of clusters", numOfClusters);
	*/
	BisectingKMeans alg = new BisectingKMeans(dotSim);
	alg.setProperty("Number of clusters", numOfClusters);

	IObjectGrouping clustering = alg.cluster(rowSet);
	clustering.removeEmptyGroups();

	// Evaluate and write the results
	time = System.currentTimeMillis();
	String evaluationFileName = 
	    resultPath+"clusteringEvaluation."+time+".xml";
	Evaluation eval = 
	    new Evaluation(matrix, clustering, categorization, dotSim, ""+time);
	//	eval.setProperty("Intrinsic Evaluation", false);
	calculateAndSaveEvaluation(eval, evaluationFileName);




	/******************************************/
	/******** Random Indexing ******************/
	/******************************************/

	//	String riFileName = "/afs/nada.kth.se/misc/tcs/language/vol/5/KTHNC/pre/RIjtteLitenPre";
	//	String riFileName = "/afs/nada.kth.se/misc/tcs/language/vol/5/KTHNC/pre/RIA1pre";
	//	String riFileName = "/afs/nada.kth.se/misc/tcs/language/vol/5/KTHNC/pre/RIDN20pre";
	//	String riFileName = "/afs/nada.kth.se/misc/tcs/language/infomat/tvilling/yrke/infomat/RIall";
	//	String riFileName = "/afs/nada.kth.se/misc/tcs/language/vol/5/20newsgroups/processed/RI1-20news-bydate-test";
	//	String riFileName = "/afs/nada.kth.se/misc/tcs/language/vol/5/20newsgroups/processed/RI2-20news-bydate-test";
	//	String riFileName = "/afs/nada.kth.se/misc/tcs/language/vol/5/20newsgroups/processed/RI3-20news-bydate-test";
	String riFileName = "/afs/nada.kth.se/misc/tcs/language/vol/5/20newsgroups/processed/RI4-20news-bydate-test";
	//	String riFileName = "/afs/nada.kth.se/misc/tcs/language/vol/5/20newsgroups/processed/RI10-20news-bydate-test";
	//	String riFileName = "/afs/nada.kth.se/misc/tcs/language/vol/5/20newsgroups/processed/RIB1-20news-bydate-test";
	//	String riFileName = "/afs/nada.kth.se/misc/tcs/language/vol/5/20newsgroups/processed/RIB2-20news-bydate-test";

	IObjectSet columnSet = matrix.getColumnIObjectSet();
	
	JSDMWrapper jsdmw = new JSDMWrapper();
	jsdmw.load(riFileName, columnSet);

	RICompositionalRepresentation riCRepr = 
	    new RICompositionalRepresentation(matrix, row, jsdmw);

	RICompositionalSimilarity riCSim = 
	    new RICompositionalSimilarity(riCRepr);

	/*
	KMeans riAlg = new KMeans(riCSim);
	riAlg.setProperty("Number of clusters", numOfClusters);
	*/
	BisectingKMeans riAlg = new BisectingKMeans(riCSim);
	riAlg.setProperty("Number of clusters", numOfClusters);

	IObjectGrouping riClustering = riAlg.cluster(rowSet);
	riClustering.removeEmptyGroups();



	// Evaluate and write the results
	time = System.currentTimeMillis();
	String riEvaluationFileName = 
	    resultPath+"RIclusteringEvaluation."+time+".xml";
	Evaluation riEval = 
	    new Evaluation(matrix, riClustering, categorization, riCSim, ""+time);
	//	riEval.setProperty("Intrinsic Evaluation", false);
	calculateAndSaveEvaluation(riEval, riEvaluationFileName);

	

	//	IObjectGroup iog = jsdmw.getWholeIObjectGroup();


	


	/*
	// Load a categorization
	String categorizationFileName = "example/row_grouping.xml";
	File categorizationFile = new File(categorizationFileName);
	IObjectGroupingIO iobio = new IObjectGroupingIO();
	boolean rows=true; // row categorization
	IObjectGrouping categorization = 
	    iobio.load(rows, matrix, categorizationFile);
	*/


	// Similarity
	//	JSDMSimilarity jsdmSim = new JSDMSimilarity(jsdmw);

	// Cluster
	/*
	KMeans kmeans = new KMeans(jsdmSim);
	kmeans.setProperty("Number of clusters", 10);
	IObjectGrouping clustering = kmeans.cluster(iog);
	clustering.removeEmptyGroups();
	*/

	/*
	BisectingKMeans bkm = new BisectingKMeans(jsdmSim);
	bkm.setProperty("Number of clusters", 10);
	IObjectGrouping clustering = bkm.cluster(iog);
	clustering.removeEmptyGroups();

	System.out.println("Klustrat");


	// The current time
	long time = System.currentTimeMillis();
	
	// Write Text Result
	//	String resultPath = "result/";
	String resultPath = 
	    "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/riResult/";
	
	String resultFileName = 
	    resultPath+"clusteringResult."+time+".xml";
	File resFile = new File(resultFileName);
	//	GroupingTextIO gtio = new GroupingTextIO();
	IObjectGroupingTextResult gtio = new IObjectGroupingTextResult();
	IMatrix matrix = null;
	gtio.save(matrix, clustering, jsdmSim, resFile);

	System.out.println("Sparat resultat textuellt");
	*/

	// Evaluate and write the results
	/*
	String evaluationFileName = 
	    resultPath+"clusteringEvaluation."+time+".xml";
	Evaluation eval = 
	    new Evaluation(matrix, clustering, jsdmSim, ""+time);
	calculateAndSaveEvaluation(eval, evaluationFileName);
	*/

	System.out.println("Clustered!!!");

    }


    private static void calculateAndSaveEvaluation(Evaluation eval, 
						   String resFileName){
	Measures measures = new Measures("Measures");
	try{
	    measures = eval.calculateAllMeasures();
	}
	catch(MeasureNameConflictException mnce){
	    System.out.println("Measure Name Conflict!");
	}
	measures.save(resFileName);
    }



}
