package infomat.mro.risynlexeval;

import infomat.algorithms.grouping.*;
import infomat.algorithms.projection.*;
import infomat.bridges.jsdm.*;
import infomat.evaluation.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.vector.*;
import infomat.vectorspace.weighting.*;
import infomat.mro.synlex.*;

import mro.util.doublehashmap.*;
import mro.util.measures.*;
import mro.util.properties.*;

import java.io.*;
import java.util.*;




/*

java -cp .:classes/:lib/sdm.jar -Xms1024m -Xmx1024m infomat.mro.risynlexeval.CountWordsAndPairs /afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/synlex/synpairs.xml /afs/nada.kth.se/misc/tcs/language/vol/5/KTHNC/pre/RIA1pre 


java -cp .:classes/:lib/sdm.jar -Xms3000m -Xmx3000m infomat.mro.risynlexeval.CountWordsAndPairs /afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/synlex/synpairs.xml /afs/nada.kth.se/home/f/u1ya3frf/RISYNLEX/risynlex-dok-1800-4orlessonly

java -cp .:classes/:lib/sdm.jar -Xms1500m -Xmx1500m infomat.mro.risynlexeval.CountWordsAndPairs /afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/synlex/synpairs.xml /afs/nada.kth.se/home/f/u1ya3frf/RISYNLEX/risynlex500-4 



*/



/**
 *
 * <br>
 * <br>
 * <code>javac -classpath ./classes/:lib/sdm.jar -sourcepath ./src ./src/infomat/mro/risynlexeval/*.java -d classes</code> 
 *
 * <br>
 * <br>
 * <code>java -cp .:classes/:lib/sdm.jar -Xms1024m -Xmx1024m infomat.mro.risynlexeval.CountWordsAndPairs</code>
 *
 *
 * @version          
 * @author           Magnus Rosell
 *
 */

public class CountWordsAndPairs{



    public static void main(String[] args) throws IOException{

	if(args.length!=2){
	    System.out.println("Fel input!");
	    System.exit(0);
	}
	String synlexFileName = args[0];
	String riFileName = args[1];


	/****** Synlex ******/

	File synlexFile = new File(synlexFileName);
	SynlexIO sio = new SynlexIO();
	IMatrix synlexMatrix = sio.load(synlexFile);

	System.out.println("synlex matrix: "+synlexMatrix.getNumberOfRowsColumnsCellsString());

	IObjectSet synSet = synlexMatrix.getRowIObjectSet();

	IVector<IMatrixCell> synlexIV = 
	    synlexMatrix.getIVectorOfElements();

	System.out.println("synlexIV: "+synlexIV.size());
	removeSymmetricCellsFromIVector(synlexIV);
	System.out.println("synlexIV: "+synlexIV.size());
	

	/****** RI ******/

	RIIO riio = new RIIO();

	JSDMWrapper jsdmw = new JSDMWrapper();
	jsdmw.load(riFileName, synSet);
	JSDMSimilarity jsdmSIM = new JSDMSimilarity(jsdmw);





	/********* Kontroll *********/

	HashSet<String> synlexSet = new HashSet<String>();
	Vector<IMatrixCell> vec = synlexIV.getUnorderedVector();
	for(int i=0; i<vec.size(); i++){
	    IMatrixCell mc = vec.get(i);
	    synlexSet.add(mc.getRowIObject().getString());
	    synlexSet.add(mc.getColumnIObject().getString());
	}
	System.out.println("synlex objects: "+synlexSet.size());
	
	HashSet<String> riSet = new HashSet<String>();
	Vector<IObject> vec2 = jsdmw.getFullIObjectGroup().getOrdered();
	for(int i=0; i<vec2.size(); i++){
	    riSet.add(vec2.get(i).getString());
	}
	System.out.println("ri objects: "+riSet.size());

	/****/

	HashSet<String> commonSet = new HashSet<String>();
	Iterator<String> it = synlexSet.iterator();
	while(it.hasNext()){
	    String s = it.next();
	    if(riSet.contains(s))
		commonSet.add(s);
	}
	System.out.println("common objects: "+commonSet.size());
	
	/****/
	
	Vector<IMatrixCell> remove = new Vector<IMatrixCell>();
	for(int i=0; i<vec.size(); i++){
	    IMatrixCell mc = vec.get(i);
	    if(!commonSet.contains(mc.getRowIObject().getString()) ||
	       !commonSet.contains(mc.getColumnIObject().getString()))
		remove.add(mc);
	}
	for(int i=0; i<remove.size(); i++){
	    IMatrixCell mc = remove.get(i);
	    synlexIV.remove(mc);
	}
	synlexSet = new HashSet<String>();
	vec = synlexIV.getUnorderedVector();
	for(int i=0; i<vec.size(); i++){
	    IMatrixCell mc = vec.get(i);
	    synlexSet.add(mc.getRowIObject().getString());
	    synlexSet.add(mc.getColumnIObject().getString());
	}
	System.out.println("synlex objects: "+synlexSet.size());

	/****/
	commonSet = new HashSet<String>();
	it = synlexSet.iterator();
	while(it.hasNext()){
	    String s = it.next();
	    if(riSet.contains(s))
		commonSet.add(s);
	}
	System.out.println("common objects: "+commonSet.size());

	/****/
	
	System.out.println("synlexIV: "+synlexIV.size());

	System.out.println("****************************************");


	/********* Frsta *********/

	synlexIV = synlexMatrix.getIVectorOfElements();
	removeSymmetricCellsFromIVector(synlexIV);


	IObjectGroup fullGroup = jsdmw.getFullIObjectGroup();
	System.out.println("fullGroup: "+fullGroup.numberOfIObjects());
	//	IObjectGroup wrappedGroup = jsdmw.getWrappedIObjectGroup();
	//	System.out.println("wrappedGroup: "+fullGroup.numberOfIObjects());

	matchGroupAndIVector(fullGroup, synlexIV);
	System.out.println("fullGroup reduced: "+fullGroup.numberOfIObjects());
	System.out.println("synlexIV: "+synlexIV.size());

	String notlistName = "/afs/nada.kth.se/home/f/u1ya3frf/RISYNLEX/tmp/notlist.xml";
	System.out.println(notlistName);
	FileWriter fw = new FileWriter(new File(notlistName));
	for(int i=0; i<remove.size(); i++){
	    IMatrixCell mc = remove.get(i);
	    fw.write("<syn level=\""+mc.getWeight()+"\">");
	    fw.write("<w1>"+mc.getRowIObject().getString()+"</w1>");
	    fw.write("<w2>"+mc.getColumnIObject().getString()+"</w2>");
	    fw.write("</syn>\n");
	}
	fw.close();

	String inlistName = "/afs/nada.kth.se/home/f/u1ya3frf/RISYNLEX/tmp/inlist.xml";
	System.out.println(inlistName);
	fw = new FileWriter(new File(inlistName));
	Vector<IMatrixCell> inVec = synlexIV.getUnorderedVector();
	for(int i=0; i<inVec.size(); i++){
	    IMatrixCell mc = inVec.get(i);
	    fw.write("<syn level=\""+mc.getWeight()+"\">");
	    fw.write("<w1>"+mc.getRowIObject().getString()+"</w1>");
	    fw.write("<w2>"+mc.getColumnIObject().getString()+"</w2>");
	    fw.write("</syn>\n");
	}
	fw.close();



	synlexIV = synlexMatrix.getIVectorOfElements();
	removeSymmetricCellsFromIVector(synlexIV);
	String alllistName = "/afs/nada.kth.se/home/f/u1ya3frf/RISYNLEX/tmp/alllist.xml";
	System.out.println(alllistName);
	fw = new FileWriter(new File(alllistName));
	Vector<IMatrixCell> allVec = synlexIV.getUnorderedVector();
	for(int i=0; i<allVec.size(); i++){
	    IMatrixCell mc = allVec.get(i);
	    fw.write("<syn level=\""+mc.getWeight()+"\">");
	    fw.write("<w1>"+mc.getRowIObject().getString()+"</w1>");
	    fw.write("<w2>"+mc.getColumnIObject().getString()+"</w2>");
	    fw.write("</syn>\n");
	}
	fw.close();



    }






    private static void roundCellWeights(IVector<IMatrixCell> synlexIV){
	Vector<IMatrixCell> vec = synlexIV.getUnorderedVector();
	for(int i=0; i<vec.size(); i++){
	    IMatrixCell mc = vec.get(i);
	    float w = (float) Math.round(mc.getWeight());
	    mc.setWeight(w);
	    synlexIV.put(mc, w);
	}

    }

    private static void removeIOsNotInSynlexIV(IObjectGroup fullGroup, IVector<IMatrixCell> synlexIV){
	
	HashSet<IObject> iosInSynlex = new HashSet<IObject>();
	Vector<IMatrixCell> vec = synlexIV.getUnorderedVector();
	for(int i=0; i<vec.size(); i++){
	    IMatrixCell mc = vec.get(i);
	    iosInSynlex.add(mc.getRowIObject());
	    iosInSynlex.add(mc.getColumnIObject());
	}	

	Vector<IObject> remove = new Vector<IObject>();
	Vector<IObject> vec2 = fullGroup.getOrdered();
	for(int i=0; i<vec2.size(); i++){
	    IObject io = vec2.get(i);
	    if(!iosInSynlex.contains(io))
		remove.add(io);
	}

	for(int i=0; i<remove.size(); i++){
	    IObject io = remove.get(i);
	    fullGroup.remove(io);
	}	
	
	fullGroup.sortInOrderValueOrder();
    }



    private static void matchGroupAndIVector(IObjectGroup fullGroup, IVector<IMatrixCell> synlexIV){
	removeIOsNotInSynlexIV(fullGroup, synlexIV);

	HashSet<IObject> set = new HashSet<IObject>(fullGroup.getOrdered());
	Vector<IMatrixCell> remove = new Vector<IMatrixCell>();
	Vector<IMatrixCell> vec = synlexIV.getUnorderedVector();
	for(int i=0; i<vec.size(); i++){
	    IMatrixCell mc = vec.get(i);
	    if(!set.contains(mc.getRowIObject()) ||
	       !set.contains(mc.getColumnIObject()))
		remove.add(mc);
	}

	for(int i=0; i<remove.size(); i++){
	    IMatrixCell mc = remove.get(i);
	    synlexIV.remove(mc);
	}

	removeIOsNotInSynlexIV(fullGroup, synlexIV);
    }

    private static void removeSymmetricCellsFromIVector(IVector<IMatrixCell> ivec){
	Vector<IMatrixCell> remove = new Vector<IMatrixCell>();
	GenericDoubleHashMap<IObject, IObject, IMatrixCell> map = 
	    new GenericDoubleHashMap<IObject, IObject, IMatrixCell>();
	Vector<IMatrixCell> cells = ivec.getUnorderedVector();
	for(int i=0; i<cells.size(); i++){
	    IMatrixCell mc = cells.get(i);
	    IMatrixCell omc = map.get(mc.getColumnIObject(), 
				      mc.getRowIObject());
	    if(omc!=null)
		remove.add(mc);
	    map.put(mc.getRowIObject(), mc.getColumnIObject(), mc);
	}

	for(int i=0; i<remove.size(); i++){
	    IMatrixCell mc = remove.get(i);
	    ivec.remove(mc);
	}
    }



    private static IVector<IMatrixCell> constructRIIVector(IVector<IMatrixCell> synlexIV, DenseISimilarity similarity){

	//	int count = 0;
	IVector<IMatrixCell> riVec = new IVector<IMatrixCell>();
	Vector<IMatrixCell> synVec = synlexIV.getUnorderedVector();
	for(int i=0; i<synVec.size(); i++){
	    IMatrixCell mc = synVec.get(i);
	    IObject rowObject = mc.getRowIObject();
	    IObject colObject = mc.getColumnIObject();
	    //	    System.out.println(rowObject.getString()+", "+colObject.getString()+": "+mc.getWeight());

	    /*	    
	    float sim = similarity.sim(rowObject, colObject);
	    //	    System.out.println("   -> "+sim);

	    if(sim!=0.0f)
		riVec.add(mc, sim);
	    */

	    float[] rowRepr = similarity.getRepresentation(rowObject);
	    if(rowRepr!=null){
		float[] colRepr = similarity.getRepresentation(colObject);
		if(colRepr!=null){
		    float sim = similarity.sim(rowRepr, colRepr);
		    riVec.add(mc, sim);
		}
	    }

		
	}
	
	//	System.out.println("count: "+count);

	return riVec;
    }




    private static void fitIVectors(IVector<IMatrixCell> ivec1, 
				    IVector<IMatrixCell> ivec2){
	
	//	System.out.println("ivec1: "+ivec1.size());
	//	System.out.println("ivec2: "+ivec2.size());

	fitFirstToSecondIVector(ivec1, ivec2); 
	fitFirstToSecondIVector(ivec2, ivec1); 
    }

    private static void fitFirstToSecondIVector(IVector<IMatrixCell> ivec1, 
						IVector<IMatrixCell> ivec2){
	Vector<IMatrixCell> remove1 = new Vector<IMatrixCell>();

	Vector<IMatrixCell> vec1 = ivec1.getUnorderedVector();
	for(int i = 0; i<vec1.size(); i++){
	    IMatrixCell mc = vec1.get(i);
	    Float f = ivec2.getValue(mc);
	    if(f==null)
		remove1.add(mc);
	}
	for(int i = 0; i<remove1.size(); i++){
	    IMatrixCell mc = remove1.get(i);
	    ivec1.remove(mc);
	}
    }

    private static void printIVectors(IVector<IMatrixCell> ivec1, 
				      IVector<IMatrixCell> ivec2){
	
	Vector<IMatrixCell> vec1 = ivec1.getOrderedVector();
	for(int i = 0; i<vec1.size(); i++){
	    IMatrixCell mc = vec1.get(i);
	    System.out.print(mc.getString()+": ");
	    System.out.println(ivec1.getValue(mc)+", "+ivec2.getValue(mc));
	}

    }
	
    private static void reduceSymmetricMatrix(IMatrix matrix){
	Vector<IMatrixCell> remove = new Vector<IMatrixCell>();
	Vector cells = matrix.getAllCells();
	for(int i=0; i<cells.size(); i++){
	    IMatrixCell mc = (IMatrixCell) cells.get(i);
	    IMatrixCell omc = 
		matrix.getCell(mc.getColumnIObject(), mc.getRowIObject());
	    if(omc!=null)
		remove.add(omc);
	}

	for(int i=0; i<remove.size(); i++){
	    IMatrixCell mc = remove.get(i);
	    matrix.removeCell(mc);
	}
	
    }



    private static void findUnsymmetricCellsInIVector(IVector<IMatrixCell> ivec){
	Vector<IMatrixCell> remove = new Vector<IMatrixCell>();
	GenericDoubleHashMap<IObject, IObject, IMatrixCell> map = 
	    new GenericDoubleHashMap<IObject, IObject, IMatrixCell>();

	Vector<IMatrixCell> cells = ivec.getUnorderedVector();
	Vector<IMatrixCell> strangeCells = new Vector<IMatrixCell>(cells);

	for(int i=0; i<cells.size(); i++){
	    IMatrixCell mc = cells.get(i);
	    IMatrixCell omc = map.get(mc.getColumnIObject(), 
				      mc.getRowIObject());
	    if(omc!=null){
		//		remove.add(mc);
		strangeCells.remove(mc);
		//		IMatrixCell mc2 = map.get(mc.getRowIObject(), mc.getColumnIObject());
		strangeCells.remove(omc);
	    }
	    map.put(mc.getRowIObject(), mc.getColumnIObject(), mc);
	}

	for(int i=0; i<strangeCells.size(); i++){
	    IMatrixCell mc = strangeCells.get(i);
	    System.out.println(mc.getRowIObject().getString()+", "+
			       mc.getColumnIObject().getString()+": "+
			       mc.getWeight());
	}
	
	System.out.println(strangeCells.size());
    }


    private static void findNumOfIObjectInIMatrixIVector(IVector<IMatrixCell> ivec){
	HashSet<IObject> set = new HashSet<IObject>();
	Vector<IMatrixCell> vec = ivec.getUnorderedVector();
	for(int i=0; i<vec.size(); i++){
	    IMatrixCell mc = vec.get(i);
	    set.add(mc.getRowIObject());
	    set.add(mc.getColumnIObject());
	}

	System.out.println("Number of different IObjects: "+set.size());

    }

}
