package infomat.mro.risynlexeval;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Vector;
import java.io.*;

import infomat.vectorspace.*;
import infomat.vectorspace.vector.*;

import mro.util.*;
import mro.util.measures.*;
import mro.util.properties.*;
import mro.util.doublehashmap.DoubleHashMap;


/**
 * For calculating extrinsic quality measures.
 * 
 * @version          080326
 * @author           Magnus Rosell
 */
public class ExtrinsicPairEvaluation {

    /*
    private IObjectGrouping evalGrouping;
    private IObjectGrouping refGrouping;
    private String stamp;
    */

    /*
    public ExtrinsicPairEvaluation(IObjectGrouping evalGrouping, 
				   IObjectGrouping refGrouping,
				   String stamp){
	this.evalGrouping = evalGrouping;
	this.refGrouping = refGrouping;
	this.stamp=stamp;
    }
    */

    public MeasureGroup 
	allMeasuresInAGroup(IObjectGrouping grouping,
			    IVector<IMatrixCell> correctPairs) 
	throws MeasureNameConflictException{

	System.out.println("ExtrinsicPairEvaluation.allMeasuresInAGroup()");

	/*
	MeasureGroup mg = 
	    new MeasureGroup("Pair Measures Based on Correct Pairs");

	mg.addMeasure(calculateGlobalPairMeasures(grouping, correctPairs));
	*/

	

	MeasureGroup mg = 
	    calculateGlobalPairMeasures(grouping, correctPairs);

	return mg;
    }

    private void 
	reduceGroupingAndCorrectPairs(IObjectGrouping grouping, 
				      IVector<IMatrixCell> correctPairs){

	HashSet<IObject> ioset = new HashSet<IObject>();

	// Remove correct-pair-cells with objects not in grouping
	Vector<IMatrixCell> removeCells = new Vector<IMatrixCell>();
	Vector<IMatrixCell> cells = correctPairs.getUnorderedVector();
	for(int i=0; i<cells.size(); i++){
	    IMatrixCell mc = cells.get(i);
	    IObject rio = mc.getRowIObject();
	    IObject cio = mc.getColumnIObject();

	    if(!grouping.contains(rio) ||
	       !grouping.contains(cio))
		removeCells.add(mc);
	    else{
		if(grouping.contains(rio))
		    ioset.add(rio);
		if(grouping.contains(cio))
		    ioset.add(cio);
	    }
	}
	for(int i=0; i<removeCells.size(); i++){
	    IMatrixCell mc = removeCells.get(i);
	    cells.remove(mc);
	    correctPairs.remove(mc);
	}

	// Remove objects in grouping not in correct-pair-cells
	Vector<IObject> removeIOs = new Vector<IObject>();
	Vector<IObject> ios = grouping.getOrdered();
	for(int i=0; i<ios.size(); i++){
	    IObject io = ios.get(i);
	    if(!ioset.contains(io)){
		removeIOs.add(io);
	    }
	}
	for(int i=0; i<removeIOs.size(); i++){
	    IObject io = removeIOs.get(i);
	    grouping.remove(io);
	}	
    }

    private double numOfIObjectInIVector(IVector<IMatrixCell> ivec){
	HashSet<IObject> ioset = new HashSet<IObject>();
	Vector<IMatrixCell> cells = ivec.getUnorderedVector();
	for(int i=0; i<cells.size(); i++){
	    IMatrixCell mc = cells.get(i);
	    IObject rio = mc.getRowIObject();
	    IObject cio = mc.getColumnIObject();
	    ioset.add(rio);
	    ioset.add(cio);
	}
	return ioset.size();
    }


    public MeasureGroup 
	calculateGlobalPairMeasures(IObjectGrouping evalGrouping,
				    IVector<IMatrixCell> evalCorrectPairs)
	throws MeasureNameConflictException{


	double originalNumOfGroups = evalGrouping.numberOfIObjectGroups();
	double originalNumOfGroupingIOs = evalGrouping.numberOfIObjects();
	double originalNumOfGroupingPairs = 
	    originalNumOfGroupingIOs * (originalNumOfGroupingIOs-1) / 2;
	double originalNumOfCorrectPairs = evalCorrectPairs.size();
	double originalNumOfIOsInCorrectPairs = 
	    numOfIObjectInIVector(evalCorrectPairs);

	// Reduce grouping and correct pairs to fit each other
	IObjectGrouping grouping = new IObjectGrouping(evalGrouping);
	IVector<IMatrixCell> correctPairs = 
	    new IVector<IMatrixCell>(evalCorrectPairs);
	reduceGroupingAndCorrectPairs(grouping, correctPairs);

	double numOfGroups = grouping.numberOfIObjectGroups();
	double numOfGroupingIOs = grouping.numberOfIObjects();
	double numOfGroupingPairs = 
	    numOfGroupingIOs * (numOfGroupingIOs-1) / 2;
	double numOfCorrectPairs = correctPairs.size();
	double numOfIOsInCorrectPairs = 
	    numOfIObjectInIVector(correctPairs);


	
	double tp=0; // True positives
	double tn=0; // True negatives
	double fp=0; // False positives
	double fn=0; // False negatives

	double wtp=0; // Weighted True positives
	//	double wtn=0; // Weighted True negatives
	//	double wfp=0; // Weighted False positives
	double wfn=0; // Weighted False negatives

	double numOfCorrectPairsInGrouping = 0.0; 

	HashMap<IObjectGroup, Integer> truePairsInGroup =
	    new HashMap<IObjectGroup, Integer>();

	//	HashMap<IObjectGroup, Double> trueWeightedPairsInGroup =
	//	    new HashMap<IObjectGroup, Double>();

	HashSet<IObject> ioset = new HashSet<IObject>();

	double numOfTruePairs = 0;
	double weightedNumOfTruePairs  = 0.0;

	Vector<IMatrixCell> cells = correctPairs.getUnorderedVector();
	for(int i=0; i<cells.size(); i++){
	    IMatrixCell mc = cells.get(i);
	    IObject rio = mc.getRowIObject();
	    IObject cio = mc.getColumnIObject();


	    IObjectGroup riog = grouping.inWhichIObjectGroup(rio);
	    IObjectGroup ciog = grouping.inWhichIObjectGroup(cio);
	    if(riog!=null && ciog!=null){

		numOfTruePairs++;
		weightedNumOfTruePairs += mc.getWeight();

		ioset.add(rio);
		ioset.add(cio);

		if(riog == ciog){
		    Integer n = truePairsInGroup.get(riog);
		    if(n==null)
			n = 0;
		    n++;
		    truePairsInGroup.put(riog, n);

		    wtp += mc.getWeight();

		    /*
		    Double d = trueWeightedPairsInGroup.get(riog);
		    if(d==null)
			d = 0.0;
		    d += mc.getWeight();
		    trueWeightedPairsInGroup.put(riog, d);
		    */
		}else{
		    fn++;
		    wfn += mc.getWeight();
		}
	    }
	}

	Vector<IObjectGroup> groups = grouping.getGroups();
	for(int i=0; i<groups.size(); i++){
	    IObjectGroup g = groups.get(i);
	    Integer gtp = truePairsInGroup.get(g);
	    if(gtp!=null){
		tp += gtp;
		fp += (g.numberOfIObjects() * (g.numberOfIObjects()-1))/2 - gtp;
	    }
	}

	/**********************/

	double percentageTruePairs = tp/numOfTruePairs;

	double weightedPercentageTruePairs = wtp/weightedNumOfTruePairs;


	/**********************/


	double totNumOfPairs = (ioset.size() * (ioset.size()-1))/2;
	tn = totNumOfPairs - tp - fp - fn;

	/**********************/
	
	double randStatistic = (tp+tn)/totNumOfPairs;
	double jaccardCoefficient = tp/(tp+fp+fn);
	double folkesAndMallows = Math.sqrt(tp * tp /
				     (tp + fp) /
				     (tp + fn));
	
	double precision = tp/(tp+fp);
	double recall = tp/(tp+fn);
	double beta = 0.5;
	double fmeasure = 
	    ((Math.pow(beta, 2) + 1)*precision*recall)/
	    (Math.pow(beta, 2)*precision + recall); 

	double weightedRecall = wtp/(wtp+wfn);

	/**********************/
	    
	MeasureGroup topMG = 
	    new MeasureGroup("Pair Measures Based on Correct Pairs");

	
	MeasureGroup mg = 
	    new MeasureGroup("Standard Extrinsic Pair Measures");
		
	Measure m = new Measure("Rand Statistic", randStatistic, 
				"Rand Statistic");
	mg.addMeasure(m);

	m = new Measure("Precision", precision, "Precision");
	mg.addMeasure(m);

	m = new Measure("Recall", recall, "Recall");
	mg.addMeasure(m);

	m = new Measure("F-Measure", fmeasure, "F-Measure (Beta=0.5)");
	mg.addMeasure(m);

	m = new Measure("Jaccard Coefficient", jaccardCoefficient, "Jaccard Coefficient");
	mg.addMeasure(m);

	m = new Measure("Folkes And Mallows", folkesAndMallows, "Folkes And Mallows");
	mg.addMeasure(m);

	m = new Measure("True Positives", tp, "True Positives");
	mg.addMeasure(m);

	m = new Measure("True Negatives", tn, "True Negatives");
	mg.addMeasure(m);

	m = new Measure("False Positives", fp, "False Positives");
	mg.addMeasure(m);

	m = new Measure("False Negatives", fn, "False Negatives");
	mg.addMeasure(m);

	m = new Measure("Total Evaluated Number of Objects", ioset.size(), "Total Evaluated Number of Objects");
	mg.addMeasure(m);


	m = new Measure("Total Evaluated Number of Pairs", totNumOfPairs, "Total Evluated Number of Pairs");
	mg.addMeasure(m);

	topMG.addMeasure(mg);

	/**********************/


	mg = new MeasureGroup("Weighted Extrinsic Pair Measures");
		
	m = new Measure("Weighted Recall", weightedRecall, "Weighted Recall");
	mg.addMeasure(m);

	m = new Measure("Weighted True Positives", wtp, "Weighted True Positives");
	mg.addMeasure(m);

	m = new Measure("Weighted False Negatives", wfn, "Weighted False Negatives");
	mg.addMeasure(m);
	topMG.addMeasure(mg);

	/**********************/

	/*
	mg = new MeasureGroup("Percentage Pair Measures");
		
	m = new Measure("Pair Percentage", percentageTruePairs, "Pair Percentage");
	mg.addMeasure(m);

	m = new Measure("Number of Found Correct Pairs", numOfTruePairs, "Number of Found Correct Pairs");
	mg.addMeasure(m);


	m = new Measure("Weighted Pair Percentage", weightedPercentageTruePairs, "Weighted Pair Percentage");
	mg.addMeasure(m);

	m = new Measure("Weighted Number of Found Correct Pairs", weightedNumOfTruePairs, "Weighted Number of Found Correct Pairs");
	mg.addMeasure(m);

	topMG.addMeasure(mg);
	*/
	
	/**********************/

	mg = new MeasureGroup("Grouping and Correct Pairs");
		
	m = new Measure("Grouping Number of IObjectGroups", numOfGroups, "Grouping Number of IObjectGroups");
	mg.addMeasure(m);

	m = new Measure("Grouping Number of IObjects", numOfGroupingIOs, "Grouping Number of IObjects");
	mg.addMeasure(m);

	m = new Measure("Grouping Number Of Pairs", numOfGroupingPairs, "Grouping Number Of Pairs");
	mg.addMeasure(m);

	m = new Measure("Number Of Correct Pairs", numOfCorrectPairs, "Number Of Correct Pairs");
	mg.addMeasure(m);

	m = new Measure("Number Of IObjects in Correct Pairs", numOfIOsInCorrectPairs, "Number Of IObjects in Correct Pairs");
	mg.addMeasure(m);

	topMG.addMeasure(mg);

	/**********************/

	mg = new MeasureGroup("Original Grouping and Correct Pairs");
		
	m = new Measure("Original Grouping Number of IObjectGroups", originalNumOfGroups, "Original Grouping Number of IObjectGroups");
	mg.addMeasure(m);

	m = new Measure("Original Grouping Number of IObjects", originalNumOfGroupingIOs, "Original Grouping Number of IObjects");
	mg.addMeasure(m);

	m = new Measure("Original Grouping Number Of Pairs", originalNumOfGroupingPairs, "Original Grouping Number Of Pairs");
	mg.addMeasure(m);

	m = new Measure("Original Number Of Correct Pairs", originalNumOfCorrectPairs, "Original Number Of Correct Pairs");
	mg.addMeasure(m);

	m = new Measure("Original Number Of IObjects in Correct Pairs", originalNumOfIOsInCorrectPairs, "Original Number Of IObjects in Correct Pairs");
	mg.addMeasure(m);




	topMG.addMeasure(mg);

	/**********************/


	return topMG;
    }

}
