package infomat.mro.risynlexeval;

import infomat.algorithms.grouping.*;
import infomat.algorithms.projection.*;
import infomat.bridges.jsdm.*;
import infomat.evaluation.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.vector.*;
import infomat.vectorspace.weighting.*;
import infomat.mro.synlex.*;

import mro.util.*;
import mro.util.doublehashmap.*;
import mro.util.measures.*;
import mro.util.properties.*;

import java.io.*;
import java.util.*;


/*



java -cp .:classes/:lib/sdm.jar -Xms2500m -Xmx2500m infomat.mro.risynlexeval.SynlexDistribution /afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/synlex/synpairs.xml /afs/nada.kth.se/home/f/u1ya3frf/RISYNLEX/risynlex1800-4 /afs/nada.kth.se/home/f/u1ya3frf/RISYNLEX/eval/1800-4/

java -cp .:classes/:lib/sdm.jar -Xms2500m -Xmx2500m infomat.mro.risynlexeval.SynlexDistribution /afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/synlex/synpairs.xml /afs/nada.kth.se/home/f/u1ya3frf/RISYNLEX/risynlex-dok-1800-4 /afs/nada.kth.se/home/f/u1ya3frf/RISYNLEX/eval/dok-1800-4/

***

java -cp .:classes/:lib/sdm.jar -Xms1024m -Xmx1024m infomat.mro.risynlexeval.SynlexDistribution /afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/synlex/synpairs.xml /afs/nada.kth.se/misc/tcs/language/vol/5/KTHNC/pre/RIA1pre /afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/riResult/distribution/

*/




/****************/



/**
 *
 * <br>
 * <br>
 * <code>javac -classpath ./classes/:lib/sdm.jar -sourcepath ./src ./src/infomat/mro/risynlexeval/*.java -d classes</code> 
 *
 * <br>
 * <br>
 * <code>java -cp .:classes/:lib/sdm.jar -Xms1024m -Xmx1024m infomat.mro.risynlexeval.RISynlexEval</code>
 *
 *
 * @version          
 * @author           Magnus Rosell
 *
 */

public class SynlexDistribution{

    public static void main(String[] args) throws IOException{

	if(args.length!=3){
	    System.out.println("Fel input!");
	    System.exit(0);
	}
	String synlexFileName = args[0];
	String riFileName = args[1];
	String resultPath = args[2];
	


	/****** Synlex ******/

	File synlexFile = new File(synlexFileName);
	SynlexIO sio = new SynlexIO();
	IMatrix synlexMatrix = sio.load(synlexFile);

	System.out.println("synlex matrix: "+synlexMatrix.getNumberOfRowsColumnsCellsString());

	IObjectSet synSet = synlexMatrix.getRowIObjectSet();

	IVector<IMatrixCell> synlexIV = 
	    synlexMatrix.getIVectorOfElements();

	/*
	findNumOfIObjectInIMatrixIVector(synlexIV);
	System.exit(0);
	*/

	/*
	findUnsymmetricCellsInIVector(synlexIV);
	System.exit(0);
	*/

	System.out.println("synlexIV: "+synlexIV.size());
	removeSymmetricCellsFromIVector(synlexIV);
	System.out.println("synlexIV: "+synlexIV.size());

	/****** Syntest ******/

	String syntestFileName = "/afs/nada.kth.se/misc/info/DH2418/sprakt07/ri/randomindexing/test1.dat";
	Syntest syntest = new Syntest();
	syntest.readTest(syntestFileName);
	

	/****** RI ******/

	RIIO riio = new RIIO();

	JSDMWrapper jsdmw = new JSDMWrapper();
	jsdmw.load(riFileName, synSet);
	JSDMSimilarity jsdmSIM = new JSDMSimilarity(jsdmw);


	/**** RI-Synlex-Measures ****/

	long time = System.currentTimeMillis();
	//	Measures riSynlexMeasures = new Measures("RI evaluated against The People's Dictionary of Synonyms");


	/**** Evaluation sim - synlex ****/

	IObjectGroup fullGroup = jsdmw.getFullIObjectGroup();

	System.out.println("fullGroup: "+fullGroup.numberOfIObjects());

	String groupName = "Original RI";
	String resPath = resultPath+"/distribution2/";
	evaluateAndSave(jsdmw, jsdmSIM, synlexIV, syntest, fullGroup, 
			groupName, resPath, time);




    }




    public static void 
	evaluateAndSave(JSDMWrapper jsdmw, 
			DenseISimilarity similarity, 
			IVector<IMatrixCell> synlexIV,
			Syntest syntest, 
			IObjectGroup fullGroup, 
			String groupName,
			String resultPath,
			long time){
	


	//	int repeat = 3;
	//	for(int r = 1; r<=repeat; r++){

	    Measures riSynlexMeasures = new Measures("RI evaluated against The People's Dictionary of Synonyms");
	    
	    try{
		System.out.println(groupName+" Evaluation");
		
		MeasureGroup mg = 
		    correlationEvaluation(jsdmw, similarity, synlexIV, "Pairs");//groupName);
		riSynlexMeasures.addMeasure(mg);
		
		System.out.println(groupName+" Evaluation. Done.");
	    }catch(MeasureNameConflictException mnce){
		System.out.println("MeasureNameConflictException for Original RI");
	    }
	    
	    riSynlexMeasures.setAllMeasuresStampsRecursively(""+time);
	    //	    String correlationEvaluationFileName = resultPath+"evalNR"+r+"/riCorrelationEvaluation."+time+".xml";
	    String correlationEvaluationFileName = resultPath+"synlexAndRIDistributionEvaluation."+time+".xml";
	    riSynlexMeasures.save(correlationEvaluationFileName);

	    //	}
    }








    public static MeasureGroup 
	correlationEvaluation(JSDMWrapper jsdmw, 
			      DenseISimilarity similarity, 
			      IVector<IMatrixCell> synlexIV,
			      String groupName){
	
	MeasureGroup topMG = new MeasureGroup(groupName);


	//////////////////////////////////

	System.out.println("************** 1: synlex");

	double[] synlexMins2 = new double[21];
	int index = 0;
	for(int i=30; i<51; i++){
	    double val = i/10.0;
	    synlexMins2[index] = val;
	    System.out.print(val+", ");
	    index++;
	}
	double[] synlexMaxs2 = synlexMins2;
	System.out.println();

	MeasureGroup mg = new MeasureGroup("Synlex");
	
	try{
	    Vector<IMatrixCell> svec = synlexIV.getOrderedVector();
	    int number = 1;
	    float simValue = svec.get(svec.size()-1).getWeight();
	    float previousSimValue = simValue;
	    for(int i=svec.size()-2; i>=0; i--){
		simValue = svec.get(i).getWeight();
		if(simValue == previousSimValue)
		    number++;
		else{
		    Measure m = new Measure("pairs Synlex (dict=["+previousSimValue+", "+(previousSimValue)+"])", number, "pairs RI (dict=["+previousSimValue+", "+(previousSimValue)+"])");
		    mg.addMeasure(m);

		    /*
		    float diff = simValue-previousSimValue;
		    if(diff>0.2){
			float tmpValue = previousSimValue;
			while(diff>0){
			    tmpValue += 0.1;
			    m = new Measure("pairs Synlex (dict=["+tmpValue+", "+(tmpValue)+"])", 0, "pairs Synlex (dict=["+tmpValue+", "+(tmpValue)+"])");
			    mg.addMeasure(m);

			    diff -= 0.1;
			    simValue = tmpValue;
			}
		    }
		    */

		    number=1;
		}
		previousSimValue=simValue;


	    }

	    if(number>0){
		Measure m = new Measure("pairs Synlex (dict=["+simValue+", "+(simValue)+"])", number, "pairs Synlex (dict=["+simValue+", "+(simValue)+"])");
		mg.addMeasure(m);
		previousSimValue = simValue;
	    }

	    float diff = 5.0f-previousSimValue;
	    if(diff>0.1){
		float tmpValue = previousSimValue;
		while(diff>0){
		    tmpValue += 0.1;
		    Measure m = new Measure("pairs Synlex (dict=["+tmpValue+", "+(tmpValue)+"])", 0, "pairs Synlex (dict=["+tmpValue+", "+(tmpValue)+"])");
		    mg.addMeasure(m);
		    
		    diff -= 0.1;
		    
		}
	    }


	    topMG.addMeasure(mg);
	}catch(MeasureNameConflictException mnce){
	    System.out.println("MeasureNameConflictException: Synlex");
	    mnce.printStackTrace();
	    System.exit(0);
	}

	//////////////////////////////////

	System.out.println("************** 2 - ri");

	double[] simMins = new double[21];
	index = 0;
	for(int i=-10; i<11; i++){
	    double val = i/10.0;
	    simMins[index] = val;
	    System.out.print(val+", ");
	    index++;
	}
	double[] simMaxs = simMins;
	System.out.println();

	mg = new MeasureGroup("RI");
	
	HashMap<String, Integer> simNumMap = new HashMap<String, Integer>();

	IObjectGroup fullGroup = jsdmw.getFullIObjectGroup();
	Vector<IObject> vec = fullGroup.getOrdered();
	Collections.shuffle(vec);
	vec = new Vector<IObject>(vec.subList(0, 5000));
	int number = vec.size()*vec.size();
	for(int i=vec.size()-1; i>=0; i--){
	    IObject o1 = vec.get(i);
	    for(int j=vec.size()-1; j>=0; j--){

		if(i==j)
		    continue;

		IObject o2 = vec.get(j);
		float simValue = similarity.sim(o1, o2);
		
		for(int k=0; k<simMins.length-1; k++){
		    if(simValue>simMins[k] && simValue<=simMins[k+1]){
			String sim = Round.roundDecimalsToTrimmedString(simMins[k], 1);
			Integer num = simNumMap.get(sim);
			if(num==null)
			    num=0;
			num++;
			simNumMap.put(sim, num);
			break;
		    }
		}

		number--;
		int step = 10000;
		if(number % step == 0)
		    System.out.print((number / step)+", "); //+":"+sim+", ");
		

	    }
	}
	System.out.println();

	try{
	    for(int i=0; i<simMins.length-1; i++){
		String sim = 
		    Round.roundDecimalsToTrimmedString(simMins[i], 1);
		String upper = 
		    Round.roundDecimalsToTrimmedString(simMins[i+1], 1);
		//		String sim = ""+simMins[i];
		Integer num = simNumMap.get(sim);
		

		Measure m = new Measure("pairs RI (RI=["+sim+", "+upper+"])", 0, "pairs RI (RI=["+0+", "+upper+"])");
		if(num!=null){
		    num /= 2;
		    m = new Measure("pairs RI (RI=["+sim+", "+upper+"])", num, "pairs RI (RI=["+sim+", "+upper+"])");
		}
		mg.addMeasure(m);
	    }
	    topMG.addMeasure(mg);
	}catch(MeasureNameConflictException mnce){
	    System.out.println("MeasureNameConflictException for Original RI");
	}
	


	/*
	//////////////////////////////////

	System.out.println("************** 4: common synlex");


	double[] simMins2 = {-1.0};
	double[] simMaxs2 = {1.0};

	mg = evaluator(similarity, synlexIV, "Common Synlex", simMins2, simMaxs2, synlexMins2, synlexMaxs2);

	try{
	    topMG.addMeasure(mg);
	}catch(MeasureNameConflictException mnce){
	    System.out.println("MeasureNameConflictException for Original RI");
	}

	*/

	//////////////////////////////////
	System.out.println("************** 3: common RI");


	double[] synlexMins = {3.0, 4.0, 5.0};
	double[] synlexMaxs = {3.0, 4.0, 5.0};

	mg = evaluator(similarity, synlexIV, "Common RI", simMins, simMaxs, synlexMins, synlexMaxs);

	try{
	    topMG.addMeasure(mg);
	}catch(MeasureNameConflictException mnce){
	    System.out.println("MeasureNameConflictException for Original RI");
	}

	
	//////////////////////////////////

	System.out.println("************** 4: common synlex");

	System.out.println("common cells: "+commonCells.size());

	

	double simMin = -1.0;
	double simMax = 1.0;

	mg = new MeasureGroup("Common Synlex");
	
	try{
	    Vector<IMatrixCell> svec = commonCells.getUnorderedVector();
	    for(int i=0; i<svec.size(); i++){
		IMatrixCell mc = svec.get(i);
		commonCells.put(mc, mc.getWeight());
	    }

	    svec = commonCells.getOrderedVector();
	    number = 1;
	    float simValue = svec.get(svec.size()-1).getWeight();
	    float previousSimValue = simValue;
	    for(int i=svec.size()-2; i>=0; i--){
		simValue = svec.get(i).getWeight();
		if(simValue == previousSimValue)
		    number++;
		else{
		    Measure m = new Measure("pairs Synlex (RI=["+simMin+", "+simMax+"], dict=["+previousSimValue+", "+previousSimValue+"])", number, "pairs Synlex (RI=["+simMin+", "+simMax+"], dict=["+previousSimValue+", "+previousSimValue+"])");
		    mg.addMeasure(m);

		    /*
		    float diff = simValue-previousSimValue;
		    if(diff>0.2){
			float tmpValue = previousSimValue;
			while(diff>0){
			    tmpValue += 0.1;
			    m = new Measure("pairs Synlex (dict=["+tmpValue+", "+(tmpValue)+"])", 0, "pairs Synlex (dict=["+tmpValue+", "+(tmpValue)+"])");
			    mg.addMeasure(m);

			    diff -= 0.1;
			    simValue = tmpValue;
			}
		    }
		    */

		    number=1;
		}
		previousSimValue=simValue;


	    }

	    if(number>0){
		Measure m = new Measure("pairs Synlex (RI=["+simMin+", "+simMax+"], dict=["+previousSimValue+", "+previousSimValue+"])", number, "pairs Synlex (RI=["+simMin+", "+simMax+"], dict=["+previousSimValue+", "+previousSimValue+"])");
		mg.addMeasure(m);
		previousSimValue = simValue;
	    }

	    float diff = 5.0f-previousSimValue;
	    if(diff>0.1){
		float tmpValue = previousSimValue;
		while(diff>0){
		    tmpValue += 0.1;

		    Measure m = new Measure("pairs Synlex (RI=["+simMin+", "+simMax+"], dict=["+previousSimValue+", "+previousSimValue+"])", 0, "pairs Synlex (RI=["+simMin+", "+simMax+"], dict=["+previousSimValue+", "+previousSimValue+"])");
		    mg.addMeasure(m);
		    
		    diff -= 0.1;
		    
		}
	    }


	    topMG.addMeasure(mg);
	}catch(MeasureNameConflictException mnce){
	    System.out.println("MeasureNameConflictException: Synlex");
	    mnce.printStackTrace();
	    System.exit(0);
	}



	return topMG;
    }
    
    private static IVector<IMatrixCell> commonCells = new IVector<IMatrixCell>();

    private static MeasureGroup evaluator(DenseISimilarity similarity, 
					  IVector<IMatrixCell> synlexIV,
					  String groupName, 
					  double[] simMins, double[] simMaxs, 
					  double[] synlexMins, double[] synlexMaxs){


	// Gr intervall
	// Synlex: [x, y]
	// RI: (x, y]


	MeasureGroup mg = new MeasureGroup(groupName);


	int number = simMins.length * simMaxs.length * 
	    synlexMins.length * synlexMaxs.length;

	for(int i=0; i<simMins.length; i++){
	    double simMin = simMins[i];
	    for(int j=0; j<simMaxs.length; j++){
		double simMax = simMaxs[j];

		//		System.out.print(number +", ");
		if(simMin>=simMax){
		    number -= synlexMins.length*synlexMaxs.length;
		    continue;
		}

		for(int k=0; k<synlexMins.length; k++){
		    double synlexMin = synlexMins[k];
		    for(int l=0; l<synlexMaxs.length; l++){
			
			number--;

			if(number % 10 == 0)
			    System.out.print(number +", ");
			    

			double synlexMax = synlexMaxs[l];
			if(synlexMin>synlexMax)
			    continue;

			/*
			double corr = simSynlexCorrelation(similarity, 
							   simMin,
							   simMax, 
							   synlexIV,
							   synlexMin,
							   synlexMax);
			*/

			double commonPairs = 0.0;
			double synlexPairs = 0.0;

			IVector<IMatrixCell> riVec = 
			    new IVector<IMatrixCell>();
			IVector<IMatrixCell> tmpSynlexIV = 
			    synlexIV.largerThanOrEquals(synlexMin);
			if(tmpSynlexIV.size()>0){
			    tmpSynlexIV = 
				tmpSynlexIV.smallerThanOrEquals(synlexMax);

			    synlexPairs = tmpSynlexIV.size();

			    if(tmpSynlexIV.size()>0){
				riVec = constructRIIVector(tmpSynlexIV, similarity);
				//				System.out.println("riVec: "+riVec.size());



				if(riVec.size()>0){
				    riVec = riVec.largerThan(simMin);
				    //				    System.out.println("riVec: "+riVec.size());
				    if(riVec.size()>0){
					riVec = riVec.smallerThanOrEquals(simMax);
					//					System.out.println("riVec: "+riVec.size());
					if(riVec.size()>0){
					    fitIVectors(tmpSynlexIV, riVec);
					    if(riVec.size()>0){
						//						System.out.println("riVec: "+riVec.size());
						commonPairs = riVec.size();
						commonCells.put(riVec);

					    }
					}
				    }
				}
			    }
			}

			

			try{
			    
			    if(groupName.equals("Common RI")){
				Measure m = new Measure("pairs RI (RI=["+simMin+", "+simMax+"], dict=["+synlexMin+", "+synlexMax+"])", commonPairs, "syn pairs (RI=["+simMin+", "+simMax+"], dict=["+synlexMin+", "+synlexMax+"])");
				mg.addMeasure(m);
			    }

			    if(groupName.equals("Common Synlex")){
				Measure m = new Measure("pairs Synlex (RI=["+simMin+", "+simMax+"], dict=["+synlexMin+", "+synlexMax+"])", commonPairs, "syn pairs (RI=["+simMin+", "+simMax+"], dict=["+synlexMin+", "+synlexMax+"])");

				mg.addMeasure(m);
			    }


			}catch(MeasureNameConflictException mnce){
			    System.out.println("MeasureNameConflictException for "+"corrcoeff (RI=["+simMin+", "+simMax+"], dict=["+synlexMin+", "+synlexMax+"])");
			}

		    }
		}
	    }
	}

	System.out.println();
	
	return mg;

    }





    private static IVector<IMatrixCell> constructRIIVector(IVector<IMatrixCell> synlexIV, DenseISimilarity similarity){

	//	int count = 0;
	IVector<IMatrixCell> riVec = new IVector<IMatrixCell>();
	Vector<IMatrixCell> synVec = synlexIV.getUnorderedVector();
	for(int i=0; i<synVec.size(); i++){
	    IMatrixCell mc = synVec.get(i);
	    IObject rowObject = mc.getRowIObject();
	    IObject colObject = mc.getColumnIObject();
	    //	    System.out.println(rowObject.getString()+", "+colObject.getString()+": "+mc.getWeight());

	    /*	    
	    float sim = similarity.sim(rowObject, colObject);
	    //	    System.out.println("   -> "+sim);

	    if(sim!=0.0f)
		riVec.add(mc, sim);
	    */

	    float[] rowRepr = similarity.getRepresentation(rowObject);
	    if(rowRepr!=null){
		float[] colRepr = similarity.getRepresentation(colObject);
		if(colRepr!=null){
		    float sim = similarity.sim(rowRepr, colRepr);
		    riVec.add(mc, sim);
		}
	    }

		
	}
	
	//	System.out.println("count: "+count);

	return riVec;
    }




    private static void fitIVectors(IVector<IMatrixCell> ivec1, 
				    IVector<IMatrixCell> ivec2){
	
	//	System.out.println("ivec1: "+ivec1.size());
	//	System.out.println("ivec2: "+ivec2.size());

	fitFirstToSecondIVector(ivec1, ivec2); 
	fitFirstToSecondIVector(ivec2, ivec1); 
    }

    private static void fitFirstToSecondIVector(IVector<IMatrixCell> ivec1, 
						IVector<IMatrixCell> ivec2){
	Vector<IMatrixCell> remove1 = new Vector<IMatrixCell>();

	Vector<IMatrixCell> vec1 = ivec1.getUnorderedVector();
	for(int i = 0; i<vec1.size(); i++){
	    IMatrixCell mc = vec1.get(i);
	    Float f = ivec2.getValue(mc);
	    if(f==null)
		remove1.add(mc);
	}
	for(int i = 0; i<remove1.size(); i++){
	    IMatrixCell mc = remove1.get(i);
	    ivec1.remove(mc);
	}
    }

    private static void printIVectors(IVector<IMatrixCell> ivec1, 
				      IVector<IMatrixCell> ivec2){
	
	Vector<IMatrixCell> vec1 = ivec1.getOrderedVector();
	for(int i = 0; i<vec1.size(); i++){
	    IMatrixCell mc = vec1.get(i);
	    System.out.print(mc.getString()+": ");
	    System.out.println(ivec1.getValue(mc)+", "+ivec2.getValue(mc));
	}

    }
	
    private static void reduceSymmetricMatrix(IMatrix matrix){
	Vector<IMatrixCell> remove = new Vector<IMatrixCell>();
	Vector cells = matrix.getAllCells();
	for(int i=0; i<cells.size(); i++){
	    IMatrixCell mc = (IMatrixCell) cells.get(i);
	    IMatrixCell omc = 
		matrix.getCell(mc.getColumnIObject(), mc.getRowIObject());
	    if(omc!=null)
		remove.add(omc);
	}

	for(int i=0; i<remove.size(); i++){
	    IMatrixCell mc = remove.get(i);
	    matrix.removeCell(mc);
	}
	
    }

    private static void removeSymmetricCellsFromIVector(IVector<IMatrixCell> ivec){
	Vector<IMatrixCell> remove = new Vector<IMatrixCell>();
	GenericDoubleHashMap<IObject, IObject, IMatrixCell> map = 
	    new GenericDoubleHashMap<IObject, IObject, IMatrixCell>();
	Vector<IMatrixCell> cells = ivec.getUnorderedVector();
	for(int i=0; i<cells.size(); i++){
	    IMatrixCell mc = cells.get(i);
	    IMatrixCell omc = map.get(mc.getColumnIObject(), 
				      mc.getRowIObject());
	    if(omc!=null)
		remove.add(mc);
	    map.put(mc.getRowIObject(), mc.getColumnIObject(), mc);
	}

	for(int i=0; i<remove.size(); i++){
	    IMatrixCell mc = remove.get(i);
	    ivec.remove(mc);
	}
    }


    private static void findUnsymmetricCellsInIVector(IVector<IMatrixCell> ivec){
	Vector<IMatrixCell> remove = new Vector<IMatrixCell>();
	GenericDoubleHashMap<IObject, IObject, IMatrixCell> map = 
	    new GenericDoubleHashMap<IObject, IObject, IMatrixCell>();

	Vector<IMatrixCell> cells = ivec.getUnorderedVector();
	Vector<IMatrixCell> strangeCells = new Vector<IMatrixCell>(cells);

	for(int i=0; i<cells.size(); i++){
	    IMatrixCell mc = cells.get(i);
	    IMatrixCell omc = map.get(mc.getColumnIObject(), 
				      mc.getRowIObject());
	    if(omc!=null){
		//		remove.add(mc);
		strangeCells.remove(mc);
		//		IMatrixCell mc2 = map.get(mc.getRowIObject(), mc.getColumnIObject());
		strangeCells.remove(omc);
	    }
	    map.put(mc.getRowIObject(), mc.getColumnIObject(), mc);
	}

	for(int i=0; i<strangeCells.size(); i++){
	    IMatrixCell mc = strangeCells.get(i);
	    System.out.println(mc.getRowIObject().getString()+", "+
			       mc.getColumnIObject().getString()+": "+
			       mc.getWeight());
	}
	
	System.out.println(strangeCells.size());
    }


    private static void findNumOfIObjectInIMatrixIVector(IVector<IMatrixCell> ivec){
	HashSet<IObject> set = new HashSet<IObject>();
	Vector<IMatrixCell> vec = ivec.getUnorderedVector();
	for(int i=0; i<vec.size(); i++){
	    IMatrixCell mc = vec.get(i);
	    set.add(mc.getRowIObject());
	    set.add(mc.getColumnIObject());
	}

	System.out.println("Number of different IObjects: "+set.size());

    }

}
