package infomat.mro.risynlexeval;

import infomat.algorithms.grouping.*;
import infomat.algorithms.projection.*;
import infomat.bridges.jsdm.*;
import infomat.evaluation.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.vector.*;
import infomat.vectorspace.weighting.*;
import infomat.mro.synlex.*;

import mro.util.doublehashmap.*;
import mro.util.measures.*;
import mro.util.properties.*;

import java.io.*;
import java.util.*;


public class Syntest{

    Vector<SyntestItem> items = new Vector<SyntestItem>();

    public void readTest(String fileName) throws IOException{
	BufferedReader in = 
	    new BufferedReader(new FileReader(new File(fileName)));
	String s = in.readLine();
	while(s!=null){
	    SyntestItem si = new SyntestItem(s);
	    items.add(si);
	    s = in.readLine();
	}
    }

    public MeasureGroup test(ISimilarity similarity, IObjectGroup group, String measureGroupName){
	HashMap<String, IObject> stringIOMap = new HashMap<String, IObject>();
	Vector<IObject> ios = group.getOrdered();
	for(int i=0; i<ios.size(); i++){
	    IObject io = ios.get(i);
	    stringIOMap.put(io.getString(), io);
	}
	
	
	int wordUnknown = 0;
	int bestUnknown = 0;
	int otherUnknown = 0;
	int numberOfKnownQuestions = 0;
	int numberOfKnownAndCorrect = 0;
	int numberOfCorrect = 0;

	for(int i=0; i<items.size(); i++){
	    SyntestItem si = items.get(i);
	    String word = si.getWord();
	    IObject wordIO = stringIOMap.get(word);
	    if(wordIO==null){
		wordUnknown++;
	    }else{
		String[] suggestions = si.getSuggestions();
		String best = suggestions[si.getCorrectSuggestionNumber()];
		IObject bestIO = stringIOMap.get(best);
		if(bestIO==null){
		    bestUnknown++;
		}else{

		    boolean known = true;
		    boolean correct = true;
		    float bestSim = similarity.sim(wordIO, bestIO);
		    for(int j=0; j<suggestions.length; j++){
			String other = suggestions[j];
			IObject otherIO = stringIOMap.get(other);
			if(otherIO==null){
			    otherUnknown++;
			    known=false;
			}else{
			    float sim = similarity.sim(wordIO, otherIO);
			    if(sim>bestSim){
				correct = false;
			    }
			}
		    }
		    if(correct)
			numberOfCorrect++;
		    if(known)
			numberOfKnownQuestions++;
		    if(known && correct)
			numberOfKnownAndCorrect++;
		}
	    }
	}


	MeasureGroup mg = new MeasureGroup(measureGroupName);

	try{

	Measure m = new Measure("Correct Guesses", numberOfCorrect, "Correct Guesses");
	mg.addMeasure(m);
	
	m = new Measure("Correct Fully Recognized Guesses", numberOfKnownAndCorrect, "Correct Fully Recognized Guesses");
	mg.addMeasure(m);
	
	m = new Measure("Percentage Correct Fully Recognized Guesses", ((float) numberOfKnownAndCorrect)/numberOfKnownQuestions, "Percentage Correct Fylly Recognized Guesses");
	mg.addMeasure(m);
	
	//	m = new Measure("Percentage Correct Guesses", ((float) numberOfCorrect)/numberOfKnownQuestions, "Percentage Correct Guesses");
	//	mg.addMeasure(m);
	
	m = new Measure("Percentage Correct Guesses of All Questions", ((float) numberOfCorrect)/items.size(), "Percentage Correct Guesses of All Questions");
	mg.addMeasure(m);
	

	/*

	m = new Measure("Percentage Correct Guesses, word and best removed", ((float) numberOfCorrect)/(items.size()-wordUnknown-bestUnknown), "Percentage Correct Guesses, word and best removed");
	mg.addMeasure(m);
	
	m = new Measure("Percentage Correct Guesses, removed all unknown", ((float) numberOfCorrect-otherUnknown)/(items.size()-wordUnknown-bestUnknown-otherUnknown), "Percentage Correct Guesses, removed all unknown");
	mg.addMeasure(m);
	
	*/


	m = new Measure("Number of Questions", items.size(), "Number of Questions");
	mg.addMeasure(m);
	
	m = new Measure("Number of Fully Recognized Questions", numberOfKnownQuestions, "Number of Fully Recognized Questions");
	mg.addMeasure(m);
	
	m = new Measure("Number with unknown word", wordUnknown, "Number with unknown word");
	mg.addMeasure(m);
	
	m = new Measure("Number with best suggestion unknown", bestUnknown, "Number with best suggestion unknown");
	mg.addMeasure(m);

	m = new Measure("Number with other suggestion unknown", otherUnknown, "Number with other suggestion unknown");
	mg.addMeasure(m);

	}catch(MeasureNameConflictException mnce){
	    // Do noting
	    System.out.println("Syntest: MeasureNameConflictException");
	}
	
	return mg;
    }


}
