package infomat.algorithms.grouping;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import mro.util.*;
import mro.util.properties.*;

//import java.util.*;
import java.util.Vector; 
import java.io.*;
/*
  import java.util.Map.Entry; 
  import java.util.HashMap; 
  import java.util.HashSet; 
  import java.util.Set; 
*/

/**
 * Bisecting KMeans
 *      
 * @version          080425
 * @author           Magnus Rosell
 */
public class BisectingKMeans extends GroupingAlgorithm 
    implements NeedsISimilarity{ 

    public static int bisectingKmeansClusteringNumber = 1;
	

    public Properties getDefaultProperties(){

	Properties properties = new Properties("Bisecting KMeans Properties", CurrentClassGetter.getClassName());
	
	try{
	    TopDownSplitter tmpTDS = new TopDownSplitter();
	    Properties tmpTDSProperties = tmpTDS.getProperties();
	    tmpTDSProperties.removeProperty("Split into");
	    PropertyGroup pg = new PropertyGroup("Bisecting Properties");
	    pg.addPropertyVector(tmpTDSProperties.getPropertyVector());
	    properties.addProperty(pg);

	    KMeans tmpKMeans = new KMeans();
	    Properties tmpKMeansProperties = tmpKMeans.getProperties();
	    tmpKMeansProperties.removeProperty("Number of clusters");
	    pg = new PropertyGroup("KMeans Properties");
	    pg.addPropertyVector(tmpKMeansProperties.getPropertyVector());
	    properties.addProperty(pg);

	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	return properties;
    }





    private TopDownSplitter tds = null;
    private KMeans kmeans = null;
    private ISimilarity similarity = null;
	
    private void constructAlgorithms(){


	tds = new TopDownSplitter();
	tds.setSimilarity(similarity);
	try{
	    PropertyGroup pg = 
		properties.getPropertyGroup("Bisecting Properties");
	    Properties tdsProperties = tds.getProperties();
	    tdsProperties.setPropertiesInPropertyVector(pg.getPropertyVector());
	    tdsProperties.setProperty("Split into", 2);

	}catch(PropertyNotFoundException pnfe){
	    // Won't happen here
	}

	kmeans = new KMeans(similarity);
	kmeans.printGroupSizesInIterations = false;
	try{
	    PropertyGroup pg = 
		properties.getPropertyGroup("KMeans Properties");
	    Properties kmeansProperties = kmeans.getProperties();
	    kmeansProperties.setPropertiesInPropertyVector(pg.getPropertyVector());
	    kmeans.setProperty("Number of clusters", 2);

	}catch(PropertyNotFoundException pnfe){
	    // Won't happen here
	}
    }


    public BisectingKMeans(){
	this.properties = getDefaultProperties();
	constructAlgorithms();
    }

    public BisectingKMeans(mro.util.properties.Properties properties){
	this.properties=properties;
	constructAlgorithms();
    }

    public BisectingKMeans(ISimilarity similarity){
	this.similarity = similarity;
	this.properties = getDefaultProperties();	
	constructAlgorithms();
    }

    public BisectingKMeans(ISimilarity similarity, 
		  mro.util.properties.Properties properties){
	this.similarity = similarity;
	this.properties=properties;
	constructAlgorithms();
    }
	
    public void setSimilarity(ISimilarity similarity){
	this.similarity = similarity;
	constructAlgorithms();
    }



    public IObjectGrouping cluster(IObjectGroup iogroup){
	IObjectGrouping grouping = tds.cluster(iogroup, kmeans);
		
	String name = "Bisecting KMeans Clustering "+
	    bisectingKmeansClusteringNumber;
	bisectingKmeansClusteringNumber++;
	grouping.setName(name);

	return grouping;
    }


    /*
    public IObjectGrouping cluster(IObjectSet ioset){

	IObjectGrouping grouping = cluster((IObjectGroup) ioset);
	String name = grouping.getName();
	int i=1;
	while(!ioset.setGrouping(grouping)){
	    grouping.setName(name+" ("+i+")");
	    i++;
	}

	return grouping;
    }
    */


    public String getInfoString(){
    	return "Bisecting KMeans";
    }

    public String getAlgorithmName(){
    	return "Bisecting KMeans";    	
    }
    
    public Vector getPropertyVector(){
    	return properties.getPropertyVector();
    }
    

    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }
    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
	constructAlgorithms();
    }

    public mro.util.properties.Properties getProperties(){
    	return properties;
    }
    public void setProperties(mro.util.properties.Properties properties){
    	this.properties=properties;
	constructAlgorithms();
    }

    public UniqueIdentifier getUniqueIdentifier(){
    	return identifier;
    }

    public void setUniqueIdentifier(UniqueIdentifier uniqueIdentifier){
    	this.identifier=uniqueIdentifier;
    }

}
