package infomat.algorithms.grouping;

import infomat.vectorspace.*;
import mro.util.*;
import mro.util.properties.*;

import java.util.*;

/**
 * GroupMerger. 
 * perhaps should be in another package, like: infomat.algorithms.group;
 * Needed for Scatter/Gather-approach!
 *      
 * @version          080414
 * @author           Magnus Rosell
 */
public class GroupMerger implements HasProperties{

    protected Properties properties;

	
    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = 
	    new mro.util.properties.Properties("Group Merger Properties", CurrentClassGetter.getClassName());
		
	try{
			
	    Property p = new Property("Merging Order", 0, "In what order the objects are put into the merged group");
	    PropertyValueDescription pvd = new PropertyValueDescription("0", "One group after the other.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("1", "In order value order from the respective groups.");
	    p.addValueDescription(pvd);

	    properties.addProperty(p);

			
			
	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	return properties;
    }
	
    public GroupMerger(){
	this.properties = getDefaultProperties();
    }

    public GroupMerger(mro.util.properties.Properties properties){
	this.properties=properties;
    }
	
    public IObjectGroup merge(IObjectGroup iog1, IObjectGroup iog2){
	Vector<IObjectGroup> groups = new Vector<IObjectGroup>();
	groups.add(iog1);
	groups.add(iog2);
	return merge(groups);
    }

	
    public IObjectGroup merge(Vector<IObjectGroup> iogs){
	String name = "Merged (" + iogs.elementAt(0).getName();
	for(int i=1; i<iogs.size(); i++){
	    name += ", "+iogs.elementAt(i).getName();
	}
	name += ")";
	IObjectGroup mg = new IObjectGroup(name);

	int mergingOrder = 
	    (Integer) properties.getPropertyValue("Merging Order");
	if(mergingOrder==0){
	    for(int i=0; i<iogs.size(); i++){
		Vector<IObject> vec = iogs.elementAt(i).getOrdered();
		mg.addAll(vec);
	    }
	}else if(mergingOrder==1){
	    for(int i=0; i<iogs.size(); i++){
		HashMap<IObject, Float> map = 
		    iogs.elementAt(i).getObjectOrderValueMap();
		Iterator<Map.Entry<IObject, Float>> it = 
		    map.entrySet().iterator();
		while(it.hasNext()){
		    Map.Entry<IObject, Float> entry = it.next();
		    mg.add(entry.getKey(), entry.getValue());
		}
	    }
	}

	return mg;
    }

	
	
    public String getInfoString(){
    	return "Group Merger";
    }
    
    public Vector getPropertyVector(){
    	return properties.getPropertyVector();
    }
    

    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }
    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
    	return properties;
    }
    public void setProperties(mro.util.properties.Properties properties){
    	this.properties=properties;
    }

}
