package infomat.bridges.jsdm;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;

import mro.util.*;
import mro.util.properties.*;

import moj.ri.SparseDistributedMemory;


import java.io.*;
import java.util.*; 
import java.util.zip.*;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * Similarity between representations in JSDMWrapper.
 * Does not normalize the group vectors.
 *
 * @version          080608
 * @author           Magnus Rosell
 */
public class JSDMSimilarity extends DenseISimilarity{

    private JSDMWrapper jsdmw;

    public JSDMSimilarity(JSDMWrapper jsdmw){
	super();
	this.jsdmw=jsdmw;
    }

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("Java Sparse Distributed Memory Similarity Properties", CurrentClassGetter.getClassName());
		
	    /*
	try{

	    Property p = new Property("Least Term Frequency", 10, "Minimum Term Frequency to be Imported");
	    properties.addProperty(p);
	    p = new Property("Least Document Frequency", 0, "Minimum Document Frequency to be Imported");
	    properties.addProperty(p);

	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	    */

	return properties;
    }

    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }


    public String getName(){
	return "JSDMSimilarity (Cosine)";
    }


    public float sim(IObject io1, IObject io2){
	float[] io1Repr = getRepresentation(io1);
	float[] io2Repr = getRepresentation(io2);
	return sim(io1Repr, io2Repr);
    }

    public float sim(IObject io, IObjectGroup iog){
	float[] ioRepr = getRepresentation(io);
	float[] iogRepr = getGroupRepresentation(iog);
	return sim(ioRepr, iogRepr);
    }

    public float sim(IObjectGroup iog1, IObjectGroup iog2){
	float[] iog1Repr = getGroupRepresentation(iog1);
	float[] iog2Repr = getGroupRepresentation(iog2);
	return sim(iog1Repr, iog2Repr);
    }

    public float sim(float[] repr1, float[] repr2){
	float sim = 0.0f;
	if(repr1!=null && repr2!=null &&
	   repr1.length!=0 && repr2.length!=0){
	    for(int i=0; i<repr1.length; i++){
		sim += repr1[i]*repr2[i];
	    }
	}
	return sim;
    }

    public float[] getRepresentation(IObject io){
	return jsdmw.getWrappedRepresentation(io);
    }

    private void normalize(float[] repr){
	//	Iterator<float[]> it = ioReprMap.values().iterator();
	double sum = 0.0;
	for(int i=0; i<repr.length; i++)
	    sum += Math.pow(repr[i], 2);
	for(int i=0; i<repr.length; i++)
	    repr[i] = (float) Math.sqrt(Math.pow(repr[i],2)/sum);
    }

    public float[] getGroupRepresentation(IObjectGroup iog){
	float[] repr = getUnnormalizedGroupRepresentation(iog);
	normalize(repr);
	return repr;
    }

    public float[] getUnnormalizedGroupRepresentation(IObjectGroup iog){

	Vector<IObject> iovec = iog.getOrdered();
	if(iovec.size()<1)
	    return null;
	IObject io = iovec.elementAt(0);

	float[] r1 = getRepresentation(io);
	float[] cumulator = new float[r1.length];

	for(int i=0; i<iovec.size(); i++){
	    io = iovec.elementAt(i);
	    float[] r = getRepresentation(io);
	    if(r1!=null)
		add(cumulator, r);
	}
	
	for(int i=0; i<cumulator.length; i++){
	    cumulator[i] /= iovec.size();
	}

	return cumulator;
    }




    private void add(float[] cumulator, float[] addition){
	for(int i=0; i<cumulator.length; i++){
	    cumulator[i] += addition[i];
	}
    }


}
