package infomat.bridges.jsdm;

import infomat.algorithms.grouping.*;
import infomat.algorithms.labeling.*;
import infomat.evaluation.*;
import infomat.io.*;
import infomat.io.textresult.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;

import mro.util.*;
import mro.util.properties.*;
import mro.util.measures.*;

import java.io.*;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.xml.sax.SAXException;


/**
 * Simple stupid class for clustering RI:s from jsdm.
 *
 * <br>
 * <br>
 * <code>javac -deprecation -classpath ./classes/:lib/sdm.jar -sourcepath ./src ./src/infomat/bridges/jsdm/*.java -d classes</code> 
 *
 * <br>
 * <br>
 * <code>java -cp .:classes/ -Xms3024m -Xmx3024m infomat.bridges.jsdm.RIClusterer</code>
 *
 * @version          080605
 * @author           Magnus Rosell
 *
 */
public class RIClusterer{

    public static void main(String[] args) throws IOException, PropertyNotFoundException{

	// Load the matrix
	// This one has texts as rows and words as columns

	RIIO riio = new RIIO();

	//	String riFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Code/RI3/testRI.xml";

	String riFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Code/RI3/rodarummet.xml";
	File riFile = new File(riFileName);
	IMatrix matrix = riio.load(riFile);

	/*
	//	String riFileName = "/var/tmp/rosell/DN.xml.zip";
	String riFileName = "/var/tmp/rosell/KTHNC.xml.zip";
	File riFile = new File(riFileName);
	IMatrix matrix = riio.loadCompressed(riFile);
	*/

	System.out.println("Lst in");
	System.out.println("Matrix: "+matrix.getSize());
	System.out.println();

	String stoplistFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/stoppord/Stoplist.txt";
	Stoplist stoplist = new Stoplist(stoplistFileName);
	boolean stopRows = true;
	stoplist.removeIObjectsInStoplist(matrix, stopRows);

	System.out.println("Stoppat");
	System.out.println("Matrix: "+matrix.getSize());
	System.out.println();
			 
	Normalizer normalizer = new Normalizer();
	normalizer.normalize(matrix);

	System.out.println("Normaliserat");
	System.out.println("Matrix: "+matrix.getSize());
	System.out.println();


	/*
	// Load a categorization
	String categorizationFileName = "example/row_grouping.xml";
	File categorizationFile = new File(categorizationFileName);
	IObjectGroupingIO iobio = new IObjectGroupingIO();
	boolean rows=true; // row categorization
	IObjectGrouping categorization = 
	    iobio.load(rows, matrix, categorizationFile);
	*/

	/*
	// Filter the matrix
	// The example is to small
	IMatrixCellFilter matrixCellFilter = new IMatrixCellFilter();
	matrixCellFilter.filter(matrix);
	*/

	/*
	// Weight the matrix
	Weighter weighter = new Weighter();
	weighter.weight(matrix);
	*/

	// Cluster the words
	IObjectSet rowSet = matrix.getRowIObjectSet();

	// Similarity
	boolean row = true; // Similarity between the rows of the matrix.
	SparseDotSimilarity dotSim = new SparseDotSimilarity(matrix, row);
	//	dotSim.setProperty("Normalize", true);

	// Cluster
	KMeans kmeans = new KMeans(dotSim);
	kmeans.setProperty("Number of clusters", 20);
	IObjectGrouping clustering = kmeans.cluster(rowSet);
	clustering.removeEmptyGroups();

	/*
	try{
	    kmeans.setProperty("Number of clusters", 2);
	}catch(PropertyNotFoundException pnfe){
	    // Do nothing
	    System.out.println("PropertyNotFoundException!");
	}
	TopDownSplitter tds = new TopDownSplitter();
	tds.setSimilarity(dotSim);
	IObjectGrouping clustering = tds.cluster(rowSet, kmeans);
	clustering.removeEmptyGroups();
	*/

	/*
	BisectingKMeans bkm = new BisectingKMeans(dotSim);
	IObjectGrouping clustering = bkm.cluster(rowSet);
	clustering.removeEmptyGroups();
	*/


	// The current time
	long time = System.currentTimeMillis();
	
	// Write Text Result
	//	String resultPath = "result/";
	String resultPath = 
	    "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/riResult/";
	
	String resultFileName = 
	    resultPath+"clusteringResult."+time+".xml";
	File resFile = new File(resultFileName);
	//	GroupingTextIO gtio = new GroupingTextIO();
	IObjectGroupingTextResult gtio = new IObjectGroupingTextResult();
	gtio.save(matrix, clustering, dotSim, resFile);

	// Evaluate and write the results
	String evaluationFileName = 
	    resultPath+"clusteringEvaluation."+time+".xml";
	Evaluation eval = 
	    new Evaluation(matrix, clustering, dotSim, ""+time);
	calculateAndSaveEvaluation(eval, evaluationFileName);
	
	System.out.println("Clustered!!!");

    }


    private static void calculateAndSaveEvaluation(Evaluation eval, 
						   String resFileName){
	Measures measures = new Measures("Measures");
	try{
	    measures = eval.calculateAllMeasures();
	}
	catch(MeasureNameConflictException mnce){
	    System.out.println("Measure Name Conflict!");
	}
	measures.save(resFileName);
    }



}
