package infomat.bridges.jsdm;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;

import mro.util.*;
import mro.util.properties.*;

import moj.ri.SparseDistributedMemory;


import java.io.*;
import java.util.*; 
import java.util.zip.*;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * Similarity between compositional representations.
 * Does not normalize the group vectors.
 *
 * @version          080608
 * @author           Magnus Rosell
 */
public class RICompositionalSimilarity extends DenseISimilarity{

    private RICompositionalRepresentation riCRepr;

    public RICompositionalSimilarity(RICompositionalRepresentation riCRepr){
	super();
	this.riCRepr=riCRepr;
    }

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("RI Compositional Similarity Properties", CurrentClassGetter.getClassName());
		
	    /*
	try{

	    Property p = new Property("Least Term Frequency", 10, "Minimum Term Frequency to be Imported");
	    properties.addProperty(p);
	    p = new Property("Least Document Frequency", 0, "Minimum Document Frequency to be Imported");
	    properties.addProperty(p);

	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	    */

	return properties;
    }

    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }


    public String getName(){
	return "RI Compositional Similarity (Cosine)";
    }


    public float sim(IObject io1, IObject io2){
	float[] io1Repr = getRepresentation(io1);
	float[] io2Repr = getRepresentation(io2);
	return sim(io1Repr, io2Repr);
    }

    public float sim(IObject io, IObjectGroup iog){
	float[] ioRepr = getRepresentation(io);
	float[] iogRepr = getGroupRepresentation(iog);
	return sim(ioRepr, iogRepr);
    }

    public float sim(IObjectGroup iog1, IObjectGroup iog2){
	float[] iog1Repr = getGroupRepresentation(iog1);
	float[] iog2Repr = getGroupRepresentation(iog2);
	return sim(iog1Repr, iog2Repr);
    }

    public float sim(float[] repr1, float[] repr2){
	float sim = 0.0f;
	for(int i=0; i<repr1.length; i++){
	    sim += repr1[i]*repr2[i];
	}
	return sim;
    }

    public float[] getRepresentation(IObject io){
	return riCRepr.getRepresentation(io);
    }


    public float[] getUnnormalizedGroupRepresentation(IObjectGroup iog){
	Vector<IObject> iovec = iog.getOrdered();
	if(iovec.size()<1)
	    return null;
	IObject io = iovec.elementAt(0);
	float[] r1 = getRepresentation(io);
	float[] cumulator = new float[r1.length];

	for(int i=0; i<iovec.size(); i++){
	    io = iovec.elementAt(i);
	    float[] r = getRepresentation(io);

	    if(r==null){
		System.out.println("_ "+io.getString()+": null");
	    }else if(r.length<cumulator.length){
		System.out.println("_ "+io.getString()+": "+r.length);
	    }else
		add(cumulator, r);
	}
	
	for(int i=0; i<cumulator.length; i++){
	    cumulator[i] /= iovec.size();
	}
	return cumulator;
    }


    /**
     * Using normalized representation.
     */
    public float[] getGroupRepresentation(IObjectGroup iog){
	Vector<IObject> iovec = iog.getOrdered();
	if(iovec.size()<1)
	    return null;
	IObject io = iovec.elementAt(0);
	float[] r1 = getRepresentation(io);
	float[] cumulator = new float[r1.length];

	cumulator = getUnnormalizedGroupRepresentation(iog);

	normalize(cumulator);

	return cumulator;
    }

    private void add(float[] cumulator, float[] addition){
	for(int i=0; i<cumulator.length; i++){
	    cumulator[i] += addition[i];
	}
    }


    private void normalize(float[] vec){
	double sqsum = 0.0;
	for(int i=0; i<vec.length; i++){
	    sqsum += Math.pow(vec[i], 2);
	}

	for(int i=0; i<vec.length; i++){
	    vec[i] = (float) Math.sqrt(Math.pow(vec[i], 2)/sqsum);
	}
    }


}
