package tctbridge.algorithms.grouping;

// Infomat
import tctbridge.io.*;
import infomat.algorithms.grouping.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import mro.util.*;
import mro.util.properties.*;

// TCT
import text.data.Corpus;
import core.metrics.similarity.CosineSimilarity;
import core.clustering.partitional.KMeans;
import core.clustering.hierarchical.AgglomerativeClusterer;
import core.clustering.*;
import core.clustering.init.*;
import core.clustering.util.*;
import core.metrics.cluster.*;
import matrix.SymMatrix;


// Java
import java.util.*;


/**
 * TCT KMeans. Using the TCT package.
 *      
 * @version          080306
 * @author           Magnus Rosell
 */
public class TCTAgglomerative extends GroupingAlgorithm 
    implements NeedsIMatrix{ 

    /*
      protected Properties properties;
      protected UniqueIdentifier identifier;
    */

    private static int tctAgglomerativeNumber = 1;
	
    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = 
	    new mro.util.properties.Properties("TCT Agglomerative Properties", CurrentClassGetter.getClassName());
		
	try{

	    Property p = new Property("Number of clusters", 5, "Number of clusers");
	    properties.addProperty(p);

	    p = new Property("Inter Cluster Metric", 0, "Inter Cluster Similarity Defintion");
	    PropertyValueDescription pvd = new PropertyValueDescription("0", "Cosine Similarity");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	    p = new Property("Cluster Linkage", 0, "Kind of cluster linkage in similarity calculation");
	    pvd = new PropertyValueDescription("0", "Average");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("1", "Complete");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("2", "Single");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("3", "Total");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	    p = new Property("Similarity Sort", true, "Sort objects in clusters according to similarity to cluster centroid.");
	    pvd = new PropertyValueDescription(true, "Do it.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription(false, "Do not do it.");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

			
	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	return properties;
    }

    private IMatrix matrix;
	
    public TCTAgglomerative(){
	this.properties = getDefaultProperties();
    }

    public TCTAgglomerative(mro.util.properties.Properties properties){
	this.properties=properties;
    }
	
    public void setIMatrix(IMatrix matrix){
	this.matrix=matrix;
    }

    public IMatrix getIMatrix(){
	return this.matrix;
    }


    public IObjectGrouping cluster(IObjectSet ioset){

	IObjectGrouping grouping = cluster((IObjectGroup) ioset);
	String name = grouping.getName();
	int i=1;
	while(!ioset.setGrouping(grouping)){
	    grouping.setName(name+" ("+i+")");
	    i++;
	}

	return grouping;
    }




    public IObjectGrouping cluster(IObjectGroup group){
	TCTBridge tctB = new TCTBridge();
	Corpus corpus = tctB.getCorpus(matrix, group);

	int simType = (Integer) 
	    properties.getPropertyValue("Inter Cluster Metric");
	CosineSimilarity metric = new CosineSimilarity();
	if(simType==0)
	    metric = new CosineSimilarity();

	int linkType = (Integer) 
	    properties.getPropertyValue("Cluster Linkage");
	Linkage linkage = Linkage.AVERAGE;
	if(linkType==0)
	    linkage = Linkage.AVERAGE;
	else if(linkType==1)
	    linkage = Linkage.COMPLETE;
	else if(linkType==2)
	    linkage = Linkage.SINGLE;
	else if(linkType==3)
	    linkage = Linkage.TOTAL;

	SymMatrix symMatrix = metric.buildDistanceMatrix(corpus);
	InterClusterMetric interClusterMetric = 
	    new InterClusterSimilarity(symMatrix, linkage);

	int k = (Integer) properties.getPropertyValue("Number of clusters");

	AgglomerativeClusterer clusterer = 
	    new AgglomerativeClusterer(interClusterMetric, k);

	HardClustering clustering = null;
	try{
	    clustering = clusterer.findClusters(corpus);
	}catch(ClusteringException cle){
	    System.out.println(cle);
	    cle.printStackTrace();
	    System.exit(0);
	}

	boolean simOrder = (Boolean) properties.getPropertyValue("Similarity Sort");

	/*
	IObjectGrouping grouping = 
	    tctB.getIObjectGrouping(matrix, group, corpus, clustering, metric, simOrder, clusterer);
	*/

	IObjectGrouping tctGrouping = 
	    tctB.getIObjectGrouping(matrix, group, corpus, clustering, metric, simOrder, clusterer);
	
	String name = tctGrouping.getName()+tctAgglomerativeNumber;
	IObjectGrouping grouping = new IObjectGrouping(name);
	/*
	int i=1;
	while(!ioset.setGrouping(grouping)){
	    grouping.setName(name+" ("+i+")");
	    i++;
	}
	*/

	Vector<IObjectGroup> groups = tctGrouping.getGroups();
	for(int g=0; g<groups.size(); g++){
	    IObjectGroup iog = groups.elementAt(g);
	    grouping.add(iog);
	    //	    printGroup(group);
	}

	tctAgglomerativeNumber++;

	return grouping;
    }

    public String getInfoString(){
    	return "TCTKAgglomerative";
    }

    public String getAlgorithmName(){
    	return "TCTAgglomerative";    	
    }
    
    public Vector getPropertyVector(){
    	return properties.getPropertyVector();
    }
    

    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }
    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
    	return properties;
    }
    public void setProperties(mro.util.properties.Properties properties){
    	this.properties=properties;
    }

    public UniqueIdentifier getUniqueIdentifier(){
    	return identifier;
    }

    public void setUniqueIdentifier(UniqueIdentifier uniqueIdentifier){
    	this.identifier=uniqueIdentifier;
    }

}
