package tctbridge.io;

import infomat.vectorspace.*;

import java.io.*;
import java.util.*; 
import java.util.zip.*;
import java.awt.Dimension;


/**
 * For exporting/importing IMatrix to/from TCT-format.
 * Note: Strictly for use with text-word-matrixes, where
 * the texts are rows and words columns.
 * TCT - Text Clustering Toolkit, see http://mlg.ucd.ie/content/view/18/
 * 
 * @version          080229
 * @author           Magnus Rosell
 */
public class TCTIO{

    
    public void saveIMatrix(IMatrix matrix, File file) throws IOException{
	PrintWriter fw = new PrintWriter(new FileWriter(file));

	Dimension d = matrix.getSize();
	fw.println(d.getHeight()+" "+d.getWidth());
	

	IObjectSet rows = matrix.getRowIObjectSet();
	Vector<IObject> rowVec = rows.getOrdered();
	HashMap<IObject, Integer> rowNrMap = new HashMap<IObject, Integer>();
	for(int i=0; i<rowVec.size(); i++)
	    rowNrMap.put(rowVec.elementAt(i), i);
	IObjectTCTComparator iotctc = new IObjectTCTComparator(rowNrMap);

	IObjectSet cols = matrix.getColumnIObjectSet();
	Vector<IObject> colVec = cols.getOrdered();
	HashMap<IObject, Integer> colNrMap = new HashMap<IObject, Integer>();
	for(int i=0; i<colVec.size(); i++){
	    IObject colO = colVec.elementAt(i);
	    colNrMap.put(colO, i);
	    HashSet cellSet = matrix.getRowIMatrixCellsForColumnObject(colO);
	    Vector cells = new Vector(cellSet);
	    Collections.sort(cells, iotctc);
	    for(int j=0; j<cells.size(); j++){
		IMatrixCell mc = (IMatrixCell) cells.elementAt(j);
		fw.println(((int) i)+" "+((int) j)+" "+
			   ((int) mc.getLocalCount()));
	    }
	}
    }




}

class IObjectTCTComparator implements Comparator{
    
    HashMap<IObject, Integer> objNrMap = null;

    public IObjectTCTComparator(HashMap<IObject, Integer> objNrMap){
	this.objNrMap=objNrMap;
    }

    public int compare(Object o1, Object o2) {
	IMatrixCell mc1 = (IMatrixCell) o1;
	IMatrixCell mc2 = (IMatrixCell) o2;
	Integer i1 = objNrMap.get(mc1.getRowIObject());
	Integer i2 = objNrMap.get(mc2.getRowIObject());
	return i1.compareTo(i2);
    }

}
