package infomat.io;

import infomat.vectorspace.*;

import mro.util.PathHelper;

import java.io.*;
import java.util.*; 
import java.util.zip.*;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * For loading and saving an IMatrix.
 * The dtd is changed as of 080319. 
 * Anything in a matrix file is read, 
 * but a minimum is written. 
 * 
 * @version          080319
 * @author           Magnus Rosell
 */
public class IMatrixIO{

    // The InfomatModel that is loaded/saved
    //    private InfomatModel im;
    private IMatrix matrix;

    public IMatrixIO(){ //InfomatModel im){
	//	this.im=im;
    }



    /**
     * Loads a IMatrix from file in XML format
     */
    public IMatrix load(File file) {
	System.out.println("************** "+file.getPath());
	try {
	    BufferedReader in = 
		new BufferedReader(new FileReader(file));
	    InputSource source = new InputSource(in);
	    return load(source);
	}
	catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
	return null;
    }

    public IMatrix loadCompressed(File file) {
	try {
	    // Create a ZipInputStream to read the zip file            
	    ZipInputStream zis = new ZipInputStream(new BufferedInputStream(new FileInputStream(file)));
	    InputSource source = new InputSource(zis);
	    ZipEntry entry;
	    if((entry = zis.getNextEntry()) != null) {
		if(!entry.isDirectory()) {
		    return load(source);
		}
	    }
	    zis.close();
	}
	catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
	return null;

    }

    private IMatrix load(InputSource source) {
	try {

	    IMatrix m = new IMatrix("dummy label", "dummy label");

	    //	    im.newIMatrix();

	    // Create parser+contentHandler and read the XML document
	    XMLReader parser = XMLReaderFactory.createXMLReader();
	    IMatrixHandler handler = new IMatrixHandler(m);
	    parser.setContentHandler(handler);
	    parser.parse(source);

	    m.getRowIObjectSet().sortInCurrentVectorOrderWithInvertedOrderNumberAsValue();
	    m.getColumnIObjectSet().sortInCurrentVectorOrderWithInvertedOrderNumberAsValue();

	    //	    m.getRowIObjectSet().makeStartGrouping();
	    //	    m.getColumnIObjectSet().makeStartGrouping();

	    return m;

	    //	    corpus.setUniqueIdentifier(handler.getUniqueIdentifier());
	} catch (Exception e) {
	    System.err.println(e);
	    e.printStackTrace();
	}
	return null;
    }


    /**
     * Exports the IMatrix in XML format
     */
    public void save(IMatrix m, File file){
	matrix=m;
	try {
	    PrintWriter out =
		new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(file))));
	    save(out);
	    out.close();
	}
	catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
    }


    /**
     * Exports the IMatrix in XML format
     * The file is saved in compressed .zip-format.
     */
    public void saveCompressed(IMatrix m, String fileName) { 
	matrix=m;
	try {
	    // Create zip compressed output stream, zip file entry and PrintWriter
	    ZipOutputStream targetStream = new ZipOutputStream(new FileOutputStream(fileName+".xml.zip"));
	    File file = new File(fileName);                        
	    targetStream.putNextEntry(new ZipEntry(file.getName() + ".xml"));
	    PrintWriter out = new PrintWriter(targetStream);
	    save(out);            
	    // Close compressed output stream and PrintWriter
	    out.flush();
	    targetStream.closeEntry();
	    targetStream.close();
	    targetStream = null;
	    out.close();
	}
	catch (java.io.IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
    }




    public void printDTD(PrintWriter out) {
	//We use the xml-format specified in corpus.dtd
	//out.println("<?xml version='1.0' encoding='ISO-8859-1'?>");

	out.println("<?xml version='1.0' encoding='ISO-8859-1' standalone=\"yes\"?>");

	//	out.println("<?xml-stylesheet type=\"text/xsl\" href=\""+corpus.sourcePath+"default/Corpus.xsl\"?>");

	out.println("<!DOCTYPE InfomatIMatrix [");

	out.println("<!ELEMENT InfomatIMatrix (Rows, Columns, Cells)>");
	out.println("<!ELEMENT Rows (IObjectSet)>");
	out.println("<!ELEMENT Columns (IObjectSet)>");

	out.println("<!--A Set of IObjects-->");
	out.println("<!ELEMENT IObjectSet (SetName, CommonLocation?, io*)>");
	out.println("<!ELEMENT SetName (#PCDATA)>");
	out.println("<!ELEMENT CommonLocation (#PCDATA)> <!-- The root location for all objects-->");

	out.println("<!ELEMENT io (s, l?, k?, gw?, id)> <!--A IObject-->");
	out.println("<!ELEMENT s (#PCDATA)> <!--The objects description-->");
	out.println("<!ELEMENT l (#PCDATA)> <!--The objects location-->");
	out.println("<!ELEMENT k (#PCDATA)> <!--Kind-->");
	out.println("<!ELEMENT gw (#PCDATA)> <!--Global Weight-->");
	out.println("<!ELEMENT id (#PCDATA)> <!--Object ID-->");


	out.println("<!--The Non-Zero Cells of the IMatrix-->");
	out.println("<!ELEMENT IMatrixCells (mc*)>");
	out.println("<!ELEMENT mc (rid, cid, c, lw?, w?)> <!--IMatrix Cell-->");
	out.println("<!ELEMENT rid (#PCDATA)> <!--Row Object ID-->");
	out.println("<!ELEMENT cid (#PCDATA)> <!--Column Object ID-->");
	out.println("<!ELEMENT c (#PCDATA)> <!--Frequency-->");
	out.println("<!ELEMENT lw (#PCDATA)> <!--Local Weight-->");
	out.println("<!ELEMENT w (#PCDATA)> <!--Weight-->");

	out.println("]>");
    }



    private void save(PrintWriter out) {

	printDTD(out);

	out.println("<InfomatMatrx>");


	out.println("<Rows>");
	out.println("<IObjectSet>");
	saveIObjectSet(matrix.getRowIObjectSet(), out);
	out.println("</IObjectSet>");
	out.println("</Rows>");

	out.println("<Columns>");
	out.println("<IObjectSet>");
	saveIObjectSet(matrix.getColumnIObjectSet(), out);
	out.println("</IObjectSet>");
	out.println("</Columns>");

	out.println("<IMatrixCells>");
	saveCells(matrix, out);
	out.println("</IMatrixCells>");

	out.println("</InfomatMatrx>");

	out.flush();

    }



    private void saveIObjectSet(IObjectSet set, PrintWriter out){
	out.println("\t<SetName>"+set.getName()+"</SetName>");
	String commonLocation = set.getLargestStartOfLocation();
	if(commonLocation!=null)
	    out.println("\t<CommonLocation>"+commonLocation+"</CommonLocation>");
	Vector<IObject> objs = set.getOrdered();
	for(int i=0; i<objs.size(); i++){
	    out.println("\t<io>");
	    IObject io = objs.elementAt(i);
	    out.println("\t\t<s>"+io.getAmbiguousString()+"</s>");
	    if(io.getLocation()!=null){
		if(commonLocation!=null){
		    String location = io.getLocation();
		    location = location.substring(commonLocation.length(), location.length());
		    out.println("\t\t<l>"+location+"</l>");
		}else{
		    out.println("\t\t<l>"+io.getLocation()+"</l>");
		}

	    }
	    //	    out.println("<k>"+io.getKind()+"</k>");
	    //	    out.println( Global Weight
	    out.println("\t\t<id>"+io.getID()+"</id>");
	    out.println("\t</io>");
	}
    }

    private void saveCells(IMatrix matrix, PrintWriter out){
	Vector cells = matrix.getAllCells();
	for(int i=0; i<cells.size(); i++){
	    IMatrixCell mc = (IMatrixCell) cells.elementAt(i);
	    out.println("\t<mc>");
	    out.println("\t\t<rid>"+mc.getRowIObject().getID()+"</rid>");
	    out.println("\t\t<cid>"+mc.getColumnIObject().getID()+"</cid>");
	    out.println("\t\t<c>"+mc.getLocalCount()+"</c>");
	    //	    out.println("<lw>"+mc.getLocalWeight()+"</lw>");
	    out.println("\t\t<w>"+mc.getWeight()+"</w>");
	    out.println("\t</mc>");
	}
    }



    //    private void saveIObjectSet(IObjectSet set, PrintWriter out){
    /*
      out.println("<!--The Non-Zero Cells of the IMatrix-->");
      out.println("<!ELEMENT Cells (IMatrixCell*)>");
      out.println("<!ELEMENT IMatrixCell (rowID, colID, count?, lw?, w)>");
      out.println("<!ELEMENT rowID (#PCDATA)> <!--Row Object ID-->");
      out.println("<!ELEMENT colID (#PCDATA)> <!--Column Object ID-->");
      out.println("<!ELEMENT c (#PCDATA)> <!--Frequency-->");
      out.println("<!ELEMENT lw (#PCDATA)> <!--Local Weight-->");
      out.println("<!ELEMENT w (#PCDATA)> <!--Weight-->");
    */
    //    }




    private class IMatrixHandler extends DefaultHandler {
	private StringBuffer buffer;

	private IMatrix m;

	// IObjectSet:s
	private boolean currentRow=false;
	private IObjectSet currentSet;
	private String commonLocation=null;
	private String s=null;
	private String l=null;
	private int k=-1;
	private boolean foundGW = false;
	private float gw=0f;
	private int id=-1;

	// IMatrix Cells
	private int rid;
	private int cid;
	private float c;
	private boolean foundLW = false;
	private float lw=0f;
	private float weight=0f;

	public IMatrixHandler(IMatrix m){ //InfomatModel im) {
	    /*
	      corpus.idDocumentMap = new HashMap<Integer,Document>();
	      corpus.wordImageList.idWordImageMap = 
	      new HashMap<Integer,WordImage>();
	      this.corpus = corpus;
	      _wordList = corpus;

	      ui = new UniqueIdentifier(corpus);
	    */
	    //	    this.im=im;
	    this.m=m;

	}

	/*
	  public UniqueIdentifier getUniqueIdentifier(){
	  return ui;
	  }
	*/

	public void startElement (String namespaceURI, String localName,
				  String qualifiedName, Attributes atts) {
	    buffer = new StringBuffer();

	    if(qualifiedName.equals("Rows")) {                
		commonLocation=null;
		currentSet=m.getRowIObjectSet();
		currentRow=true;
	    }else if(qualifiedName.equals("Columns")) {                
		commonLocation=null;
		currentSet=m.getColumnIObjectSet();
		currentRow=false;
	    }
	}

	public void endElement (String namespaceURI, String localName, String qualifiedName) {

	    //	    System.out.println(buffer.toString().trim());

	    // IObjectSet:s 
	    if(qualifiedName.equals("SetName")){
		currentSet.setName(buffer.toString().trim());
	    }else if(qualifiedName.equals("CommonLocation")){
		commonLocation=PathHelper.convertToCurrentFileSeparator(buffer.toString().trim());
	    }else if(qualifiedName.equals("s")){
		s=buffer.toString().trim();
	    }else if(qualifiedName.equals("l")){
		l=PathHelper.convertToCurrentFileSeparator(buffer.toString().trim());
		if(commonLocation!=null && !l.startsWith(commonLocation)){
		    l=commonLocation+l;
		}
	    }else if(qualifiedName.equals("k")){
		k=Integer.parseInt(buffer.toString().trim());
	    }else if(qualifiedName.equals("gw")){
		gw=Integer.parseInt(buffer.toString().trim());
		foundGW=true;
	    }else if(qualifiedName.equals("id")){
		id=Integer.parseInt(buffer.toString().trim());
	    }else if(qualifiedName.equals("io")){
		IObject io = null;
		if(s!=null){
		    if(l!=null && k!=-1 && id!=-1){
			io = new IObject(s, l, k, id);
		    }else if(l!=null && id!=-1)
			io = new IObject(s, l, id);
		    else if(id!=-1)
			io = new IObject(s, id);
		    //		    System.out.println("matrixIO.add");
		    currentSet.add(io);
		}
		if(io!=null && foundGW){
		    if(currentRow)
			m.setRowIObjectGlobalWeight(io, gw);
		    else
			m.setColumnIObjectGlobalWeight(io, gw);
		    foundGW=false;
		}
		s=null;
		l=null;
		k=-1;
		id=-1;
	    }

	    // IMatrix Cells
	    else if(qualifiedName.equals("rid")){
		rid=Integer.parseInt(buffer.toString().trim());
	    }else if(qualifiedName.equals("cid")){
		cid=Integer.parseInt(buffer.toString().trim());
	    }else if(qualifiedName.equals("c")){
		c=Float.parseFloat(buffer.toString().trim());
	    }else if(qualifiedName.equals("lw")){
		lw=Float.parseFloat(buffer.toString().trim());
		foundLW = true;
	    }else if(qualifiedName.equals("w")){
		weight=Float.parseFloat(buffer.toString().trim());
	    }else if(qualifiedName.equals("mc")){
		// There must be a weight for the GUI to work.
		if(weight==0) 
		    weight = c;
		if(foundLW){
		    m.addCell(rid, cid, c, lw, weight);
		    foundLW=false;
		}else
		    m.addCell(rid, cid, c, weight);
	    }
	}



	public void characters (char ch[], int start, int length) {
	    if(buffer != null)
		buffer.append(ch,start,length);
	}      
    }




}
