package infomat.io.textresult;

import mro.util.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import infomat.vectorspace.*;
import infomat.vectorspace.ordering.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.vector.*;

import java.io.*;
import java.util.*; 
import java.util.zip.*;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * For loading and saving groups to text.
 * 
 * @version          080626
 * @author           Magnus Rosell
 */
public class IObjectGroupTextResult implements HasProperties{

    private Properties properties;
    private String separator;

    public IObjectGroupTextResult(){
	this.properties=getDefaultProperties();
	separator = System.getProperties().getProperty("file.separator");
    }

    public IObjectGroupTextResult(Properties properties){
	this.properties=properties;
	separator = System.getProperties().getProperty("file.separator");
    }

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("IObjectGroup Text Result Properties", CurrentClassGetter.getClassName());
		
	try{

	    Property p = new Property("Number of Description IObjects For Group", 10, "The number of Description IObjects that is presented.");
	    properties.addProperty(p);

	    p = new Property("Number of Representation IObjects For Group", 10, "The number of Representation IObjects that is presented.");
	    properties.addProperty(p);

	    p = new Property("Number of IObjects For Group", 10, "The number of IObjects that is presented.");
	    properties.addProperty(p);

	}catch (PropertyNameConflictException pnce){
	    //			never happens
	}
	return properties;
    }


    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }






    public static void printGroupDTD(PrintWriter out) {
	//We use the xml-format specified in corpus.dtd
	//out.println("<?xml version='1.0' encoding='ISO-8859-1'?>");

	out.println("<!DOCTYPE InfomatIObjectGroupText [");

	printPartGroupDTD(out);

	IVectorTextResult.printPartIVectorDTD(out);

	out.println("]>");
    }

    public static void printPartGroupDTD(PrintWriter out) {

	out.println("<!ELEMENT InfomatIObjectGroupText (InfomatIObjectGroupTextGeneralInfo, DescriptionIObjects?, RepresentationIObjects?, IObjects)>");

	out.println("<!ELEMENT InfomatIObjectGroupTextGeneralInfo (IObjectGroupName, IObjectGroupLocation, NumberOfIObjects, NumberOfDescriptionIObjects?, NumberOfRepresentationIObjects?)>");
	out.println("<!ELEMENT IObjectGroupName (#PCDATA)>");
	out.println("<!ELEMENT IObjectGroupLocation (#PCDATA)>");
	out.println("<!ELEMENT NumberOfIObjects (#PCDATA)>");
	out.println("<!ELEMENT NumberOfDescriptionIObjects (#PCDATA)>");
	out.println("<!ELEMENT NumberOfRepresentationIObjects (#PCDATA)>");

	out.println("<!--The DescriptionIObjects of the IObjectGroup-->");
	out.println("<!ELEMENT DescriptionIObjects (IVector)>");

	out.println("<!--The RepresentationIObjects of the IObjectGroup-->");
	out.println("<!ELEMENT RepresentationIObjects (IVector)>");

	out.println("<!--The IObjects of the IObjectGroup-->");
	out.println("<!ELEMENT IObjects (IVector)>");
    }

    /*
    public void save(File file, IObjectGroup group, IVector descriptionVec, ISimilarity similarity, String ioUnknownLocation) throws IOException{
	try {
	    PrintWriter out =
		new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(file))));
	    save(out, group, descriptionVec, similarity, ioUnknownLocation);
	    out.close();
	}
	catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
    }
    */


    public <T extends IVectorIndex> void saveGroupFile(PrintWriter out, String xslGroupFileName, IObjectGroup group, String groupLocation, IVector<T> descriptionVec, ISimilarity similarity, String ioUnknownLocation) throws IOException{

	// XML
	XMLHelper.printXMLHead(out);
	printIObjectGroupXSLReference(out, xslGroupFileName); 
	printGroupDTD(out);

	savePartGroupFile(out, group, groupLocation, descriptionVec, similarity, ioUnknownLocation);

	out.flush();
    }






    public <T extends IVectorIndex> void savePartGroupFile(PrintWriter out, IObjectGroup group, String groupLocation, IVector<T> descriptionVec, ISimilarity similarity, String ioUnknownLocation) throws IOException{

	int exportNumberOfDescriptionObjects = (Integer) properties.getPropertyValue("Number of Description IObjects For Group");

	int exportNumberOfRepresentativeObjects = (Integer) properties.getPropertyValue("Number of Representation IObjects For Group");

	int exportNumberOfObjects = (Integer) properties.getPropertyValue("Number of IObjects For Group");

	out.println("<InfomatIObjectGroupText>");
	out.println("<InfomatIObjectGroupTextGeneralInfo>");
	out.println("<IObjectGroupName>"+group.getName()+"</IObjectGroupName>");
	if(groupLocation!=null){
	    out.println("<IObjectGroupLocation>"+groupLocation+"</IObjectGroupLocation>");
	}

	out.println("<NumberOfIObjects>"+group.getSortedObjectOrderValueVector().size()+"</NumberOfIObjects>");
	
	if(descriptionVec!=null){
	    out.println("<NumberOfDescriptionIObjects>"+descriptionVec.size()+"</NumberOfDescriptionIObjects>");
	}

	SparseISimilarity sis = null;
	if(similarity instanceof SparseISimilarity){
	    sis = (SparseISimilarity) similarity;
	}
	if(sis!=null){
	    out.println("<NumberOfRepresentationIObjects>"+sis.getGroupRepresentation(group).size()+"</NumberOfRepresentationIObjects>");
	}
	out.println("</InfomatIObjectGroupTextGeneralInfo>");

	if(descriptionVec!=null){
	    out.println("<DescriptionIObjects>");
	    IVectorTextResult.saveIVector(out, descriptionVec, exportNumberOfDescriptionObjects, ioUnknownLocation);
	    out.println("</DescriptionIObjects>");
	}

	if(sis!=null){
	    IVector<IVectorIndex> reprVec = sis.getGroupRepresentation(group);

	    out.println("<RepresentationIObjects>");
	    IVectorTextResult.saveIVector(out, reprVec, exportNumberOfRepresentativeObjects, ioUnknownLocation);
	    out.println("</RepresentationIObjects>");
	}

	IVector<IObject> objVec = group.getIVector();
	out.println("<IObjects>");
	IVectorTextResult.saveIVector(out, objVec, exportNumberOfObjects, ioUnknownLocation);
	out.println("</IObjects>");

	out.println("</InfomatIObjectGroupText>");

	out.flush();
    }








    /**
     * Prints a basic XSL-file for IObjectGroup XML-files to a 
     * {@link java.io.PrintWriter}.
     */
    public void printGroupXSL(PrintWriter pw) throws IOException{
	pw.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
	pw.println("<xsl:stylesheet version=\"1.0\"");
	pw.println("xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\">");
	pw.println("<xsl:template match=\"/\">");
	pw.println("<html>");
	pw.println("<body>");
	pw.println("<xsl:apply-templates/>");
	pw.println("</body>");
	pw.println("</html>");
	pw.println("</xsl:template>");
	pw.println("");

	pw.println("<xsl:template match=\"InfomatIObjectGroupTextGeneralInfo\">");
	pw.println("<h2><xsl:value-of select=\"IObjectGroupName\"/></h2>");
	pw.println("<xsl:value-of select=\"NumberOfIObjects\"/> IObjects.");
	pw.println("<xsl:value-of select=\"NumberOfRepresentationIObjects\"/> Representation IObjects.");
	pw.println("<br />");
	pw.println("</xsl:template>");
	pw.println("");
	
	String context = "";
	IVectorTextResult.printPartDescriptionIVectorXSL(pw, context);
	IVectorTextResult.printPartRepresentationIVectorXSL(pw, context);
	IVectorTextResult.printPartIObjectGroupIVectorXSL(pw, context);

	pw.println("</xsl:stylesheet>");
	pw.println("");
    }


    public static void printPartOfGroupingXSL(PrintWriter pw, String context) throws IOException{
	pw.println("<xsl:template match=\""+context+"InfomatIObjectGroupText\">");
	pw.println("<li>");
	pw.println("<xsl:apply-templates select=\"InfomatIObjectGroupTextGeneralInfo\"/> ");
	pw.println("<br />");
	pw.println("<xsl:apply-templates select=\"DescriptionIObjects/IVector\"/> ");
	pw.println("<xsl:apply-templates select=\"RepresentationIObjects/IVector\"/> ");
	pw.println("<xsl:apply-templates select=\"IObjects/IVector\"/> ");

	pw.println("</li>");
	pw.println("</xsl:template>");
	pw.println("");



	pw.println("<xsl:template match=\"InfomatIObjectGroupTextGeneralInfo\">");

	pw.println("<xsl:element name=\"a\">");
	pw.println("<xsl:attribute name=\"href\">");
	pw.print("file:///");
	pw.println("<xsl:value-of select=\"IObjectGroupLocation\"/>");
	pw.println("</xsl:attribute>");
	pw.println("<xsl:value-of select=\"IObjectGroupName\"/>");
	pw.println("</xsl:element>");
	pw.println("&#160;");
	pw.println("<xsl:value-of select=\"NumberOfIObjects\"/> IObjects.");
	pw.println("<xsl:value-of select=\"NumberOfRepresentationIObjects\"/> Representation IObjects.");

	pw.println("</xsl:template>");
	pw.println("");






	/*
	pw.println("<xsl:template match=\""+context+"InfomatIObjectGroupTextGeneralInfo\">");
	//	pw.println("<li>");

	pw.println("<xsl:element name=\"a\">");
	pw.println("<xsl:attribute name=\"href\">");
	pw.println("<xsl:value-of select=\"IObjectGroupLocation\"/>");
	pw.println("</xsl:attribute>");
	pw.println("<xsl:value-of select=\"IObjectGroupName\"/>");
	pw.println("</xsl:element>");
	pw.println("&#160;");
	pw.println("<xsl:value-of select=\"NumberOfIObjects\"/> IObjects.");
	pw.println("<xsl:value-of select=\"NumberOfRepresentationIObjects\"/> Representation IObjects.");
	pw.println("<br />");

	//	pw.println("</li>");

	pw.println("</xsl:template>");
	pw.println("");
	*/
	
    }

    /**
     * Prints a reference to the Properties-XSL-file to a 
     * {@link java.io.PrintWriter}.
     */
    public void printIObjectGroupXSLReference(PrintWriter pw, String xslGroupFileName) throws IOException{
	pw.println("<?xml-stylesheet type=\"text/xsl\" href=\""+xslGroupFileName+"\"?>");
    }





}
