package infomat.view;

import infomat.algorithms.grouping.*;
import infomat.algorithms.labeling.*;
import infomat.bridges.jsdm.*;
import infomat.io.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.view.algorithms.*;
import infomat.view.tools.*;
import infomat.view.grouping.settings.*;
import infomat.view.pixel.*;
import infomat.view.sorter.*;
import infomat.view.similarity.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import mro.util.properties.dialog.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import javax.swing.*;


/**
 * 
 *
 * @version          081031
 * @author           Magnus Rosell
 */
public class GUIGroupingsForGroup{

    private IObjectGroup group;

    private Vector<IObjectGrouping> groupings;
    private HashMap<String, IObjectGrouping> nameGroupingMap;
    private IObjectGrouping currentGrouping=null;

    public GUIGroupingsForGroup(IObjectGroup group){
	this.group = group;
	groupings = new Vector<IObjectGrouping>();
	nameGroupingMap = new HashMap<String, IObjectGrouping>();

	makeStartGrouping();
	//	currentGrouping = makeStartGrouping();
	//	groupings.add(currentGrouping);
    }

    /**
     * Adds an IObjectGrouping.
     * Returns false if the name for this new grouping is already 
     * taken. 
     */
    public boolean addGrouping(IObjectGrouping cog){
	String name = cog.getName();
	if(nameGroupingMap.containsKey(name))
	    return false;
	groupings.add(cog);
	nameGroupingMap.put(name, cog);
	return true;
    }

    /**
     * Removes an IObjectGrouping.
     */
    public void removeGrouping(IObjectGrouping cog){
	groupings.remove(cog);
	nameGroupingMap.remove(cog.getName());
    }

    /**
     * Removes all IObjects that's not in the current IObjectGrouping
     * from the IObjectGroup and all its IObjectGroupings.
     */
    public void purgeToCurrentGrouping(){
	Vector<IObject> removed = new Vector<IObject>();
	Vector<IObject> ordered = group.getOrdered();
	for(int i=0; i<ordered.size(); i++){
	    IObject io = ordered.elementAt(i);
	    if(currentGrouping.getIObjectOrderIndex(io)==-1)
		removed.add(io);
	}
	for(int i=0; i<removed.size(); i++){
	    IObject io = removed.elementAt(i);
	    // remove from group
	    group.remove(io); 
	    // remove from groupings
	    for(int j=0; j<groupings.size(); j++){
		groupings.elementAt(j).remove(io);
	    }
	}
	group.sortInOrderValueOrder();
    }

    /**
     * Removed IObject:s from the IObjectGrouping:s that are not in the
     * IObjectGroup. 
     *
     */
    public void purgeGroupingsToGroup(){
	for(int j=0; j<groupings.size(); j++){
	    groupings.elementAt(j).purgeGroupingToGroup(group);
	}	
    }

    /**
     * Sets the current IObjectGrouping.
     * Returns false if the IObjectGrouping has a conflict
     * as in #addGrouping(). 
     */
    public boolean setGrouping(IObjectGrouping cog){
	boolean ret=true;
	if(cog!=null && !groupings.contains(cog))
	    ret = addGrouping(cog);
	if(ret)
	    currentGrouping=cog;
	return ret;
    }

    /**
     * Returns the current IObjectGrouping.
     */
    public IObjectGrouping getCurrentGrouping(){
	return currentGrouping;
    }

    /**
     * Returns all IObjectGrouping:s of this IObjectGroup.
     */
    public Vector<IObjectGrouping> getGroupings(){
	return groupings;
    }

    /**
     * Returns the IObjectGrouping with the name given.
     */
    public IObjectGrouping getGroupingWithName(String name){
	return nameGroupingMap.get(name);
    }

    /**
     * Returns true if the IObjectGrouping orders this IObjectGroup.
     */
    public boolean containsIObjectGrouping(IObjectGrouping ioging){
	IObjectGrouping dummy = nameGroupingMap.get(ioging.getName());
	if(dummy!=null)
	    return true;
	return false;
    }

    /**
     * Returns true if any of the IObjectGrouping:s that order this
     * IObjectGroup contains the IObjectGroup.
     */
    public boolean containsIObjectGroup(IObjectGroup iog){
	for(int i=0; i<groupings.size(); i++){
	    IObjectGrouping ioging = groupings.elementAt(i);
	    if(ioging.containsIObjectGroup(iog))
		return true;
	}
	return false;
    }


    /**
     * Makes a IObjectGrouping consisting of one IObjectGroup and
     * sets it to be the current IObjectGrouping. Used as a first 
     * IObjectGrouping as many function needs one to work.
     */
    private void makeStartGrouping(){ 
	IObjectGrouping startGrouping = 
	    new IObjectGrouping("Initial Grouping for "+group.getString());
	//	int id=-1;
	IObjectGroup startGroup = 
	    //	    new IObjectGroup("Initial Grouping Group", id); 
	    new IObjectGroup("Initial Grouping Group");
	startGrouping.add(startGroup);

	Vector<IObject> ordered = group.getOrdered();
	for(int i=ordered.size()-1; i>=0; i--){
	    IObject co = ordered.elementAt(i);
	    startGroup.add(co);
	}

	startGrouping.sort();
	int i=1;
	while(!setGrouping(startGrouping)){
	    startGrouping.setName(startGrouping.getName()+" ("+i+")");
	    i++;
	}
	//	return startGrouping;
    }




}
