package infomat.view.algorithms;

import infomat.algorithms.description.*;
import infomat.algorithms.grouping.*;
import infomat.vectorspace.*;
import infomat.view.*;

import javax.swing.*;

import java.awt.event.*;
import java.awt.*;
import java.util.*;


/**
 * A JPanel for choosing which Description Algorithm to use. 
 *
 * @see DescriptionFrame
 *
 * @version          090105
 * @author           Magnus Rosell
 */
public class DescriberSelectionPanel extends JPanel
    implements ActionListener{

    private DescriptionFrame descriptionFrame;
    private JComboBox cBox;


    private String selected;


    private static Vector<String> describerNames = null;
    private static HashMap<String, Describer> nameDescriberMap = null;
	
    private static void initAlgs(){
	if(describerNames==null){
	    describerNames = new Vector<String>();
	    nameDescriberMap = new HashMap<String, Describer>();

	    /*
	    Describer describer = new CombinedCentroidDescriber();
	    nameDescriberMap.put(describer.getName(), describer);
	    describerNames.add(describer.getName());

	    describer = new CentroidEntropyDescriber();
	    nameDescriberMap.put(describer.getName(), describer);
	    describerNames.add(describer.getName());

	    describer = new CombinedCentroidEntropyDescriber();
	    nameDescriberMap.put(describer.getName(), describer);
	    describerNames.add(describer.getName());

	    describer = new CentroidRankingBoostDescriber();
	    nameDescriberMap.put(describer.getName(), describer);
	    describerNames.add(describer.getName());
	    */

	    Describer describer = new CentroidDescriber();
	    nameDescriberMap.put(describer.getName(), describer);
	    describerNames.add(describer.getName());

	}
    }



    public DescriberSelectionPanel(DescriptionFrame descriptionFrame){
	this.descriptionFrame = descriptionFrame;
	initAlgs();
	selected = describerNames.elementAt(0);
	makeComboBox();
    }

    public boolean actionLeadToChange(){
	String newSelected = (String) cBox.getSelectedItem();
	//	System.out.println("newSelected: "+newSelected);
	if(selected != newSelected){
	    selected = newSelected;
	    return true;
	}
	return false;
    }

    public Describer getSelectedDescriber(){
	return nameDescriberMap.get(selected);
    }


    public void paintComponent(Graphics g) {
	super.paintComponent(g);
    }



    public void makeComboBox(){
	removeAll();
	cBox = new JComboBox(describerNames);
	cBox.setSelectedItem(selected);
	add(cBox);
	cBox.addActionListener(this);
	revalidate();
	repaint();
    }

    public void actionPerformed(ActionEvent e) {
	if(e.getSource() instanceof JComboBox){ //AlgorithmSelectionPanel){
	    if(actionLeadToChange()){
		Describer describer = getSelectedDescriber();
		descriptionFrame.setDescriber(describer);
	    }
	}


    }


}


