package infomat.view.algorithms;

import infomat.algorithms.grouping.*;
import infomat.algorithms.labeling.*;
import infomat.io.*;
import infomat.vectorspace.*;
import infomat.view.*;
import infomat.view.grouping.settings.*;
import infomat.vectorspace.similarity.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import mro.util.properties.dialog.*;

import java.awt.*;
import java.awt.Dialog.*;//ModalityType;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;
import java.io.*;

import javax.swing.*;


	
public class LabelFrame extends JFrame
    implements ActionListener{

    public Dimension LABELFRAME_PREFERRED_SIZE = 
	new Dimension(300, 500);
		
    //    private IMatrix matrix;
    private InfomatModel model;
    private InfomatView view;

    //    private HashMap<IObjectGroup, ILabel> labeling;		
		
    private JButton rowsOrColumns;
    private JButton apply;
    private JButton cancel;

    private JButton clusterApply;


    CentroidLabeler labeler = new CentroidLabeler();
    LabelReducer labelReducer = new LabelReducer();

		
    public LabelFrame(InfomatModel model, InfomatView view){
	super("Centroid Labeler");
	//super(model.getMainWindow().getMainFrame(), algorithm.getAlgorithmName(), true);
	this.model=model;
	this.view=view;
			
	if(view==null || model==null || model.getIMatrix() == null){
	    dispose();
	    return;
	}
			
	//setModalityType(Dialog.ModalityType.APPLICATION_MODAL);
			
	Container container = getContentPane();
	container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));
			
			
	JTextField dummy = new JTextField("Apply");
	JButton dummyButton = new JButton("Cancel");
	int height = (int) dummy.getPreferredSize().getHeight();
	int width = (int) dummyButton.getPreferredSize().getWidth();
	Dimension buttonSize = new Dimension(width, height);


	JPanel infoAndButtons = new JPanel();
	infoAndButtons.setLayout(new GridLayout(2,1));

	/*			
			JLabel description = new JLabel("Description: "+algorithm.getInfoString());
			infoAndButtons.add(description);
			JLabel info = new JLabel("Alter properties below and press \"Apply\"");
			infoAndButtons.add(info);
	*/	

			
	JPanel dimPanel = new JPanel();
	dimPanel.setLayout(new BoxLayout(dimPanel, BoxLayout.X_AXIS));
	JLabel dimInfo = new JLabel("Choose rows or columns: ");
	dimPanel.add(dimInfo);
	rowsOrColumns = new JButton("Rows");
	rowsOrColumns.setPreferredSize(buttonSize);
	rowsOrColumns.addActionListener(this);
	dimPanel.add(rowsOrColumns);
	infoAndButtons.add(dimPanel);
			
	JPanel buttonPanel = new JPanel();
	buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
	apply = new JButton("Apply");
	apply.setPreferredSize(buttonSize);
	apply.addActionListener(this);
	buttonPanel.add(apply);
	cancel = new JButton("Cancel");
	cancel.setPreferredSize(buttonSize);
	cancel.addActionListener(this);
	buttonPanel.add(cancel);
	infoAndButtons.add(buttonPanel);
	infoAndButtons.setBorder(new TitledBorder("Labeling"));
	container.add(infoAndButtons);
			

	JPanel clusterPanel = new JPanel();
	clusterPanel.setLayout(new BoxLayout(clusterPanel, BoxLayout.Y_AXIS));
	clusterApply = new JButton("Cluster");
	clusterApply.addActionListener(this);
	clusterPanel.add(clusterApply);

	clusterPanel.setBorder(new TitledBorder("Cluster"));
	container.add(clusterPanel);
		
	PropertiesPanel pp = new PropertiesPanel(labelReducer);
	container.add(pp);
	
	setVisible(true);
	setPreferredSize(LABELFRAME_PREFERRED_SIZE);
	//setSize(ALGORITHMFRAME_PREFERRED_SIZE);
	//setMinimumSize(ALGORITHMFRAME_PREFERRED_SIZE);
	pack();
    }

    private HashMap<IObjectGroup, ILabel> makeLabeling(){
	GUIIMatrix matrix = model.getGUIIMatrix();
	boolean row=true;
	if(rowsOrColumns.getText().equals("Columns"))
	    row=false;
	IObjectGrouping ioging = matrix.getCurrentColumnGrouping();
	if(row)
	    ioging = matrix.getCurrentRowGrouping();

	SparseDotSimilarity dotSim = 
	    new SparseDotSimilarity(matrix.getIMatrix(), row);
	HashMap<IObjectGroup, ILabel> labeling = 
	    labeler.labelGrouping(ioging, dotSim); //matrix);
	Labeler.printLabeling(ioging, labeling);

	//		Labeler.testLabeler(ioging, matrix);

	return labeling;
    }

		
    public void actionPerformed(ActionEvent e) {
	GUIIMatrix matrix = model.getGUIIMatrix();
	if(e.getSource() instanceof JButton){
	    JButton source = (JButton) e.getSource();
	    if(source == apply){
		ProgressFrame pf = new ProgressFrame("Labeling... ");

		HashMap<IObjectGroup, ILabel> labeling = makeLabeling();


		pf.dispose();
		/*
		  dispose();
		*/
	    }else if(source == cancel){
		dispose();
	    }else if(source == rowsOrColumns){
		if(rowsOrColumns.getText().equals("Columns"))
		    rowsOrColumns.setText("Rows");
		else if(rowsOrColumns.getText().equals("Rows"))
		    rowsOrColumns.setText("Columns");
	    
	    }else if(source == clusterApply){
		ProgressFrame pf = new ProgressFrame("Reducing");

		System.out.println("klustra");

		HashMap<IObjectGroup, ILabel> labeling = makeLabeling();

		boolean row=true;
		if(rowsOrColumns.getText().equals("Columns"))
		    row=false;
		IObjectGrouping ioging = matrix.getCurrentColumnGrouping();
		if(row)
		    ioging = matrix.getCurrentRowGrouping();
		
		SparseDotSimilarity similarity = 
		    new SparseDotSimilarity(matrix.getIMatrix(), row);
		IObjectGrouping[] iogings = labelReducer.reduceGroupingByLabeling(ioging, labeling, matrix.getIMatrix(), similarity);
		IObjectGrouping newIOGING = iogings[0];

		newIOGING.removeEmptyGroups();
		
		//		System.out.println("newIOGING: "+newIOGING.getName());
		//		System.out.println("newIOGING: "+newIOGING.numberOfIObjectGroups());
					
		view.getGroupingManager(row).putNewGrouping(newIOGING);
					
		
		//		System.out.println("Current: "+matrix.getCurrentRowGrouping().getName());
			
		view.resetImage();
		view.forceChange();
		view.getOverview().forceChange();

					
		model.setChanged();
		model.notifyObservers();

		pf.dispose();
		dispose();

	    }

				
	}
    }
		
}

	
	
