package infomat.view.algorithms;

import infomat.algorithms.grouping.*;
import infomat.algorithms.reduction.*;
import infomat.io.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.view.*;
import infomat.view.util.*;
import infomat.view.grouping.settings.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import mro.util.properties.dialog.*;

import java.awt.*;
import java.awt.Dialog.*;//ModalityType;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;
import java.io.*;

import javax.swing.*;


	
/**
 *
 * @version          080409
 * @author           Magnus Rosell
 */
public class ReductionFrame extends JFrame
    implements ActionListener{

    public Dimension REDUCTIONFRAME_PREFERRED_SIZE = 
	new Dimension(350, 600);

    private JPanel generalReducerPanel;

    private Reducer reducer;
    private RelativeReducer relativeReducer;
    private PropertiesPanel propertiesPanel;
	
    private InfomatModel model;
    private InfomatView view;
		
		
    private AlternatingButton reductionChoiceB;
    private AlternatingButton rowsOrColumns;
    private JButton apply;
    private JButton cancel;
		


    public ReductionFrame(InfomatModel model, InfomatView view){
	super("Reduction");

	this.model=model;
	this.view=view;
			
	if(model.getIMatrix()==null || view==null || model==null){
	    dispose();
	    return;
	}

	String[] redAlternatives = {"Reduction", "Relative Reduction"};
	reductionChoiceB = new AlternatingButton(redAlternatives);
	reductionChoiceB.addActionListener(this);



	String[] alternatives = {"Rows", "Columns"};
	//	AlternatingButton alt = new AlternatingButton(alternatives);
	//	rowsOrColumns = alt;
	rowsOrColumns = new AlternatingButton(alternatives);
	rowsOrColumns.addActionListener(this);

	setVisible(true);
	setPreferredSize(REDUCTIONFRAME_PREFERRED_SIZE);

	
	reducer = new Reducer();
	relativeReducer = new RelativeReducer();

	generalReducerPanel = new JPanel();
	generalReducerPanel.setPreferredSize(REDUCTIONFRAME_PREFERRED_SIZE);
	generalReducerPanel.setLayout(new BoxLayout(generalReducerPanel, BoxLayout.Y_AXIS));



	putAll();
	add(generalReducerPanel);
	pack();
    }

    public void putAll(){
	//	removeAll();
	generalReducerPanel.removeAll();
	
	Container container = getContentPane();

	JTextField dummy = new JTextField("Apply");
	JButton dummyButton = new JButton("Cancel");
	int height = (int) dummy.getPreferredSize().getHeight();
	int width = (int) dummyButton.getPreferredSize().getWidth();
	Dimension buttonSize = new Dimension(width, height);
			

	JPanel reductionChoicePanel = new JPanel();
	reductionChoicePanel.setLayout(new BoxLayout(reductionChoicePanel, BoxLayout.Y_AXIS));

	JPanel choicePanel = new JPanel();
	choicePanel.setLayout(new BoxLayout(choicePanel, BoxLayout.X_AXIS));
	JLabel choiceInfo = new JLabel("Choose reduction typ: ");
	choicePanel.add(choiceInfo);
	reductionChoiceB.setPreferredSize(buttonSize);
	choicePanel.add(reductionChoiceB);
	reductionChoicePanel.add(choicePanel);
	generalReducerPanel.add(reductionChoicePanel);

	String reductionType = reductionChoiceB.getText();


	JPanel infoAndButtons = new JPanel();
	infoAndButtons.setLayout(new GridLayout(3,1));
	JLabel info = new JLabel("Alter properties below and press \"Apply\"");
	infoAndButtons.add(info);
			
	JPanel dimPanel = new JPanel();
	dimPanel.setLayout(new BoxLayout(dimPanel, BoxLayout.X_AXIS));
	JLabel dimInfo = new JLabel("Choose rows or columns: ");
	dimPanel.add(dimInfo);

	rowsOrColumns.setPreferredSize(buttonSize);
	dimPanel.add(rowsOrColumns);
	infoAndButtons.add(dimPanel);
			
	JPanel buttonPanel = new JPanel();
	buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
	apply = new JButton("Apply");
	apply.setPreferredSize(buttonSize);
	apply.addActionListener(this);
	buttonPanel.add(apply);
	cancel = new JButton("Cancel");
	cancel.setPreferredSize(buttonSize);
	cancel.addActionListener(this);
	buttonPanel.add(cancel);
	infoAndButtons.add(buttonPanel);
	infoAndButtons.setBorder(new TitledBorder(reductionType));
	generalReducerPanel.add(infoAndButtons);
			
			
	String path = (String) model.getPropertyValue("Result Path");
	if(reductionType.equals("Reduction"))
	    propertiesPanel = new PropertiesPanel(reducer, path);
	else if(reductionType.equals("Relative Reduction"))
	    propertiesPanel = new PropertiesPanel(relativeReducer, path);

	generalReducerPanel.add(propertiesPanel);
			
	//setSize(REDUCTIONFRAME_PREFERRED_SIZE);
	//setMinimumSize(REDUCTIONFRAME_PREFERRED_SIZE);
	//	pack();

	//	add(generalReducerPanel);
	//	pack();
	generalReducerPanel.revalidate();
	generalReducerPanel.repaint();
	repaint();
    }
		
    public void actionPerformed(ActionEvent e) {

	if(e.getSource() instanceof JButton){
	    JButton source = (JButton) e.getSource();
	    if(source == rowsOrColumns){
		rowsOrColumns.press();
		generalReducerPanel.repaint();
		repaint();
		
	    }else if(source == reductionChoiceB){
		reductionChoiceB.press();
		putAll();
	    }else if(source == apply){
		
		boolean row=true;
		if(rowsOrColumns.getText().equals("Columns"))
		    row=false;
		
		String reductionType = 
		    reductionChoiceB.getText();
		
		if(reductionType.equals("Reduction")){

		    ProgressFrame pf = 
			new ProgressFrame("Reduction");
		    
		    IObjectGrouping ioging = 
			model.getGUIIMatrix().getCurrentGrouping(row);
		    
		    SparseISimilarity similarity = 
			model.getSimilarityManager().getSimilarity(row);
		    
		    IObjectGrouping clustering = 
			reducer.reduceGrouping(similarity, ioging);
		    clustering.removeEmptyGroups();
		    
		    GUIIMatrix guimatrix = model.getGUIIMatrix();
		    guimatrix.getGroupings(row).setGrouping(clustering);
		    view.getGroupingManager(row).putNewGrouping(clustering);
		    view.validateIMatrixBorders();

		    view.forceChange();
		    view.getOverview().forceChange();
		    
		    
		    model.update();
		    pf.dispose();
		    
		}else if(reductionType.equals("Relative Reduction")){
		    
		    ProgressFrame pf = 
			new ProgressFrame("Relative Reduction");
		    
		    IObjectGrouping ioging = 
			model.getGUIIMatrix().getCurrentGrouping(row);
		    IObjectGrouping relativeIOGING = 
			model.getGUIIMatrix().getCurrentGrouping(!row);
		    if(ioging.numberOfIObjectGroups() != 
		       relativeIOGING.numberOfIObjectGroups()){
			pf.dispose();
			System.out.println("Relative Reduction: Not the same number of groups in groupings!");
			return;
		    }
		    
		    
		    SparseISimilarity similarity = 
			model.getSimilarityManager().getSimilarity(row);
		    
		    IObjectGrouping clustering = 
			relativeReducer.reduceGrouping(similarity, ioging, relativeIOGING);
		    clustering.removeEmptyGroups();
		    
		    GUIIMatrix guimatrix = model.getGUIIMatrix();
		    guimatrix.getGroupings(row).setGrouping(clustering);
		    view.getGroupingManager(row).putNewGrouping(clustering);
		    view.validateIMatrixBorders();

		    view.forceChange();
		    view.getOverview().forceChange();
		    
		    
		    model.update();
		    pf.dispose();
		}
			 
	    }else if(source == cancel){
		setVisible(false);
	    }
		   
		   
	}
    }
    

    public void paint(Graphics g){
	super.paint(g);
    }
    
		
}

    
	
