package infomat.view.grouping.settings;

import java.util.*;
import infomat.view.*;
import infomat.vectorspace.*;

/**
 * One for each dimension, row and column. Keeps track of all display 
 * settings for all IObjectGroupings. 
 * Keeps track of the current GroupingColoring.
 * 
 * @author Magnus Rosell
 * @version 070511
 *
 */
public class GroupingSettingsManager {

    public boolean row;
    private InfomatView view;
    private InfomatModel model;

    private HashMap<IObjectGrouping, GroupingSettings> groupingSettingsMap = 
	new HashMap<IObjectGrouping, GroupingSettings>();

    private GroupingColoring currentColoring = null;
	
    public GroupingSettingsManager(InfomatView view, InfomatModel model, boolean row){
	this.model=model;
	this.row=row;
	this.view=view;

	makeNewSettingsForAllGroupings();
	//makeNewColoringsForAllGroupings();
    }

    public void invert(GroupingSettingsManager gm){
	boolean tmpRow = gm.row;
	InfomatView tmpView = gm.view;
	InfomatModel tmpModel = gm.model;
	HashMap<IObjectGrouping, GroupingSettings> tmpGroupingSettingsMap = gm.groupingSettingsMap;
	GroupingColoring tmpCurrentColoring = gm.currentColoring;
		
	gm.row=row;
	gm.view=view;
	gm.model=model;
	gm.groupingSettingsMap=groupingSettingsMap;
	/*
	  if(currentColoring!=null)
	  gm.setCurrentColoringByGrouping(currentColoring.getIObjectGrouping());
	  else
	  gm.currentColoring=null;
	*/
	//gm.currentColoring=currentColoring;

	row=tmpRow;
	view=tmpView;
	model=tmpModel;
	groupingSettingsMap=tmpGroupingSettingsMap;
	/*
	  if(tmpCurrentColoring!=null)
	  setCurrentColoringByGrouping(tmpCurrentColoring.getIObjectGrouping());
	  else
	  currentColoring=null;
	*/
	//currentColoring=tmpCurrentColoring;
		
	makeNewSettingsForAllGroupings();
	gm.makeNewSettingsForAllGroupings();	
    }

    public IObjectGrouping getCurrentGrouping(){
	if(row)
	    return model.getGUIIMatrix().getCurrentRowGrouping();
	return model.getGUIIMatrix().getCurrentColumnGrouping();
    }


    /**
     * For debugging.
     */
    public void print(){
	System.out.println(view.getRowGroupingManager()+", row: "+view.getRowGroupingManager().row);
	Set<IObjectGrouping> keySet = groupingSettingsMap.keySet();
	Iterator<IObjectGrouping> it = keySet.iterator();
	while(it.hasNext())
	    System.out.println(it.next().getName());

    }
	
    private void makeNewSettingsForAllGroupings(){
	groupingSettingsMap = 
	    new HashMap<IObjectGrouping, GroupingSettings>();
	
	GUIIMatrix guimatrix = model.getGUIIMatrix();
	if(guimatrix == null || guimatrix.getIMatrix() == null)
	    return;
	Vector<IObjectGrouping> vec = 
	    guimatrix.getGroupings(row).getGroupings();
	for(int i=0; i<vec.size(); i++){
	    IObjectGrouping iog = vec.elementAt(i);
	    putNewGrouping(iog);
	}


	/*		
	IMatrix m = model.getIMatrix();
	if(m==null)
	    return;
	IObjectSet ios = m.getColumnIObjectSet();
	if(row)
	    ios = m.getRowIObjectSet();
	Vector<IObjectGrouping> vec = ios.getGroupings();
	for(int i=0; i<vec.size(); i++){
	    IObjectGrouping iog = vec.elementAt(i);
	    putNewGrouping(iog);
	}
	//currentColoring=null;
	*/
    }
	
	
    public void putNewGrouping(IObjectGrouping grouping){
	GroupingSettings gs = new GroupingSettings(grouping, row);
	gs.setColoring(new GroupingColoring(grouping, row));
	groupingSettingsMap.put(grouping, gs);
	view.getGroupFrameManager().disposeAll();
    }

    public void putNewGroupingAsColoring(IObjectGrouping grouping){
	GroupingSettings gs = new GroupingSettings(grouping, row);
	gs.setColoring(new GroupingColoring(grouping, row));
	groupingSettingsMap.put(grouping, gs);
	setCurrentColoringByGrouping(grouping);
	//	view.getGroupFrameManager().disposeAll();
    }



    /*
      public void putNewColoringForGrouping(IObjectGrouping grouping){
      groupingColoringMap.put(grouping, new GroupingColoring(grouping, row));
      }
    */
	

	
    /*
      public void makeNewColoringsForAllGroupings(){
      IMatrix m = model.getIMatrix();
      if(m==null)
      return;
      IObjectSet ios = m.getColumnIObjectSet();
      if(row)
      ios = m.getRowIObjectSet();
      Vector<IObjectGrouping> vec = ios.getGroupings();
      for(int i=0; i<vec.size(); i++){
      IObjectGrouping iog = vec.elementAt(i);
      GroupingSettings gs = getSettingsForGrouping(iog);
      gs.setColoring(new GroupingColoring(iog, row));
      }
      currentColoring=null;
      }
    */
	

    public GroupingSettings getSettingsForGrouping(IObjectGrouping grouping){
	return groupingSettingsMap.get(grouping);
    }
	
	
	
	
	
    public GroupingColoring getCurrentColoring(){
	return currentColoring;
    }

    public GroupingColoring getColoringForGrouping(IObjectGrouping grouping){
	GroupingSettings gs = getSettingsForGrouping(grouping);
	if(gs==null)
	    return null;
	return gs.getColoring();
	//return groupingColoringMap.get(grouping);
    }

    public void setCurrentColoringByGrouping(IObjectGrouping grouping){
	currentColoring=getColoringForGrouping(grouping);
    }

}
