package infomat.view.menu;

import infomat.view.*;
import infomat.io.textresult.ConfusionMatrixExporter;
import infomat.vectorspace.*;

import mro.util.TracePrinter;

import java.io.*;

import java.awt.event.*;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;

/**
 * Responsible for the menu. Set it up and listen for events.
 *
 * @version          090313
 * @author           Magnus Rosell
 * 
 */
public class Menu extends JMenuBar implements ActionListener {

    private MainWindow mainWindow;

    private InfomatModel infomatModel;

    /************************************************/
    private final static String FILE = "File";
    private final static String OPENMATRIXFILE = "Open IMatrix File";
    private final static String SAVEMATRIXFILE = "Save IMatrix File";
    private final static String OPENTOKENFILETEXTWORD = "Open Token File";

    private final static String OPENRIFILE = "Open Random Index File";
    private final static String OPENDIRTEXTWORD = "Open Directory for Text-Word";
    private final static String OPENDIRWORDWORD = "Open Directory for Word-Word";


    private final static String SAVEMATRIXPICTURE = "Save IMatrix Picture";

    /*
    private final static String EXPORTCONFUSIONMATRIXASLATEXFILE = "Export Confusion Matrix as Latex File";
    private final static String RESIZE = "Resize";
    */

    private final static String PROPERTIES = "Infomat Properties";
    /*
    private final static String SAVESESSION = "Save Session";
    private final static String LOADSESSION = "Load Session";
    */
    private final static String EXIT = "Exit";

    /************************************************/
    private final static String IMAGE = "Image";

    private final static String COLOURING = "Colouring";
    private final static String GRIDCOLUMN = "Toggle Column Grid";
    private final static String GRIDROW = "Toggle Row Grid";
    private final static String GUIDE = "Toggle Guide";

    //private final static String NCOLUMNPOINT = "Select # Columns at Point";
    //sprivate final static String NROWPOINT = "Select # Rows at Point";
    private final static String REMOVE_COLUMNS = "Remove Columns";
    private final static String REMOVE_ROWS = "Remove Rows";

    private final static String PIXELSELECTION= "Pixel Selection";
    private final static String MOVE = "Drag";
    private final static String ZOOMSELECTION = "Zoom Selection";
    //    private final static String ZOOM = "Zoom";

    private final static String RESET = "Reset zoom";

    /************************************************/
    private final static String WINDOWS = "Views";
    private final static String TOOLBARONOFF = "Toolbar On/Off";
    //private final static String INFOPANELONOFF = "Info Panel On/Off";
    private final static String GROUPINGPANELONOFF = "Grouping Panel On/Off";
    private final static String OVERVIEWONOFF = "Overview On/Off";
    private final static String PIXELINFOONOFF = "Pixel View On/Off";
    private final static String SELECTORONOFF = "Selection View On/Off";
    private final static String MOUSEPOINTERINFO = "Mouse Pointer Info On/Off";
    private final static String SIMILARITYONOFF = "Similarity View On/Off";

    /************************************************/
    private final static String TOOLS = "Tools";
    private final static String INVERTMATRIX = "Transpose Matrix";
    private final static String PURGEMATRIX = "Purge Matrix";
    private final static String EVALUATION = "Evaluation";
    private final static String MATRIXSUMMARY = "Matrix Summary";	
    private final static String EXPORTTEXT = "Export to Text";	
    private final static String STOPLIST = "Stoplist";	
    private final static String SEARCH = "Search";	
	
    /************************************************/
    private final static String ALGORITHMS = "Algorithms";
    private final static String CLUSTERINGALGORITHMS = "Clustering Algorithms";
    private final static String SORTER = "Cluster Sorter";
    private final static String FILTERMATRIX = "Filter Matrix";
    private final static String WEIGHTMATRIX = "Weight Matrix";
    private final static String DESCRIPTION = "Description";
    private final static String REDUCTION = "Reduction";

    //    private final static String LABELER = "Labeler";
	
    /************************************************/
    private final static String HELP = "Help";
    private final static String ABOUT = "About";
    private final static String MANUAL = "Manual";


    private ToolbarController toolbarController;

    private InfomatView infomatView;

    private JFileChooser fileChooser;
    private JFileChooser directoryChooser;
    private JFileChooser imgFileChooser;

    private String fileChooserPath=""; //"E:/Java/Infomat/example/"; //"/var/tmp/rosell/Result/xml/";

    private boolean printOn=true;
    private TracePrinter tp = new TracePrinter(printOn);


    /**
     * Create the menu.
     * 
     * @param frame
     *            The frame to dispose on exit.
     */
    public Menu(MainWindow frame, InfomatModel infoModel,
		ToolbarController toolbarController, InfomatView infomatView) {
	this.toolbarController = toolbarController;
	this.infomatView = infomatView;
	infomatModel = infoModel;
	mainWindow = frame;

	fileChooserPath = (String) infomatModel.getPropertyValue("Matrix File Path");
	fileChooser = new JFileChooser(fileChooserPath);
	fileChooser.setFileFilter(new FileFilter() {
		public String getDescription() {
		    return "XML files";
		}

		public boolean accept(File f) {
		    if (f.isDirectory())
			return true;
		    int findresult = f.getName().indexOf(".xml");
		    int len = f.getName().length();
		    return findresult != -1 && findresult == len - 4;
		}

	    });

	directoryChooser = new JFileChooser(fileChooserPath);
	directoryChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	directoryChooser.setAcceptAllFileFilterUsed(false);

	fileChooserPath = (String) infomatModel.getPropertyValue("Result Path");
	imgFileChooser = new JFileChooser(fileChooserPath);
	imgFileChooser.setFileFilter(new FileFilter() {
		public String getDescription() {
		    return "PNG files";
		}

		public boolean accept(File f) {
		    if (f.isDirectory())
			return true;
		    int findresult = f.getName().indexOf(".png");
		    int len = f.getName().length();
		    return findresult != -1 && findresult == len - 4;
		}

	    });


	JMenu menu;
	JMenuItem menuItem;

	/************************************************/		
	// File menu
	menu = new JMenu(FILE);
	menu.setMnemonic(KeyEvent.VK_F);
	add(menu);

	menuItem = new JMenuItem(OPENMATRIXFILE, KeyEvent.VK_O);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem(SAVEMATRIXFILE, KeyEvent.VK_O);
	menuItem.addActionListener(this);
	menu.add(menuItem);
		
	menu.addSeparator();

	menuItem = new JMenuItem(OPENTOKENFILETEXTWORD, KeyEvent.VK_O);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menu.addSeparator();

	JMenu otherMenu = new JMenu("Other");

	menuItem = new JMenuItem(OPENDIRTEXTWORD, KeyEvent.VK_O);
	menuItem.addActionListener(this);
	otherMenu.add(menuItem);

	menuItem = new JMenuItem(OPENDIRWORDWORD, KeyEvent.VK_O);
	menuItem.addActionListener(this);
	otherMenu.add(menuItem);

	menuItem = new JMenuItem(OPENRIFILE, KeyEvent.VK_O);
	menuItem.addActionListener(this);
	otherMenu.add(menuItem);

	menu.add(otherMenu);

	menu.addSeparator();


	menuItem = new JMenuItem(SAVEMATRIXPICTURE, KeyEvent.VK_P);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menu.addSeparator();

	/*
	menuItem = new JMenuItem(EXPORTCONFUSIONMATRIXASLATEXFILE, KeyEvent.VK_P);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menu.addSeparator();

	menuItem = new JMenuItem(RESIZE, KeyEvent.VK_P);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menu.addSeparator();
	*/


	menuItem = new JMenuItem(PROPERTIES, KeyEvent.VK_P);
	menuItem.addActionListener(this);
	menu.add(menuItem);


		

		
	menu.addSeparator();

	menuItem = new JMenuItem(EXIT, KeyEvent.VK_X);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	/************************************************/
	// Image menu
	menu = new JMenu(IMAGE);
	menu.setMnemonic(KeyEvent.VK_I);
	add(menu);

	/*
	  menuItem = new JMenuItem(TOOLBARONOFF, KeyEvent.VK_T);
	  menuItem.addActionListener(this);
	  menu.add(menuItem);
	*/
	/*
	  menuItem = new JMenuItem(COLOURING, KeyEvent.VK_C);
	  menuItem.addActionListener(this);
	  menuItem.setEnabled(false);
	  menu.add(menuItem);
	  menu.addSeparator();
	*/

	menuItem = new JMenuItem(PIXELSELECTION, KeyEvent.VK_M);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem(MOVE, KeyEvent.VK_M);
	menuItem.addActionListener(this);
	menu.add(menuItem);


	menuItem = new JMenuItem(ZOOMSELECTION, KeyEvent.VK_S);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem(REMOVE_COLUMNS, KeyEvent.VK_E);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem(REMOVE_ROWS, KeyEvent.VK_M);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	/*
	menuItem = new JMenuItem(ZOOM, KeyEvent.VK_Z);
	menuItem.addActionListener(this);
	menu.add(menuItem);
	*/

	menu.addSeparator();

	menuItem = new JMenuItem(GRIDCOLUMN, KeyEvent.VK_W);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem(GRIDROW, KeyEvent.VK_D);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menu.addSeparator();

	menuItem = new JMenuItem(GUIDE, KeyEvent.VK_S);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem(RESET, KeyEvent.VK_R);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	/************************************************/
	// Panel menu
	menu = new JMenu(WINDOWS);
	menu.setMnemonic(KeyEvent.VK_I);
	add(menu);

	menuItem = new JMenuItem(PIXELINFOONOFF, KeyEvent.VK_T);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem(SELECTORONOFF, KeyEvent.VK_T);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem(SIMILARITYONOFF, KeyEvent.VK_T);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem(OVERVIEWONOFF, KeyEvent.VK_T);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem(GROUPINGPANELONOFF, KeyEvent.VK_T);
	menuItem.addActionListener(this);
	menu.add(menuItem);
		
	menuItem = new JMenuItem(TOOLBARONOFF, KeyEvent.VK_T);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem(MOUSEPOINTERINFO, KeyEvent.VK_T);
	menuItem.addActionListener(this);
	menu.add(menuItem);



	/*
	  menuItem = new JMenuItem(INFOPANELONOFF, KeyEvent.VK_T);
	  menuItem.addActionListener(this);
	  menu.add(menuItem);
	*/
		
		
		
	/************************************************/
	// Tools menu
	menu = new JMenu(TOOLS);
	menu.setMnemonic(KeyEvent.VK_H);
	add(menu);

	menuItem = new JMenuItem(EVALUATION, KeyEvent.VK_A);
	menuItem.addActionListener(this);
	menu.add(menuItem);
		
	menuItem = new JMenuItem(MATRIXSUMMARY, KeyEvent.VK_A);
	menuItem.addActionListener(this);
	menu.add(menuItem);
		
	menuItem = new JMenuItem(EXPORTTEXT, KeyEvent.VK_A);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem(SEARCH, KeyEvent.VK_A);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem(STOPLIST, KeyEvent.VK_A);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem(INVERTMATRIX, KeyEvent.VK_A);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem(PURGEMATRIX, KeyEvent.VK_A);
	menuItem.addActionListener(this);
	menu.add(menuItem);
		
	/************************************************/
	// Algorithm menu
	menu = new JMenu(ALGORITHMS);
	menu.setMnemonic(KeyEvent.VK_H);
	add(menu);

	menuItem = new JMenuItem(CLUSTERINGALGORITHMS, KeyEvent.VK_A);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem(FILTERMATRIX, KeyEvent.VK_A);
	menuItem.addActionListener(this);
	menu.add(menuItem);
		
	menuItem = new JMenuItem(WEIGHTMATRIX, KeyEvent.VK_A);
	menuItem.addActionListener(this);
	menu.add(menuItem);
		
	menuItem = new JMenuItem(SORTER, KeyEvent.VK_A);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem(DESCRIPTION, KeyEvent.VK_A);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem(REDUCTION, KeyEvent.VK_A);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	/*
	menuItem = new JMenuItem(LABELER, KeyEvent.VK_A);
	menuItem.addActionListener(this);
	menu.add(menuItem);
	*/

		
	/************************************************/
	// Help menu
	menu = new JMenu(HELP);
	menu.setMnemonic(KeyEvent.VK_H);
	add(menu);

	menuItem = new JMenuItem(ABOUT, KeyEvent.VK_A);
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menuItem = new JMenuItem(MANUAL, KeyEvent.VK_M);
	menuItem.addActionListener(this);
	menu.add(menuItem);
    }

    public void actionPerformed(ActionEvent e) {
	JMenuItem source = (JMenuItem) (e.getSource());
	String cmd = source.getText();
	String s = "Menu action event detected." + "    Event source: "
	    + source.getText();
	tp.println(s);

	/************************************************/
	// File menu
	if (cmd.equals(EXIT)) {
	    mainWindow.getMainFrame().dispose();
	    //infomatView.disposeOverviewFrame();
	} else if (cmd.equals(OPENTOKENFILETEXTWORD)) {
	    fileChooser.setSelectedFile(new File(""));
	    if (fileChooser.showOpenDialog(mainWindow.getSessionPanel()) == JFileChooser.APPROVE_OPTION) {
		infomatModel.openTokenFileForTextWord(fileChooser.getSelectedFile());
		infomatView.forceChange();
		tp.println("ppnat!!!");
	    }
	} else if (cmd.equals(OPENMATRIXFILE)) {
	    fileChooser.setSelectedFile(new File(""));
	    if (fileChooser.showOpenDialog(mainWindow.getSessionPanel()) == JFileChooser.APPROVE_OPTION) {
		infomatModel.openIMatrixFile(fileChooser.getSelectedFile());
		tp.println("ppnat!!!");
	    }
	} else if (cmd.equals(SAVEMATRIXFILE)) {
	    fileChooser.setSelectedFile(new File(""));
	    if (fileChooser.showSaveDialog(mainWindow.getSessionPanel()) == JFileChooser.APPROVE_OPTION) {
		infomatModel.saveIMatrixFile(fileChooser.getSelectedFile());
		tp.println("Sparat!!!");
	    }

	} else if (cmd.equals(OPENDIRTEXTWORD)) {
	    directoryChooser.setSelectedFile(new File(""));
	    if (directoryChooser.showOpenDialog(mainWindow.getSessionPanel()) == JFileChooser.APPROVE_OPTION) {

		infomatModel.openDirectoryTextWord(directoryChooser.getSelectedFile());
		tp.println("Opened!!!");
	    }

	} else if (cmd.equals(OPENDIRWORDWORD)) {
	    directoryChooser.setSelectedFile(new File(""));
	    if (directoryChooser.showOpenDialog(mainWindow.getSessionPanel()) == JFileChooser.APPROVE_OPTION) {

		infomatModel.openDirectoryWordWord(directoryChooser.getSelectedFile());
		tp.println("Opened!!!");
	    }

	} else if (cmd.equals(OPENRIFILE)) {
	    fileChooser.setSelectedFile(new File(""));
	    if (fileChooser.showOpenDialog(mainWindow.getSessionPanel()) == JFileChooser.APPROVE_OPTION) {
		infomatModel.openRIFile(fileChooser.getSelectedFile());
		tp.println("Opened!!!");
	    }

	} else if (cmd.equals(SAVEMATRIXPICTURE)) {
	    imgFileChooser.setSelectedFile(new File(""));
	    if (imgFileChooser.showSaveDialog(mainWindow.getSessionPanel()) == JFileChooser.APPROVE_OPTION) {
		tp.println("SAVEMATRIXPICTURE!!!");
		tp.println(imgFileChooser.getSelectedFile().getName());
		infomatModel.saveIMatrixPicture(imgFileChooser.getSelectedFile());
	    }
	/*
	} else if (cmd.equals(EXPORTCONFUSIONMATRIXASLATEXFILE)) {
	    tp.println("EXPORTCONFUSIONMATRIXASLATEXFILE!!!");
	    String fileName = (String) infomatModel.getPropertyValue("Result Path");
	    fileName += File.separator+"confusionMatrix.tex";
	    File file = new File(fileName);

	    GUIIMatrix gm = infomatModel.getGUIIMatrix();
	    GUIGroupingsForGroup gg = gm.getRowGroupings();
	    

	    IObjectGrouping grouping = 
		infomatModel.getCurrentGrouping(true);
	    IObjectGrouping reference = 
		gg.getGroupingWithName("Location Grouping 1");


	    try{
		ConfusionMatrixExporter.save(file, grouping, reference);
		//		ConfusionMatrixExporter.save(file, reference, grouping);
	    }catch(IOException ioe){
		// Ouch!
	    }

	}else if (cmd.equals(RESIZE)) {
	    tp.println("RESIZE!!!");
	    infomatModel.resize();
	*/
       	}else if (cmd.equals(PROPERTIES)) {
	    tp.println("Properties!!!");
	    infomatModel.showProperties();
	} 

	/************************************************/
	// Image menu
	else if (cmd.equals(RESET)) {
	    tp.println("Reset image position and zoom");
	    infomatModel.resetImage();
	} else if (cmd.equals(PIXELSELECTION)) {
	    tp.println("Click the hand");
	    toolbarController.menuClick(ToolbarController.PIXELSELECTION);
	} else if (cmd.equals(MOVE)) {
	    tp.println("Click the hand");
	    toolbarController.menuClick(ToolbarController.HAND);
	} else if (cmd.equals(ZOOMSELECTION)) {
	    tp.println("Click the zoomselection");
	    toolbarController.menuClick(ToolbarController.ZOOMSELECTION);
	    /*
	} else if (cmd.equals(ZOOM)) {
	    tp.println("Click the zoom");
	    toolbarController.menuClick(ToolbarController.ZOOMIN);
	    */
	} else if (cmd.equals(GRIDCOLUMN)) {
	    tp.println("Click the word grid");
	    toolbarController.menuClick(ToolbarController.GRIDCOLUMN);
	} else if (cmd.equals(GRIDROW)) {
	    tp.println("Click the document grid");
	    toolbarController.menuClick(ToolbarController.GRIDROW);
	} else if (cmd.equals(GUIDE)) {
	    infomatModel.toggleGuide();
	} else if (cmd.equals(REMOVE_COLUMNS)) {
	    toolbarController.menuClick(ToolbarController.REMOVE_COLUMNS);
	} else if (cmd.equals(REMOVE_ROWS)) {
	    toolbarController.menuClick(ToolbarController.REMOVE_ROWS);
	} 
		
	/************************************************/
	// Panel menu
	else if (cmd.equals(TOOLBARONOFF)) {
	    mainWindow.getSessionPanel().toggleToolbar();
	    //}else if (cmd.equals(INFOPANELONOFF)) {
	    //	mainWindow.getSessionPanel().toggleInfoPanel();
	}else if (cmd.equals(GROUPINGPANELONOFF)) {
	    mainWindow.getSessionPanel().toggleGroupingPanel();
	}else if (cmd.equals(PIXELINFOONOFF)) {
	    //	    infomatModel.pixelFrame();
	    infomatModel.togglePixelFrame();
	}else if (cmd.equals(SELECTORONOFF)) {
	    infomatModel.toggleSelectorFrame();
	}else if (cmd.equals(SIMILARITYONOFF)) {
	    infomatModel.toggleSimilarityFrame();
	}else if (cmd.equals(OVERVIEWONOFF)) {
	    mainWindow.getSessionPanel().toggleOverview();
	}else if (cmd.equals(MOUSEPOINTERINFO)){
	    mainWindow.getSessionPanel().toggleMousePointerInfoPanel();
	}

		
	/************************************************/
	// Tools menu
	else if (cmd.equals(INVERTMATRIX)) {
	    tp.println("INVERT MATRIX");
	    infomatModel.invertMatrix();
	} else if (cmd.equals(PURGEMATRIX)) {
	    tp.println("PURGE MATRIX");
	    infomatModel.purgeMatrix();
	} else if (cmd.equals(EVALUATION)) {
	    tp.println("EVALUATION");
	    infomatModel.evaluation();
	} else if (cmd.equals(MATRIXSUMMARY)) {
	    tp.println("Matrix Summary");
	    infomatModel.matrixSummary();
	} else if (cmd.equals(EXPORTTEXT)) {
	    tp.println("Export to Text");
	    infomatModel.exportText();
	} else if (cmd.equals(STOPLIST)) {
	    tp.println("Stoplist");
	    infomatModel.stoplist();
	} else if (cmd.equals(SEARCH)) {
	    tp.println("Search");
	    infomatModel.search();
	}
		
		
	/************************************************/
	// Algorithm menu
	else if (cmd.equals(CLUSTERINGALGORITHMS)) {
	    infomatModel.cluster();
	} else if (cmd.equals(SORTER)) {
	    tp.println("Cluster Sorter");
	    infomatModel.sorter();
	} else if (cmd.equals(FILTERMATRIX)) {
	    tp.println("FILTER MATRIX");
	    infomatModel.filterMatrix();
	} else if (cmd.equals(WEIGHTMATRIX)) {
	    tp.println("WEIGHT MATRIX");
	    infomatModel.weightMatrix();
	} else if (cmd.equals(DESCRIPTION)){
	    tp.println("DESCRIPTION");
	    infomatModel.description();
	} else if (cmd.equals(REDUCTION)){
	    tp.println("REDUCTION");
	    infomatModel.reduction();
	}


	/*
	}else if (cmd.equals(LABELER)) {
	    infomatModel.labeler();
	}
	*/
		
	/************************************************/
	// Help menu
	else if (cmd.equals(ABOUT)) {
	    JOptionPane
		.showMessageDialog(
				   this,
				   "Infomat "
				   + " - A Vector Space Visualization Tool.\n"
				   + "(version: "+infomat.InfomatGUI.VERSION+")\n\n" //MainWindow.VERSION
+"(C) 2009 Magnus Rosell, KTH CSC",
				   "About Infomat",
				   JOptionPane.INFORMATION_MESSAGE);
	} else if (cmd.equals(MANUAL)) {
	    JOptionPane.showMessageDialog(this, "The manual is a pdf-document in the Infomat directory.",
					  "Infomat Interface Manual",
					  JOptionPane.INFORMATION_MESSAGE);
	} 
		
	/************************************************/
	else {
	    System.err.println("Menu action not definied! " + source.getText());
	}
    }
}
