package mro.util.doublehashmap;

import java.util.*; 

/**
 * A DoubleHashMap has two indexes. 
 * There is only one copy of each index value.
 *
 * @version          080311
 * @author           Magnus Rosell
 */
public class DoubleHashMap{

    // The indexes are only saved one time
    /*
      private HashMap firstIndexMap = new HashMap(); 
      private HashMap secondIndexMap = new HashMap();
    */
    private HashSet firstIndexSet = new HashSet();
    private HashSet secondIndexSet = new HashSet();
	
    // The Map, i.e. the first HashMap, which contains several HashMaps,
    // HashMap:s for the second index.
    private HashMap firstMap = new HashMap();

    // The alternate view.
    // The second HashMap, which contains several HashMaps,
    // HashMap:s for the first index.
    private HashMap secondMap = new HashMap();

    // The number of Objects in the DoubleHashMap
    private int size=0;


    /**
     * Adds a Object "value" indexed with the Object:s
     * "first" and "second"
     * @param first
     * @param second
     * @param value
     */
    public void put(Object first, Object second, Object value){
	/*
	  first = getIndexObject(first, firstIndexMap);
	  second = getIndexObject(second, secondIndexMap);
	*/
	firstIndexSet.add(first);
	secondIndexSet.add(second);
		
	putIntoMap(first, second, value, firstMap);
	putIntoMap(second, first, value, secondMap);

	/*
	HashMap secondMap = (HashMap) firstMap.get(first);
	if(secondMap==null){
	    secondMap = new HashMap();
	    firstMap.put(first, secondMap);
	}
	secondMap.put(second, value);
	*/

	size++;
    }

    /**
     * Convenience method, for putting objects into a HashMap of HashMap:s.
     */
    private void putIntoMap(Object o1, Object o2, Object value, HashMap map){
	HashMap map2 = (HashMap) map.get(o1);
	if(map2==null){
	    map2 = new HashMap();
	    map.put(o1, map2);
	}
	map2.put(o2, value);
    }


    /**
     * Returns the Object that was indexed with "first" and "second".
     * @param first
     * @param second
     * @return an Object
     */
    public Object get(Object first, Object second){
	HashMap secondMap = (HashMap) firstMap.get(first);
	if(secondMap==null)
	    return null;
	Object value = secondMap.get(second);
	if(value==null)
	    return null;
	return value;
    }

    /**
     * Removes the Object with the indexes.
     * @param first
     * @param second
     * @return an Object
     */
    public Object remove(Object first, Object second){

	// The following two must be the same.
	// No check right now. It should not be possible
	// for it to go wrong.

	boolean removed = false;
	Object value = removeFromMap(first, second, firstMap);
	if(value!=null)
	    removed = true;
	value = removeFromMap(second, first, secondMap);
	if(value!=null)
	    removed = true;

	if(removed)
	    size--;

	if(firstMap.get(first)==null)
	    firstIndexSet.remove(first);
	
	if(secondMap.get(second)==null)
	    secondIndexSet.remove(second);

	/*
	HashMap secondMap = (HashMap) firstMap.get(first);
	if(secondMap==null)
	    return null;
	Object value = secondMap.remove(second);
	if(value==null)
	    return null;
	if(secondMap.size()==0)
	    firstMap.remove(secondMap);
	*/
	return value;
    }

    /**
     * Convenience method, for removing objects in a HashMap of HashMap:s.
     */
    private Object removeFromMap(Object o1, Object o2, HashMap map){
	HashMap map2 = (HashMap) map.get(o1);
	if(map2==null)
	    return null;
	Object value = map2.remove(o2);
	if(value==null)
	    return null;
	if(map2.size()==0)
	    map.remove(map2);
	return value;
    }

    /**
     * Removes all Objects with the specified "first" index.
     * @param first
     */
    public void removeAllFirst(Object first){
	HashMap removed = (HashMap) firstMap.remove(first);
	size-=removed.size();

	Iterator it = secondMap.values().iterator();
	while(it.hasNext()){
	    HashMap tempFirstMap = (HashMap) it.next();
	    tempFirstMap.remove(first);
	}

	firstIndexSet.remove(first);
    }

    /**
     * Removes all Objects with the specified "second" index.
     * @param second
     */
    public void removeAllSecond(Object second){
	HashMap removed = (HashMap) secondMap.remove(second);
	size-=removed.size();

	Iterator it = firstMap.values().iterator();
	while(it.hasNext()){
	    HashMap tempSecondMap = (HashMap) it.next();
	    tempSecondMap.remove(second);
	}

	secondIndexSet.remove(second);
    }


    /**
     * Returns all Objects in the DoubleHashMap as a Vector.
     * An object is returned for each mapping, meaning that
     * an object can appear at several positions in the Vector.
     * @return a Vector
     */
    public Vector values(){
	Vector ret = new Vector();
	Iterator it = firstMap.values().iterator();
	while(it.hasNext()){
	    HashMap secondMap = (HashMap) it.next();
	    ret.addAll(secondMap.values());
	}
	return ret;
    }
    /*
    public Set values(){
	HashSet ret = new HashSet();
	Iterator it = firstMap.values().iterator();
	while(it.hasNext()){
	    HashMap secondMap = (HashMap) it.next();
	    HashSet set = new HashSet(secondMap.values());
	    ret.addAll(set);
	}
	return ret;
    }
    */


    /**
     * Returns a Vector of those value Objects that are 
     * stored with the specified "first" index.
     * @param first
     * @return a Vector
     */	
    public Vector valuesForFirstIndex(Object first){
	HashMap second = (HashMap) firstMap.get(first);
	if(second==null)
	    //	    return null;
	    return new Vector();
	return new Vector(second.values());
    }
    /*
    public Set valuesForFirstIndex(Object first){
	HashMap second = (HashMap) firstMap.get(first);
	if(second==null)
	    return null;
	return new HashSet(second.values());
    }
    */

    /**
     * Returns a Vector of those value Objects that are 
     * stored with the specified "second" index.
     * @param second
     * @return a Vector
     */	
    public Vector valuesForSecondIndex(Object second){
	HashMap first = (HashMap) secondMap.get(second);
	if(first==null)
	    //	    return null;
	    return new Vector();
	return new Vector(first.values());
    }
    /*
    public Set valuesForSecondIndex(Object second){
	HashMap first = (HashMap) secondMap.get(second);
	if(first==null)
	    return null;
	return new HashSet(first.values());
    }
    */

    /**
     * Inverts the mapping, so the fist index becomes the second.
     */
    public void invert(){
	HashSet tmpSet = firstIndexSet;
	firstIndexSet = secondIndexSet;
	secondIndexSet = tmpSet;

	HashMap tmpMap = firstMap;
	firstMap = secondMap;
	secondMap = tmpMap;
    }

    public int size(){
	return size;
    }

    public int sizeOfFirstIndex(){
	return firstIndexSet.size();
    }

    public int sizeOfSecondIndex(){
	return secondIndexSet.size();
    }

    public int sizeOfSecondIndexForElementInFirst(Object first){
	HashMap second = (HashMap) firstMap.get(first);
	if(second==null)
	    return 0;
	return second.size();
    }

    public Set keySetOfSecondIndexForElementInFirst(Object first){
	HashMap second = (HashMap) firstMap.get(first);
	if(second==null)
	    return null;
	return second.keySet();
    }

    /*
      private Object getIndexObject(Object index, HashMap hMap){
      Object i = hMap.get(index);
      if(i==null)
      hMap.put(index, index);
      else
      index=i;
      return index;
      }
    */


}
