package mro.util.experimentation;

import mro.util.*;
import mro.util.doublehashmap.*;

import java.io.*;
import java.util.*; 

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.SAXException;


/**
 * 
 *
 * @version          080513
 * @author           Magnus Rosell
 */

public class MeasureAbbreviationTable{

    private HashMap<String, Vector<String>> abbreviationMap = 
	new HashMap<String, Vector<String>>();
    
    public void add(String abbreviation, Vector<String> measureNames){
	abbreviationMap.put(abbreviation, measureNames);
    }

    public Vector<String> get(String abbreviation){
	return abbreviationMap.get(abbreviation);
    }


    /********* IO *********/

    /**
     * Saves a Table-object to a file with the specified file name.
     */
    public void save(String fileName) throws IOException{
	save(new File(fileName));
    }

    /**
     * Saves a Table-object to the specified file.
     */
    public void save(File file) throws IOException{
	//	try {
	    File dir = file.getParentFile();
	    if(!dir.exists())
		dir.mkdirs();

	    // XSL
	    String xslFileName = file.getParent()+File.separator+
		"MeasureAbbreviationTable.xsl";

	    PrintWriter out = new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(xslFileName))));
	    printXSL(out);
	    out.close();

	    // XML
	    out = new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(file))));

	    //            printXMLHead(out);
	    printXML(out);
	    out.close();
	    /*
	}
	catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
	    */
    }

    /**
     * Prints the complete DTD of a Table-XML-file to a 
     * {@link java.io.PrintWriter}.
     */
    public static void printDTD(PrintWriter pw) throws IOException{
	pw.println("<!DOCTYPE measureabbrevationtable [");
	printPartDTD(pw);

	pw.println("]>");
    }

    /**
     * Prints the DTD of a Table-XML-file to a 
     * {@link java.io.PrintWriter}. Only the elements.
     * This method may be used when the Table-XML should
     * be part of a larger XML.
     */
    public static void printPartDTD(PrintWriter pw) throws IOException{
	pw.println("<!--A measure abbreviation table-->");
	pw.println("<!ELEMENT measureabbrevationtable (mae*)>");

	pw.println("<!--A measureabbrevationtable entry-->");
	pw.println("<!ELEMENT mae (abb, n+)>");

	pw.println("<!--The abbreviation-->");
	pw.println("<!ELEMENT abb (#PCDATA)>");

	pw.println("<!--A measure name-->");
	pw.println("<!ELEMENT n (#PCDATA)>");

    }

    /**
     * Prints a reference to the Table-XSL-file to a 
     * {@link java.io.PrintWriter}.
     */
    public static void printTableXSLReference(PrintWriter pw) throws IOException{
	pw.println("<?xml-stylesheet type=\"text/xsl\" href=\"MeasureAbbreviationTable.xsl\"?>");
    }

    /**
     * Prints the Table to a 
     * {@link java.io.PrintWriter}.
     */
    public void printXML(PrintWriter pw) throws IOException{
	XMLHelper.printXMLHead(pw);
	printTableXSLReference(pw); 
	printDTD(pw);
	printPartXML(pw);
    }

    public void printPartXML(PrintWriter pw) throws IOException{
	pw.println("<measureabbreviationtable>");
	/*
abbreviationMap
	for(int i=0; i<relativeVector.size(); i++)
	    relativeVector.elementAt(i).printPartXML(pw);
	*/
	pw.println("</measureabbreviationtable>");
   }


    /**
     * Prints a basic XSL-file for Table XML-files to a 
     * {@link java.io.PrintWriter}.
     */
    public void printXSL(PrintWriter pw) throws IOException{

	/*
	pw.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
	pw.println("<xsl:stylesheet version=\"1.0\"");
	pw.println("xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\">");
	pw.println("<xsl:template match=\"/\">");
	pw.println("<html>");
	pw.println("<body>");
	pw.println("<table border=\"1\">");

	pw.println("<xsl:apply-templates/>");

	pw.println("</table>");
	pw.println("</body>");
	pw.println("</html>");
	pw.println("</xsl:template>");


	pw.println("<xsl:template match=\"ptr\">");
	pw.println("<tr>");
	pw.println("<xsl:for-each select=\"ptc\">");
	pw.print("<td><xsl:value-of select=\".\"/></td>");
	pw.println("\n</xsl:for-each>");
	pw.println("</tr>");
	pw.println("</xsl:template>");

	pw.println("</xsl:stylesheet>");
	*/
    }

}

