package mro.util.experimentation;

import mro.util.*;
import mro.util.doublehashmap.*;

import java.io.*;
import java.util.*; 

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.SAXException;


/**
 * A class for handling xml tables.
 *
 * @version          080505
 * @author           Magnus Rosell
 */

public class ResultTableStructure{

    private String caption;
    private MeasureAbbreviationTable measureAbbreviationTable = null;
    private Table<String> columnTitles = null;
    private Table<String> rowTitles = null;
    private PathTable pathTable = new PathTable();
    private RelativeTable relativeTable = null;

    protected void setCaption(String caption){
	this.caption = caption;
    }

    protected void setCommonPath(String commonPath){
	pathTable.setCommonPath(commonPath);
    }

    protected void setColumnTitles(Table<String> columnTitles){
	this.columnTitles = columnTitles;
    }

    protected void setRowTitles(Table<String> rowTitles){
	this.rowTitles = rowTitles;
    }

    protected void setPaths(PathTable pathTable){
	this.pathTable = pathTable;
    }

    public void setRelativeTable(RelativeTable relativeTable){
	this.relativeTable=relativeTable;
    }

    public void setMeasureAbbreviationTable(MeasureAbbreviationTable 
					    measureAbbreviationTable){
	this.measureAbbreviationTable = measureAbbreviationTable;
    }

    public String getCaption(){
	return caption;
    }

    public String getCommonPath(){
	return pathTable.getCommonPath();
    }

    public Table<String> getColumnTitles(){
	return columnTitles;
    }

    public Table<String> getRowTitles(){
	return rowTitles;
    }

    public PathTable getPathTable(){
	return pathTable;
    }

    public RelativeTable getRelativeTable(){
	return relativeTable;
    }

    public MeasureAbbreviationTable getMeasureAbbreviationTable(){
	return measureAbbreviationTable;
    }



    /********* IO *********/

    /**
     * Saves a ResultTableStructure-object to a file with the specified file name.
     */
    public void save(String fileName) throws IOException{
	save(new File(fileName));
    }

    /**
     * Saves a ResultTableStructure-object to the specified file.
     */
    public void save(File file) throws IOException{
	//	try {
	    File dir = file.getParentFile();
	    if(!dir.exists())
		dir.mkdirs();

	    // XSL
	    String xslFileName = file.getParent()+File.separator+
		"ResultTableStructure.xsl";

	    PrintWriter out = new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(xslFileName))));
	    printXSL(out);
	    out.close();

	    // XML
	    out = new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(file))));

	    //            printXMLHead(out);
	    printXML(out);
	    out.close();
	    /*
	}
	catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
	    */
    }

    /**
     * Prints the complete DTD of a ResultTableStructure-XML-file to a 
     * {@link java.io.PrintWriter}.
     */
    public static void printDTD(PrintWriter pw) throws IOException{
	pw.println("<!DOCTYPE ResultTableStructure [");
	pw.println("<!ELEMENT ResultTableStructure (measureabbreviationtable, columntitles?, rowtitles?, pathtable, relativetable?)>");
	MeasureAbbreviationTable.printPartDTD(pw);
	pw.println("<!--The column titles-->");
	pw.println("<!ELEMENT columntitles (table)>");
	pw.println("<!--The row titles-->");
	pw.println("<!ELEMENT rowtitles (table)>");

	//	pw.println("<!--The paths-->");
	//	pw.println("<!ELEMENT paths (table)>");
	PathTable.printPartDTD(pw);

	Table.printPartDTD(pw);

	RelativeTable.printPartDTD(pw);

	pw.println("]>");
    }


    /**
     * Prints a reference to the ResultTableStructure-XSL-file to a 
     * {@link java.io.PrintWriter}.
     */
    public static void printResultTableStructureXSLReference(PrintWriter pw) throws IOException{
	pw.println("<?xml-stylesheet type=\"text/xsl\" href=\"ResultTableStructure.xsl\"?>");
    }

    private static void badInput(){
	System.out.println("Bad input!");
	System.exit(0);
    }

    public static void main(String[] args) throws IOException{
	if(args.length!=2){
	    badInput();
	}
	
	String fileName = args[1];
	File file = new File(fileName);
	PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(file)));
	if(args[0].equals("-dtd")){
	    printDTD(pw);
	    pw.flush();
	    pw.close();
	//	}else if(args[0].equals("")){
	//	    printXSL(pw);
	}else
	    badInput();
    }


    /**
     * Prints the ResultTableStructure to a 
     * {@link java.io.PrintWriter}.
     */
    public void printXML(PrintWriter pw) throws IOException{
	XMLHelper.printXMLHead(pw);
	printResultTableStructureXSLReference(pw); 
	printDTD(pw);
	printPartXML(pw);
    }

    public void printPartXML(PrintWriter pw) throws IOException{
	pw.println("<ResultTableStructure>");
	pw.println("<columntitles>");
	columnTitles.printPartXML(pw);
	pw.println("</columntitles>");
	pw.println("<rowtitles>");
	rowTitles.printPartXML(pw);
	pw.println("</rowtitles>");

	/*
	pw.println("<paths>");
	paths.printPartXML(pw);
	pw.println("</paths>");
	*/
	
	pathTable.printPartXML(pw);

	pw.println("</ResultTableStructure>");
    }


    /**
     * Prints a basic XSL-file for ResultTableStructure XML-files to a 
     * {@link java.io.PrintWriter}.
     */
    public void printXSL(PrintWriter pw) throws IOException{
	pw.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
	pw.println("<xsl:stylesheet version=\"1.0\"");
	pw.println("xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\">");
	pw.println("<xsl:template match=\"/\">");
	pw.println("<html>");
	pw.println("<body>");

	pw.println("<xsl:apply-templates/>");

	pw.println("</body>");
	pw.println("</html>");
	pw.println("</xsl:template>");
	pw.println("");

	pw.println("<xsl:template match=\"commonpath\">");
	pw.print("<h3>Common path</h3> <xsl:value-of select=\".\"/>");
	pw.println("</xsl:template>");
	pw.println("");

	pw.println("<xsl:template match=\"columntitles\">");
	pw.println("<h3>Column Titles</h3>");
	pw.println("<xsl:apply-templates select=\"table\"/>");
	pw.println("</xsl:template>");
	pw.println("");

	pw.println("<xsl:template match=\"rowtitles\">");
	pw.print("<h3>Row Titles</h3>");
	pw.print("<xsl:apply-templates select=\"table\"/>");
	pw.println("</xsl:template>");
	pw.println("");

	pw.println("<xsl:template match=\"paths\">");
	pw.print("<h3>Paths</h3>");
	pw.print("<xsl:apply-templates select=\"table\"/>");
	pw.println("</xsl:template>");
	pw.println("");

	pw.println("<xsl:template match=\"table\">");
	pw.println("<table border=\"1\">");
	pw.println("<xsl:for-each select=\"r\">");
	pw.println("<tr>");
	pw.println("<xsl:for-each select=\"c\">");
	pw.print("<td><xsl:value-of select=\".\"/></td>");
	pw.println("\n</xsl:for-each>");
	pw.println("</tr>");
	pw.println("</xsl:for-each>");
	pw.println("</table>");
	pw.println("</xsl:template>");
	pw.println("");

	pw.println("</xsl:stylesheet>");

    }

    /**
     * Loads a ResultTableStructure-object from a file with the specified file name.
     */
    public static ResultTableStructure load(String fileName)
	throws IOException, SAXException{
	return load(new File(fileName));
    }


    /**
     * Loads a ResultTableStructure-object from the specified file.
     */
    public static ResultTableStructure load(File file)
	throws IOException, SAXException{
	ResultTableStructure table = null;
	BufferedReader in = new BufferedReader(new FileReader(file));
	InputSource source = new InputSource(in);

	    //	    ResultTableStructure iop = new ResultTableStructure("Dummy");
	table=loadXML(source);
	    /*
	catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
	    */
	return table;
    }



    /**
     * Loads a ResultTableStructure-object from an {@link java.io.InputSource}.
     */
    public static ResultTableStructure loadXML(InputSource source) 
	throws IOException, SAXException{
	ResultTableStructure table = null;
	//	try {
	    // Create parser+contentHandler and read the XML document
	    XMLReader parser = XMLReaderFactory.createXMLReader();
	    ResultTableStructureHandler handler = new ResultTableStructureHandler();
	    parser.setContentHandler(handler);
	    parser.parse(source);
	    table = handler.getResultTableStructure();
	    /*
	} catch (Exception e) {
	    System.err.println(e);
	    e.printStackTrace();
	}
	    */
	return table;
    }

}

/**
 * Class used for XML-parsing of {@link ResultTableStructure}-objects.
 */
class ResultTableStructureHandler extends DefaultHandler {

    private StringBuffer buffer;

    private ResultTableStructure resultTableStructure = 
	new ResultTableStructure();
    
    private int currentTableKind = -1; 
    // 0 - MeasureAbbreviationTable
    // 1 - StringTable (rowtitles, columntitles)
    // 2 - PathTable
    // 3 - RelativeTable
    // 4 - for Caption

    /*
    private boolean nowTable=false;
    private boolean nowPathTable=false;
    private boolean nowRelativeTable=false;
    */

    private Table<String> currentTable = null;
    private PathTable pathTable = null;
    private PathEntry pathEntry = null;
    private Vector<String> paths = null;
    private RelativeTable relativeTable = null;
    private RelativeEntry rele = null;
    private MeasureAbbreviationTable measureAbbreviationTable = null;
    private String abbreviation = null;
    private Vector<String> measureNames = null;

    private int row = -1;
    private int col = -1;

    public ResultTableStructure getResultTableStructure(){
	return resultTableStructure;
    }


    public void startElement (String namespaceURI, String localName,
			      String qualifiedName, Attributes atts) {
	buffer = new StringBuffer();

	if(qualifiedName.equals("measureabbreviationtable")){
	    currentTableKind = 0;
	    measureAbbreviationTable = new MeasureAbbreviationTable();
	}else if(qualifiedName.equals("mae")){          
	    measureNames = new Vector<String>();	    
	}else if(qualifiedName.equals("table")){
	    currentTableKind = 1;
	    /*
	    nowTable=true;
	    nowPathTable=false;
	    nowRelativeTable=false;
	    */

	    currentTable = new Table<String>();
	    row=-1;
	}else if(qualifiedName.equals("r")){          
	    row++;
	    col=-1;
	}else if(qualifiedName.equals("c")){
	    col++;
	}

	else if(qualifiedName.equals("pathtable")){
	    currentTableKind = 2;
	    /*
	    nowTable=false;
	    nowPathTable=true;
	    nowRelativeTable=false;
	    */

	    pathTable = new PathTable();
	    row=-1;
	}else if(qualifiedName.equals("ptr")){          
	    row++;
	    col=-1;
	}else if(qualifiedName.equals("ptc")){
	    col++;
	    paths = new Vector<String>();
	    pathEntry = new PathEntry();
	}

	else if(qualifiedName.equals("relativetable")){
	    currentTableKind = 3;
	    /*
	    nowTable=false;
	    nowPathTable=false;
	    nowRelativeTable=true;
	    */

	    relativeTable = new RelativeTable();
	}else if(qualifiedName.equals("rele")){
	    rele = new RelativeEntry();
	}
	else if(qualifiedName.equals("caption")){
	    currentTableKind = 4;
	}


    }


    public void endElement (String namespaceURI, String localName, String qualifiedName) {
	// private int currentTableKind = -1; 
	// 0 - MeasureAbbreviationTable
	// 1 - StringTable (rowtitles, columntitles)
	// 2 - PathTable
	// 3 - RelativeTable

	if(currentTableKind==0){
	    if(qualifiedName.equals("abb")){
		abbreviation = buffer.toString();
	    }else if(qualifiedName.equals("n")){
		measureNames.add(buffer.toString());
	    }else if(qualifiedName.equals("mae")){
		measureAbbreviationTable.add(abbreviation, measureNames);
	    }else if(qualifiedName.equals("measureabbreviationtable")){
		resultTableStructure.setMeasureAbbreviationTable(measureAbbreviationTable);
	    }
	}else if(currentTableKind==1){
	    if(qualifiedName.equals("c")){
		currentTable.put(row, col, buffer.toString());
	    }else if(qualifiedName.equals("commonpath")){
		resultTableStructure.setCommonPath(buffer.toString());
	    }else if(qualifiedName.equals("columntitles")){
		resultTableStructure.setColumnTitles(currentTable);
	    }else if(qualifiedName.equals("rowtitles")){
		resultTableStructure.setRowTitles(currentTable);
	    }
	}else if(currentTableKind==2){
	    if(qualifiedName.equals("commonpath")){
		pathTable.setCommonPath(buffer.toString());
	    }else if(qualifiedName.equals("m")){
		pathEntry.setMeasureAbbreviation(buffer.toString());
	    }else if(qualifiedName.equals("rd")){
		pathEntry.setRoundString(buffer.toString());
	    }else if(qualifiedName.equals("p")){
		pathEntry.putPath(buffer.toString());
	    }else if(qualifiedName.equals("ptc")){
		pathTable.put(row, col, pathEntry);
	    }else if(qualifiedName.equals("pathtable")){
		resultTableStructure.setPaths(pathTable);
	    }
	}else if(currentTableKind==3){
	    if(qualifiedName.equals("dim")){
		rele.setDimension(buffer.toString());
	    }else if(qualifiedName.equals("nr")){
		rele.setNumber(buffer.toString());
	    }else if(qualifiedName.equals("relnr")){
		rele.setRelativeNumber(buffer.toString());
	    }else if(qualifiedName.equals("place")){
		rele.setPlace(buffer.toString());
	    }else if(qualifiedName.equals("rele")){
		relativeTable.add(rele);
	    }else if(qualifiedName.equals("relativetable")){
		resultTableStructure.setRelativeTable(relativeTable);
	    }
	}else if(currentTableKind==4){
	    if(qualifiedName.equals("caption")){
		resultTableStructure.setCaption(buffer.toString());
	    }
	}
    }

    public void characters (char ch[], int start, int length) {
	if(buffer != null)
	    buffer.append(ch,start,length);
    }      
}

